@@ -50,3 +50,5 @@ $ #MaxOverSlit
 #   Slit -
 $ #SloppyWindowGrouping
 # Sloppy   
+$ #WorkspaceWarping
+# Workspace Warping
