@@ -50,3 +50,5 @@ $ #MaxOverSlit
 # Maximisez au dessus du Slit
 $ #SloppyWindowGrouping
 # Groupement des fentres stupide
+$ #WorkspaceWarping
+# Workspace Warping
