@@ -50,3 +50,5 @@ $ #MaxOverSlit
 # Slitۤƺ粽
 $ #SloppyWindowGrouping
 # Window˥֥ɥåפǤ⥿֥롼פ
+$ #WorkspaceWarping
+# Workspace Warping
