@@ -50,3 +50,5 @@ $ #MaxOverSlit
 # Maximizar Sobre Slit
 $ #SloppyWindowGrouping
 # Sloppy Janela Em Grupo
+$ #WorkspaceWarping
+# Workspace Warping
