@@ -50,3 +50,5 @@ $ #MaxOverSlit
 #   
 $ #SloppyWindowGrouping
 #  "" 
+$ #WorkspaceWarping
+# Workspace Warping
