@@ -50,3 +50,5 @@ $ #MaxOverSlit
 # Maximize Over Slit
 $ #SloppyWindowGrouping
 # Hafsig Fnster Gruppering
+$ #WorkspaceWarping
+# Workspace Warping
