@@ -50,3 +50,5 @@ $ #MaxOverSlit
 # Maximize Over Slit
 $ #SloppyWindowGrouping
 # Sloppy Window Grouping
+$ #WorkspaceWarping
+# Workspace Warping
