@@ -22,9 +22,9 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: BaseDisplay.cc,v 1.6 2002/02/11 10:57:23 fluxgen Exp $
+// $Id: BaseDisplay.cc,v 1.13 2002/04/08 22:23:41 fluxgen Exp $
 
-// use some GNU extensions
+// use GNU extensions
 #ifndef	 _GNU_SOURCE
 #define	 _GNU_SOURCE
 #endif // _GNU_SOURCE
@@ -35,6 +35,7 @@
 
 #include "BaseDisplay.hh"
 #include "i18n.hh"
+#include "Timer.hh"
 
 #include <X11/Xutil.h>
 #include <X11/cursorfont.h>
@@ -43,31 +44,30 @@
 #	include <X11/extensions/shape.h>
 #endif // SHAPE
 
-
-#ifdef		HAVE_FCNTL_H
-#	include <fcntl.h>
+#ifdef HAVE_FCNTL_H
+#include <fcntl.h>
 #endif // HAVE_FCNTL_H
 
-#ifdef		HAVE_STDIO_H
-#	include <stdio.h>
+#ifdef	HAVE_STDIO_H
+#include <stdio.h>
 #endif // HAVE_STDIO_H
 
-#ifdef		STDC_HEADERS
-#	include <stdlib.h>
-#	include <string.h>
+#ifdef	STDC_HEADERS
+#include <stdlib.h>
+#include <string.h>
 #endif // STDC_HEADERS
 
-#ifdef		HAVE_UNISTD_H
-#	include <sys/types.h>
-#	include <unistd.h>
+#ifdef	HAVE_UNISTD_H
+#include <sys/types.h>
+#include <unistd.h>
 #endif // HAVE_UNISTD_H
 
-#ifdef		HAVE_SYS_SELECT_H
-#	include <sys/select.h>
+#ifdef	HAVE_SYS_SELECT_H
+#include <sys/select.h>
 #endif // HAVE_SYS_SELECT_H
 
-#ifdef		HAVE_SIGNAL_H
-#	include <signal.h>
+#ifdef	HAVE_SIGNAL_H
+#include <signal.h>
 #endif // HAVE_SIGNAL_H
 
 #ifndef	 SA_NODEFER
@@ -106,11 +106,7 @@ static int handleXErrors(Display *d, XErrorEvent *e) {
 	fprintf(stderr,
 		I18n::instance()->
 		getMessage(
-		#ifdef	NLS
-			BaseDisplaySet, BaseDisplayXError,
-		#else // !NLS
-			0, 0,
-		#endif // NLS
+			FBNLS::BaseDisplaySet, FBNLS::BaseDisplayXError,
 			 "%s:	X error: %s(%d) opcodes %d/%d\n	resource 0x%lx\n"),
 			base_display->getApplicationName(), errtxt, e->error_code,
 			e->request_code, e->minor_code, e->resourceid);
@@ -164,11 +160,7 @@ static void signalhandler(int sig) {
 
 		fprintf(stderr,
 			i18n->getMessage(
-			#ifdef NLS
-				 BaseDisplaySet, BaseDisplaySignalCaught,
-			#else // !NLS
-				 0, 0,
-			#endif // NLS
+				 FBNLS::BaseDisplaySet, FBNLS::BaseDisplaySignalCaught,
 				 "%s:	signal %d caught\n"),
 			base_display->getApplicationName(), sig);
 
@@ -178,23 +170,15 @@ static void signalhandler(int sig) {
 			re_enter = 1;
 			fprintf(stderr,
 				i18n->getMessage(
-			#ifdef NLS
-					BaseDisplaySet, BaseDisplayShuttingDown,
-			#else // !NLS
-				 0, 0,
-			#endif // NLS
-				 "shutting down\n"));
+					FBNLS::BaseDisplaySet, FBNLS::BaseDisplayShuttingDown,
+					"shutting down\n"));
 			base_display->shutdown();
 		}
 
 		if (sig != SIGTERM && sig != SIGINT) {
 			fprintf(stderr,
 				i18n->getMessage(
-			#ifdef NLS
-				 BaseDisplaySet, BaseDisplayAborting,
-			#else // !NLS
-				 0, 0,
-			#endif // NLS
+				 FBNLS::BaseDisplaySet, FBNLS::BaseDisplayAborting,
 				 "aborting... dumping core\n"));
 			abort();
 		}
@@ -219,7 +203,7 @@ void bexec(const char *command, char* displaystring) {
 #endif // !__EMX__
 
 
-BaseDisplay::BaseDisplay(char *app_name, char *dpy_name):
+BaseDisplay::BaseDisplay(char *app_name, char *dpy_name):FbAtoms(0),
 m_startup(true), m_shutdown(false), 
 m_display_name(XDisplayName(dpy_name)), m_app_name(app_name),
 m_server_grabs(0)
@@ -259,27 +243,21 @@ m_server_grabs(0)
 		fprintf(stderr,
 			i18n->
 			getMessage(
-		#ifdef NLS
-			BaseDisplaySet, BaseDisplayXConnectFail,
-		#else // !NLS
-				0, 0,
-		#endif // NLS
-		"BaseDisplay::BaseDisplay: connection to X server failed.\n"));
+				FBNLS::BaseDisplaySet, FBNLS::BaseDisplayXConnectFail,
+				"BaseDisplay::BaseDisplay: connection to X server failed.\n"));
 		
 		throw static_cast<int>(2); //throw error 2
 	} else if (fcntl(ConnectionNumber(m_display), F_SETFD, 1) == -1) {
 		fprintf(stderr,
 			i18n->
 				getMessage(
-		#ifdef NLS
-				BaseDisplaySet, BaseDisplayCloseOnExecFail,
-		#else // !NLS
-				0, 0,
-		#endif // NLS
-				"BaseDisplay::BaseDisplay: couldn't mark display connection "
-				"as close-on-exec\n"));
+					FBNLS::BaseDisplaySet, FBNLS::BaseDisplayCloseOnExecFail,
+					"BaseDisplay::BaseDisplay: couldn't mark display connection "
+					"as close-on-exec\n"));
 		throw static_cast<int>(2); //throw error 2
 	}
+	//initiate atoms
+	initAtoms(m_display);
 
 	number_of_screens = ScreenCount(m_display);
 
@@ -289,97 +267,6 @@ m_server_grabs(0)
 #else // !SHAPE
 	shape.extensions = False;
 #endif // SHAPE
-//---------- setup atoms
-
-	xa_wm_colormap_windows =
-		XInternAtom(m_display, "WM_COLORMAP_WINDOWS", False);
-	xa_wm_protocols = XInternAtom(m_display, "WM_PROTOCOLS", False);
-	xa_wm_state = XInternAtom(m_display, "WM_STATE", False);
-	xa_wm_change_state = XInternAtom(m_display, "WM_CHANGE_STATE", False);
-	xa_wm_delete_window = XInternAtom(m_display, "WM_DELETE_WINDOW", False);
-	xa_wm_take_focus = XInternAtom(m_display, "WM_TAKE_FOCUS", False);
-	motif_wm_hints = XInternAtom(m_display, "_MOTIF_WM_HINTS", False);
-
-	blackbox_hints = XInternAtom(m_display, "_BLACKBOX_HINTS", False);
-	blackbox_attributes = XInternAtom(m_display, "_BLACKBOX_ATTRIBUTES", False);
-	blackbox_change_attributes =
-		XInternAtom(m_display, "_BLACKBOX_CHANGE_ATTRIBUTES", False);
-
-	blackbox_structure_messages =
-		XInternAtom(m_display, "_BLACKBOX_STRUCTURE_MESSAGES", False);
-	blackbox_notify_startup =
-		XInternAtom(m_display, "_BLACKBOX_NOTIFY_STARTUP", False);
-	blackbox_notify_window_add =
-		XInternAtom(m_display, "_BLACKBOX_NOTIFY_WINDOW_ADD", False);
-	blackbox_notify_window_del =
-		XInternAtom(m_display, "_BLACKBOX_NOTIFY_WINDOW_DEL", False);
-	blackbox_notify_current_workspace =
-		XInternAtom(m_display, "_BLACKBOX_NOTIFY_CURRENT_WORKSPACE", False);
-	blackbox_notify_workspace_count =
-		XInternAtom(m_display, "_BLACKBOX_NOTIFY_WORKSPACE_COUNT", False);
-	blackbox_notify_window_focus =
-		XInternAtom(m_display, "_BLACKBOX_NOTIFY_WINDOW_FOCUS", False);
-	blackbox_notify_window_raise =
-		XInternAtom(m_display, "_BLACKBOX_NOTIFY_WINDOW_RAISE", False);
-	blackbox_notify_window_lower =
-		XInternAtom(m_display, "_BLACKBOX_NOTIFY_WINDOW_LOWER", False);
-
-	blackbox_change_workspace =
-		XInternAtom(m_display, "_BLACKBOX_CHANGE_WORKSPACE", False);
-	blackbox_change_window_focus =
-		XInternAtom(m_display, "_BLACKBOX_CHANGE_WINDOW_FOCUS", False);
-	blackbox_cycle_window_focus =
-		XInternAtom(m_display, "_BLACKBOX_CYCLE_WINDOW_FOCUS", False);
-
-#ifdef NEWWMSPEC
-
-	net_supported = XInternAtom(m_display, "_NET_SUPPORTED", False);
-	net_client_list = XInternAtom(m_display, "_NET_CLIENT_LIST", False);
-	net_client_list_stacking = XInternAtom(m_display, "_NET_CLIENT_LIST_STACKING", False);
-	net_number_of_desktops = XInternAtom(m_display, "_NET_NUMBER_OF_DESKTOPS", False);
-	net_desktop_geometry = XInternAtom(m_display, "_NET_DESKTOP_GEOMETRY", False);
-	net_desktop_viewport = XInternAtom(m_display, "_NET_DESKTOP_VIEWPORT", False);
-	net_current_desktop = XInternAtom(m_display, "_NET_CURRENT_DESKTOP", False);
-	net_desktop_names = XInternAtom(m_display, "_NET_DESKTOP_NAMES", False);
-	net_active_window = XInternAtom(m_display, "_NET_ACTIVE_WINDOW", False);
-	net_workarea = XInternAtom(m_display, "_NET_WORKAREA", False);
-	net_supporting_wm_check = XInternAtom(m_display, "_NET_SUPPORTING_WM_CHECK", False);
-	net_virtual_roots = XInternAtom(m_display, "_NET_VIRTUAL_ROOTS", False);
-
-	net_close_window = XInternAtom(m_display, "_NET_CLOSE_WINDOW", False);
-	net_wm_moveresize = XInternAtom(m_display, "_NET_WM_MOVERESIZE", False);
-
-	net_properties = XInternAtom(m_display, "_NET_PROPERTIES", False);
-	net_wm_name = XInternAtom(m_display, "_NET_WM_NAME", False);
-	net_wm_desktop = XInternAtom(m_display, "_NET_WM_DESKTOP", False);
-	net_wm_window_type = XInternAtom(m_display, "_NET_WM_WINDOW_TYPE", False);
-	net_wm_state = XInternAtom(m_display, "_NET_WM_STATE", False);
-	net_wm_strut = XInternAtom(m_display, "_NET_WM_STRUT", False);
-	net_wm_icon_geometry = XInternAtom(m_display, "_NET_WM_ICON_GEOMETRY", False);
-	net_wm_icon = XInternAtom(m_display, "_NET_WM_ICON", False);
-	net_wm_pid = XInternAtom(m_display, "_NET_WM_PID", False);
-	net_wm_handled_icons = XInternAtom(m_display, "_NET_WM_HANDLED_ICONS", False);
-
-	net_wm_ping = XInternAtom(m_display, "_NET_WM_PING", False);
-	
-#endif // NEWWMSPEC
-
-#ifdef GNOME
-	
-	gnome_wm_win_layer = XInternAtom(m_display, "_WIN_LAYER", False);
-	gnome_wm_win_state = XInternAtom(m_display, "_WIN_STATE", False);
-	gnome_wm_win_hints = XInternAtom(m_display, "_WIN_HINTS", False);
-	gnome_wm_win_app_state = XInternAtom(m_display, "_WIN_APP_STATE", False);
-	gnome_wm_win_expanded_size = XInternAtom(m_display, "_WIN_EXPANDED_SIZE", False);
-	gnome_wm_win_icons = XInternAtom(m_display, "_WIN_ICONS", False);
-	gnome_wm_win_workspace = XInternAtom(m_display, "_WIN_WORKSPACE", False);
-	gnome_wm_win_workspace_count = XInternAtom(m_display, "_WIN_WORKSPACE_COUNT", False);
-	gnome_wm_win_workspace_names = XInternAtom(m_display, "_WIN_WORKSPACE_NAMES", False);
-	gnome_wm_win_client_list = XInternAtom(m_display, "_WIN_CLIENT_LIST", False);
-	gnome_wm_prot = XInternAtom(m_display, "_WIN_PROTOCOLS", False);
-	gnome_wm_supporting_wm_check = XInternAtom(m_display, "_WIN_SUPPORTING_WM_CHECK", False);
-	
-#endif // GNOME
 
 	cursor.session = XCreateFontCursor(m_display, XC_left_ptr);
 	cursor.move = XCreateFontCursor(m_display, XC_fleur);
@@ -410,8 +297,6 @@ BaseDisplay::~BaseDisplay(void) {
 void BaseDisplay::eventLoop(void) {
 	run();
 
-	int xfd = ConnectionNumber(m_display);
-
 	while ((! m_shutdown) && (! internal_error)) {
 		if (XPending(m_display)) {
 			XEvent e;
@@ -422,11 +307,7 @@ void BaseDisplay::eventLoop(void) {
 			fprintf(stderr,
 				I18n::instance()->
 					getMessage(
-			#ifdef NLS
-					BaseDisplaySet, BaseDisplayBadWindowRemove,
-			#else // !NLS
-					0, 0,
-			#endif // NLS
+					FBNLS::BaseDisplaySet, FBNLS::BaseDisplayBadWindowRemove,
 					"BaseDisplay::eventLoop(): removing bad window "
 					"from event queue\n"));
 			#endif // DEBUG
@@ -435,75 +316,7 @@ void BaseDisplay::eventLoop(void) {
 				process_event(&e);
 			}
 		} else {
-			fd_set rfds;
-			timeval now, tm, *timeout = (timeval *) 0;
-
-			FD_ZERO(&rfds);
-			FD_SET(xfd, &rfds);
-
-			if (timerList.size() > 0) {
-				gettimeofday(&now, 0);
-
-				tm.tv_sec = tm.tv_usec = 0l;
-
-				BTimer *timer = timerList.front();
-
-				tm.tv_sec = timer->getStartTime().tv_sec +
-					timer->getTimeout().tv_sec - now.tv_sec;
-				tm.tv_usec = timer->getStartTime().tv_usec +
-					timer->getTimeout().tv_usec - now.tv_usec;
-
-				while (tm.tv_usec >= 1000000) {
-					tm.tv_sec++;
-					tm.tv_usec -= 1000000;
-				}
-
-				while (tm.tv_usec < 0) {
-					if (tm.tv_sec > 0) {
-						tm.tv_sec--;
-						tm.tv_usec += 1000000;
-					} else {
-						tm.tv_usec = 0;
-						break;
-					}
-				}
-
-				timeout = &tm;
-			}
-
-			select(xfd + 1, &rfds, 0, 0, timeout);
-
-			// check for timer timeout
-			gettimeofday(&now, 0);
-
-			TimerList::iterator it = timerList.begin();
-			//must check end ...the timer might remove
-			//it self from the list (should be fixed in the future)
-			for(; it != timerList.end(); ++it) {
-	
-				tm.tv_sec = (*it)->getStartTime().tv_sec +
-					(*it)->getTimeout().tv_sec;
-				tm.tv_usec = (*it)->getStartTime().tv_usec +
-					(*it)->getTimeout().tv_usec;
-
-				if ((now.tv_sec < tm.tv_sec) ||
-						(now.tv_sec == tm.tv_sec && now.tv_usec < tm.tv_usec))
-					break;
-
-				(*it)->fireTimeout();
-
-				// restart the current timer so that the start time is updated
-				if (! (*it)->doOnce())
-					(*it)->start();
-				else { 
-					(*it)->stop();
-					// must do this because the stupid cyclic dep 
-					// between BaseDisplay and BTimer, the timer removes
-					// it self from the list
-					it--;
-				}
-					
-			}
+			BTimer::updateTimers(ConnectionNumber(m_display)); //handle all timers
 		}
 	}
 }
@@ -534,26 +347,7 @@ void BaseDisplay::ungrab(void) {
 }
 
 
-void BaseDisplay::addTimer(BTimer *timer) {
-	assert(timer);
 
-	TimerList::iterator it = timerList.begin();
-	TimerList::iterator it_end = timerList.end();
-	int index = 0;
-	for (; it != it_end; ++it, ++index) {
-		if (((*it)->getTimeout().tv_sec > timer->getTimeout().tv_sec) ||
-				(((*it)->getTimeout().tv_sec == timer->getTimeout().tv_sec) &&
-				((*it)->getTimeout().tv_usec >= timer->getTimeout().tv_usec))) {
-			break;
-		}
-	}
-	timerList.insert(it, timer);
-}
-
-
-void BaseDisplay::removeTimer(BTimer *timer) {
-	timerList.remove(timer);
-}
 
 
 ScreenInfo::ScreenInfo(BaseDisplay *d, int num) {
@@ -600,4 +394,161 @@ ScreenInfo::ScreenInfo(BaseDisplay *d, int num) {
 		visual = DefaultVisual(basedisplay->getXDisplay(), screen_number);
 		colormap = DefaultColormap(basedisplay->getXDisplay(), screen_number);
 	}
+
+#ifdef XINERAMA
+	// check if we have Xinerama extension enabled
+	if (XineramaIsActive(basedisplay->getXDisplay())) {
+		m_hasXinerama = true;
+		xineramaLastHead = 0;
+		xineramaInfos =
+			XineramaQueryScreens(basedisplay->getXDisplay(), &xineramaNumHeads);
+	} else {
+		m_hasXinerama = false;
+		xineramaInfos = 0; // make sure we don't point anywhere we shouldn't
+	}
+#endif // XINERAMA 
 }
+
+ScreenInfo::~ScreenInfo(void) {
+#ifdef XINERAMA
+	if (m_hasXinerama) { // only free if we first had it
+		XFree(xineramaInfos);
+		xineramaInfos = 0;
+	}
+#endif // XINERAMA
+}
+
+#ifdef XINERAMA
+
+//---------------- getHead ---------------
+// Searches for the head at the coordinates
+// x,y. If it fails or Xinerama isn't
+// activated it'll return head nr 0
+//-----------------------------------------
+unsigned int ScreenInfo::getHead(int x, int y) const {
+
+	// is Xinerama extensions enabled?
+	if (hasXinerama()) {
+		// check if last head is still active
+/*		if ((xineramaInfos[xineramaLastHead].x_org <= x) &&
+				((xineramaInfos[xineramaLastHead].x_org +
+				xineramaInfos[xineramaLastHead].width) > x) &&
+				(xineramaInfos[xineramaLastHead].y_org <= y) &&
+				((xineramaInfos[xineramaLastHead].y_org +
+				xineramaInfos[xineramaLastHead].height) > y)) {
+			return xineramaLastHead;
+		} else { */
+			// go trough all the heads, and search
+			for (int i = 0; (signed) i < xineramaNumHeads; i++) {
+				if (xineramaInfos[i].x_org <= x &&
+						(xineramaInfos[i].x_org + xineramaInfos[i].width) > x &&
+						xineramaInfos[i].y_org <= y &&
+						(xineramaInfos[i].y_org + xineramaInfos[i].height) > y)
+					// return (xineramaLastHead = i);
+					return i;
+			}
+	//	}
+	}
+
+	return 0;
+}
+
+//------------- getCurrHead --------------
+// Searches for the head that the pointer
+// currently is on, if it isn't found
+// the first one is returned
+//----------------------------------------
+unsigned int ScreenInfo::getCurrHead(void) const {
+
+	// is Xinerama extensions enabled?
+	if (hasXinerama()) {
+		int x, y, wX, wY;
+		unsigned int mask;
+		Window rRoot, rChild;
+		// get pointer cordinates
+		if ( (XQueryPointer(basedisplay->getXDisplay(), root_window,
+				&rRoot, &rChild, &x, &y, &wX, &wY, &mask)) != 0 )  {			
+			return getHead(x, y);
+		}
+	}
+
+	return 0;
+}
+
+//----------- getHeadWidth ------------
+// Returns the width of head
+//-------------------------------------
+unsigned int ScreenInfo::getHeadWidth(unsigned int head) const {
+
+	if (hasXinerama()) {
+		if ((signed) head >= xineramaNumHeads) {
+			#ifdef DEBUG
+			cerr << __FILE__ << ":" <<__LINE__ << ": " <<
+				"Head: " << head << " doesn't exist!" << endl;
+			#endif // DEBUG
+			return xineramaInfos[xineramaNumHeads - 1].width;
+		} else
+			return xineramaInfos[head].width;
+	}
+	
+	return getWidth();
+
+}
+
+//----------- getHeadHeight ------------
+// Returns the heigt of head
+//--------------------------------------
+unsigned int ScreenInfo::getHeadHeight(unsigned int head) const {
+
+	if (hasXinerama()) {
+		if ((signed) head >= xineramaNumHeads) {
+			#ifdef DEBUG
+			cerr << __FILE__ << ":" <<__LINE__ << ": " <<
+				"Head: " << head << " doesn't exist!" << endl;
+			#endif // DEBUG
+			return xineramaInfos[xineramaNumHeads - 1].height;
+		} else
+			return xineramaInfos[head].height;
+	}
+
+	return getHeight();
+}
+
+
+//----------- getHeadX -----------------
+// Returns the X start of head nr head
+//--------------------------------------
+int ScreenInfo::getHeadX(unsigned int head) const {
+	if (hasXinerama()) {
+		if ((signed) head >= xineramaNumHeads) {
+			#ifdef DEBUG
+			cerr << __FILE__ << ":" <<__LINE__ << ": " <<
+				"Head: " << head << " doesn't exist!" << endl;
+			#endif // DEBUG
+			return xineramaInfos[head = xineramaNumHeads - 1].x_org;
+		} else 
+			return xineramaInfos[head].x_org;
+	}
+
+	return 0;
+}
+
+//----------- getHeadY -----------------
+// Returns the Y start of head
+//--------------------------------------
+int ScreenInfo::getHeadY(unsigned int head) const {
+	if (hasXinerama()) {
+		if ((signed) head >= xineramaNumHeads) {
+			#ifdef DEBUG
+			cerr << __FILE__ << ":" <<__LINE__ << ": " <<
+				"Head: " << head << " doesn't exist!" << endl;
+			#endif // DEBUG
+			return xineramaInfos[xineramaNumHeads - 1].y_org;
+		} else 
+			return xineramaInfos[head].y_org;
+	}
+
+	return 0;
+}
+
+#endif // XINERAMA
