@@ -22,10 +22,9 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: Basemenu.cc,v 1.10 2002/02/08 13:47:11 fluxgen Exp $
+// $Id: Basemenu.cc,v 1.21 2002/04/09 12:22:06 fluxgen Exp $
 
-// stupid macros needed to access some functions in version 2 of the GNU C
-// library
+//use GNU extensions
 #ifndef	 _GNU_SOURCE
 #define	 _GNU_SOURCE
 #endif // _GNU_SOURCE
@@ -49,27 +48,30 @@
 #	include <string.h>
 #endif // STDC_HEADERS
 
+#ifdef DEBUG
 #include <iostream>
+using namespace std;
+#endif //DEBUG
 
 static Basemenu *shown = (Basemenu *) 0;
 
-Basemenu::Basemenu(BScreen *scrn) {
-	screen = scrn;
-	fluxbox = Fluxbox::instance();
-	image_ctrl = screen->getImageControl();
-	display = fluxbox->getXDisplay();
-	parent = (Basemenu *) 0;
-	alignment = ALIGNDONTCARE;
+Basemenu::Basemenu(BScreen *screen):
+m_fluxbox(Fluxbox::instance()),
+m_screen(screen),
+m_display(m_fluxbox->getXDisplay()),
+m_parent(0),
+m_image_ctrl(m_screen->getImageControl()),
+m_alignment(ALIGNDONTCARE) {
 
 	title_vis =
 		movable =
-		hide_tree = True;
+		hide_tree = true;
 
 	shifted =
 		internal_menu =
 		moving =
 		torn =
-		visible = False;
+		visible = false;
 
 	menu.x =
 		menu.y =
@@ -87,35 +89,34 @@ Basemenu::Basemenu(BScreen *scrn) {
 		menu.hilite_pixmap =
 		menu.sel_pixmap = None;
 
-	menu.bevel_w = screen->getBevelWidth();
+	menu.bevel_w = m_screen->getBevelWidth();
 
 	I18n *i18n = I18n::instance();
 	
 	if (i18n->multibyte()) {
 		menu.width = menu.title_h = menu.item_w = menu.frame_h =
-			screen->getMenuStyle()->titlefont.set_extents->max_ink_extent.height +
+			m_screen->getMenuStyle()->titlefont->getFontSetExtents()->max_ink_extent.height +
 			(menu.bevel_w	* 2);
 	} else {
 		menu.width = menu.title_h = menu.item_w = menu.frame_h =
-			screen->getMenuStyle()->titlefont.fontstruct->ascent +
-			screen->getMenuStyle()->titlefont.fontstruct->descent +
+			m_screen->getMenuStyle()->titlefont->getFontStruct()->ascent +
+			m_screen->getMenuStyle()->titlefont->getFontStruct()->descent +
 			(menu.bevel_w * 2);
 	}
-	menu.label = 0;
 	
 	menu.sublevels =
 		menu.persub =
 		menu.minsub = 0;
 	
 	if (i18n->multibyte()) {
-		menu.item_h = screen->getMenuStyle()->framefont.set_extents->max_ink_extent.height +
+		menu.item_h = m_screen->getMenuStyle()->framefont->getFontSetExtents()->max_ink_extent.height +
 			(menu.bevel_w);
 	} else {
-		menu.item_h = screen->getMenuStyle()->framefont.fontstruct->ascent +
-			screen->getMenuStyle()->framefont.fontstruct->descent +
+		menu.item_h = m_screen->getMenuStyle()->framefont->getFontStruct()->ascent +
+			m_screen->getMenuStyle()->framefont->getFontStruct()->descent +
 			(menu.bevel_w);
 	}
-	menu.height = menu.title_h + screen->getBorderWidth() + menu.frame_h;
+	menu.height = menu.title_h + m_screen->getBorderWidth() + menu.frame_h;
 	
 	//set attributes for menu window
 	unsigned long attrib_mask = CWBackPixmap | CWBackPixel | CWBorderPixel |
@@ -123,77 +124,73 @@ Basemenu::Basemenu(BScreen *scrn) {
 	XSetWindowAttributes attrib;
 	attrib.background_pixmap = None;
 	attrib.background_pixel = attrib.border_pixel =
-		screen->getBorderColor()->getPixel();
-	attrib.colormap = screen->getColormap();
+		m_screen->getBorderColor()->getPixel();
+	attrib.colormap = m_screen->getColormap();
 	attrib.override_redirect = True;
 	attrib.event_mask = ButtonPressMask | ButtonReleaseMask |
 		ButtonMotionMask | ExposureMask;
 
 	//create menu window
 	menu.window =
-		XCreateWindow(display, screen->getRootWindow(), menu.x, menu.y, menu.width,
-			menu.height, screen->getBorderWidth(), screen->getDepth(),
-			InputOutput, screen->getVisual(), attrib_mask, &attrib);
+		XCreateWindow(m_display, m_screen->getRootWindow(), menu.x, menu.y, menu.width,
+			menu.height, m_screen->getBorderWidth(), m_screen->getDepth(),
+			InputOutput, m_screen->getVisual(), attrib_mask, &attrib);
 	
-	fluxbox->saveMenuSearch(menu.window, this);
+	m_fluxbox->saveMenuSearch(menu.window, this);
 	
 	//attibutes for title to menuwindow
 	attrib_mask = CWBackPixmap | CWBackPixel | CWBorderPixel | CWEventMask;
-	attrib.background_pixel = screen->getBorderColor()->getPixel();
+	attrib.background_pixel = m_screen->getBorderColor()->getPixel();
 	attrib.event_mask |= EnterWindowMask | LeaveWindowMask;
 	//create menu title
 	menu.title =
-		XCreateWindow(display, menu.window, 0, 0, menu.width, menu.height, 0,
-			screen->getDepth(), InputOutput, screen->getVisual(),
+		XCreateWindow(m_display, menu.window, 0, 0, menu.width, menu.height, 0,
+			m_screen->getDepth(), InputOutput, m_screen->getVisual(),
 			attrib_mask, &attrib);
-	fluxbox->saveMenuSearch(menu.title, this);
+	m_fluxbox->saveMenuSearch(menu.title, this);
 
 	attrib.event_mask |= PointerMotionMask;
-	menu.frame = XCreateWindow(display, menu.window, 0,
-				menu.title_h + screen->getBorderWidth(),
+	menu.frame = XCreateWindow(m_display, menu.window, 0,
+				menu.title_h + m_screen->getBorderWidth(),
 				menu.width, menu.frame_h, 0,
-				screen->getDepth(), InputOutput,
-				screen->getVisual(), attrib_mask, &attrib);
-	fluxbox->saveMenuSearch(menu.frame, this);
-
-	// even though this is the end of the constructor the menu is still not
-	// completely created.	items must be inserted and it must be update()'d
+				m_screen->getDepth(), InputOutput,
+				m_screen->getVisual(), attrib_mask, &attrib);
+	m_fluxbox->saveMenuSearch(menu.frame, this);
+	
 }
 
 
 Basemenu::~Basemenu(void) {
-	XUnmapWindow(display, menu.window);
+	XUnmapWindow(m_display, menu.window);
 
-	if (shown && shown->getWindowID() == getWindowID())
+	if (shown && shown->windowID() == windowID())
 		shown = (Basemenu *) 0;
 
+	//TODO: this looks kind of strange
 	int n = menuitems.size() - 1;
 	for (int i = 0; i < n; ++i)
 		remove(0);
 
-	if (menu.label)
-		delete [] menu.label;
-
 	if (menu.title_pixmap)
-		image_ctrl->removeImage(menu.title_pixmap);
+		m_image_ctrl->removeImage(menu.title_pixmap);
 
 	if (menu.frame_pixmap)
-		image_ctrl->removeImage(menu.frame_pixmap);
+		m_image_ctrl->removeImage(menu.frame_pixmap);
 
 	if (menu.hilite_pixmap)
-		image_ctrl->removeImage(menu.hilite_pixmap);
+		m_image_ctrl->removeImage(menu.hilite_pixmap);
 
 	if (menu.sel_pixmap)
-		image_ctrl->removeImage(menu.sel_pixmap);
+		m_image_ctrl->removeImage(menu.sel_pixmap);
 
-	fluxbox->removeMenuSearch(menu.title);
-	XDestroyWindow(display, menu.title);
+	m_fluxbox->removeMenuSearch(menu.title);
+	XDestroyWindow(m_display, menu.title);
 
-	fluxbox->removeMenuSearch(menu.frame);
-	XDestroyWindow(display, menu.frame);
+	m_fluxbox->removeMenuSearch(menu.frame);
+	XDestroyWindow(m_display, menu.frame);
 
-	fluxbox->removeMenuSearch(menu.window);
-	XDestroyWindow(display, menu.window);
+	m_fluxbox->removeMenuSearch(menu.window);
+	XDestroyWindow(m_display, menu.window);
 }
 
 
@@ -217,27 +214,13 @@ int Basemenu::insert(const char *label, Basemenu *submenu, int pos) {
 		menuitems.insert(menuitems.begin() + pos, item);
 	}
 
-	submenu->parent = this;
+	submenu->m_parent = this;
 
 	return menuitems.size();
 }
 
-
-int Basemenu::insert(const char **ulabel, int pos, int function) {
-	assert(ulabel);
-	BasemenuItem *item = new BasemenuItem(*ulabel, function);
-	if (pos == -1) {
-		menuitems.push_back(item);
-	} else {
-		menuitems.insert(menuitems.begin() + pos, item);
-	}
-
-	return menuitems.size();
-}
-
-
-int Basemenu::remove(int index) {
-	if (index < 0 || index >= menuitems.size()) {
+int Basemenu::remove(unsigned int index) {
+	if (index >= menuitems.size()) {
 		#ifdef DEBUG
 		std::cout << "Bad index (" << index << ") given to Basemenu::remove()"
 		          << " -- should be between 0 and " << menuitems.size()-1
@@ -264,9 +247,9 @@ int Basemenu::remove(int index) {
 		delete item;
 	}
 
-	if (which_sub == index)
+	if (static_cast<unsigned int>(which_sub) == index)
 		which_sub = -1;
-	else if (which_sub > index)
+	else if (static_cast<unsigned int>(which_sub) > index)
 		which_sub--;
 
 	return menuitems.size();
@@ -277,37 +260,33 @@ void Basemenu::update(void) {
 	I18n *i18n = I18n::instance();
 	
 	if (i18n->multibyte()) {
-		menu.item_h = screen->getMenuStyle()->framefont.set_extents->max_ink_extent.height +
+		menu.item_h = m_screen->getMenuStyle()->framefont->getFontSetExtents()->max_ink_extent.height +
 			menu.bevel_w;
-		menu.title_h =	screen->getMenuStyle()->titlefont.set_extents->max_ink_extent.height +
+		menu.title_h =	m_screen->getMenuStyle()->titlefont->getFontSetExtents()->max_ink_extent.height +
 			(menu.bevel_w * 2);
 	} else {
-		menu.item_h = screen->getMenuStyle()->framefont.fontstruct->ascent +
-			screen->getMenuStyle()->framefont.fontstruct->descent +
+		menu.item_h = m_screen->getMenuStyle()->framefont->getFontStruct()->ascent +
+			m_screen->getMenuStyle()->framefont->getFontStruct()->descent +
 			menu.bevel_w;
-		menu.title_h =	screen->getMenuStyle()->titlefont.fontstruct->ascent +
-				screen->getMenuStyle()->titlefont.fontstruct->descent +
+		menu.title_h =	m_screen->getMenuStyle()->titlefont->getFontStruct()->ascent +
+				m_screen->getMenuStyle()->titlefont->getFontStruct()->descent +
 				(menu.bevel_w * 2);
 	}
 		
 	if (title_vis) {
-		const char *s = (menu.label) ? menu.label :
-				i18n->getMessage(
-#ifdef		NLS
-						 BasemenuSet, BasemenuBlackboxMenu,
-#else // !NLS
-						 0, 0,
-#endif // NLS
-						 "fluxbox Menu");
+		const char *s = (menu.label!="") ? menu.label.c_str() :
+			i18n->getMessage(
+				 FBNLS::BasemenuSet, FBNLS::BasemenuBlackboxMenu,
+				 "fluxbox Menu");
 		int l = strlen(s);
 		
 
 		if (i18n->multibyte()) {
 			XRectangle ink, logical;
-			XmbTextExtents(screen->getMenuStyle()->titlefont.set, s, l, &ink, &logical);
+			XmbTextExtents(m_screen->getMenuStyle()->titlefont->getFontSet(), s, l, &ink, &logical);
 			menu.item_w = logical.width;
 		} else
-			menu.item_w = XTextWidth(screen->getMenuStyle()->titlefont.fontstruct, s, l);
+			menu.item_w = XTextWidth(m_screen->getMenuStyle()->titlefont->getFontStruct(), s, l);
 		
 		menu.item_w += (menu.bevel_w * 2);
 	}	else
@@ -319,15 +298,15 @@ void Basemenu::update(void) {
 	for (; it != it_end; ++it) {
 		BasemenuItem *itmp = (*it);
 
-		const char *s = itmp->label();
-		int l = strlen(s);
+		const char *s = itmp->label().c_str();
+		int l = itmp->label().size();
 
 		if (i18n->multibyte()) {
 			XRectangle ink, logical;
-			XmbTextExtents(screen->getMenuStyle()->framefont.set, s, l, &ink, &logical);
+			XmbTextExtents(m_screen->getMenuStyle()->framefont->getFontSet(), s, l, &ink, &logical);
 			ii = logical.width;
 		} else
-			ii = XTextWidth(screen->getMenuStyle()->framefont.fontstruct, s, l);
+			ii = XTextWidth(m_screen->getMenuStyle()->framefont->getFontStruct(), s, l);
 
 		ii += (menu.bevel_w * 2) + (menu.item_h * 2);
 
@@ -337,9 +316,9 @@ void Basemenu::update(void) {
 	if (menuitems.size()) {
 		menu.sublevels = 1;
 
-		while (((menu.item_h * (menuitems.size() + 1) / menu.sublevels)
-			+ menu.title_h + screen->getBorderWidth()) >
-		 screen->getHeight())
+		while (((menu.item_h * (menuitems.size() + 1) / menu.sublevels) +
+				menu.title_h + m_screen->getBorderWidth()) >
+				m_screen->getHeight())
 			menu.sublevels++;
 
 		if (menu.sublevels < menu.minsub) menu.sublevels = menu.minsub;
@@ -355,7 +334,7 @@ void Basemenu::update(void) {
 	if (! menu.width) menu.width = menu.item_w;
 
 	menu.frame_h = (menu.item_h * menu.persub);
-	menu.height = ((title_vis) ? menu.title_h + screen->getBorderWidth() : 0) +
+	menu.height = ((title_vis) ? menu.title_h + m_screen->getBorderWidth() : 0) +
 		menu.frame_h;
 	if (! menu.frame_h) menu.frame_h = 1;
 	if (menu.height < 1) menu.height = 1;
@@ -364,41 +343,41 @@ void Basemenu::update(void) {
 	BTexture *texture;
 	if (title_vis) {
 		tmp = menu.title_pixmap;
-		texture = &(screen->getMenuStyle()->title);
+		texture = &(m_screen->getMenuStyle()->title);
 		if (texture->getTexture() == (BImage::FLAT | BImage::SOLID)) {
 			menu.title_pixmap = None;
-			XSetWindowBackground(display, menu.title,
+			XSetWindowBackground(m_display, menu.title,
 				 texture->getColor()->getPixel());
 		} else {
 			menu.title_pixmap =
-				image_ctrl->renderImage(menu.width, menu.title_h, texture);
-			XSetWindowBackgroundPixmap(display, menu.title, menu.title_pixmap);
+				m_image_ctrl->renderImage(menu.width, menu.title_h, texture);
+			XSetWindowBackgroundPixmap(m_display, menu.title, menu.title_pixmap);
 		}
-		if (tmp) image_ctrl->removeImage(tmp);
-		XClearWindow(display, menu.title);
+		if (tmp) m_image_ctrl->removeImage(tmp);
+		XClearWindow(m_display, menu.title);
 	}
 
 	tmp = menu.frame_pixmap;
-	texture = &(screen->getMenuStyle()->frame);
+	texture = &(m_screen->getMenuStyle()->frame);
 	if (texture->getTexture() == (BImage::FLAT | BImage::SOLID)) {
 		menu.frame_pixmap = None;
-		XSetWindowBackground(display, menu.frame,
+		XSetWindowBackground(m_display, menu.frame,
 			 texture->getColor()->getPixel());
 	} else {
 		menu.frame_pixmap =
-			image_ctrl->renderImage(menu.width, menu.frame_h, texture);
-		XSetWindowBackgroundPixmap(display, menu.frame, menu.frame_pixmap);
+			m_image_ctrl->renderImage(menu.width, menu.frame_h, texture);
+		XSetWindowBackgroundPixmap(m_display, menu.frame, menu.frame_pixmap);
 	}
-	if (tmp) image_ctrl->removeImage(tmp);
+	if (tmp) m_image_ctrl->removeImage(tmp);
 
 	tmp = menu.hilite_pixmap;
-	texture = &(screen->getMenuStyle()->hilite);
+	texture = &(m_screen->getMenuStyle()->hilite);
 	if (texture->getTexture() == (BImage::FLAT | BImage::SOLID))
 		menu.hilite_pixmap = None;
 	else
 		menu.hilite_pixmap =
-			image_ctrl->renderImage(menu.item_w, menu.item_h, texture);
-	if (tmp) image_ctrl->removeImage(tmp);
+			m_image_ctrl->renderImage(menu.item_w, menu.item_h, texture);
+	if (tmp) m_image_ctrl->removeImage(tmp);
 
 	tmp = menu.sel_pixmap;
 	if (texture->getTexture() == (BImage::FLAT | BImage::SOLID))
@@ -406,47 +385,48 @@ void Basemenu::update(void) {
 	else {
 		int hw = menu.item_h / 2;
 		menu.sel_pixmap =
-			image_ctrl->renderImage(hw, hw, texture);
+			m_image_ctrl->renderImage(hw, hw, texture);
 	}
-	if (tmp) image_ctrl->removeImage(tmp);
+	if (tmp) m_image_ctrl->removeImage(tmp);
 
-	XResizeWindow(display, menu.window, menu.width, menu.height);
+	XResizeWindow(m_display, menu.window, menu.width, menu.height);
 
 	if (title_vis)
-		XResizeWindow(display, menu.title, menu.width, menu.title_h);
+		XResizeWindow(m_display, menu.title, menu.width, menu.title_h);
 
-	XMoveResizeWindow(display, menu.frame, 0,
+	XMoveResizeWindow(m_display, menu.frame, 0,
 				((title_vis) ? menu.title_h +
-				 screen->getBorderWidth() : 0), menu.width,
+				 m_screen->getBorderWidth() : 0), menu.width,
 				menu.frame_h);
 
-	XClearWindow(display, menu.window);
-	XClearWindow(display, menu.title);
-	XClearWindow(display, menu.frame);
+	XClearWindow(m_display, menu.window);
+	XClearWindow(m_display, menu.title);
+	XClearWindow(m_display, menu.frame);
 
 	if (title_vis && visible) redrawTitle();
 
-	int i = 0;
-	for (i = 0; visible && i < menuitems.size(); i++)
-		if (i == which_sub) {
+	unsigned int i = 0;
+	for (i = 0; visible && i < menuitems.size(); i++) {
+		if (i == (unsigned int)which_sub) {
 			drawItem(i, True, 0);
 			drawSubmenu(i);
 		} else
 			drawItem(i, False, 0);
+	}
 
-	if (parent && visible)
-		parent->drawSubmenu(parent->which_sub);
+	if (m_parent && visible)
+		m_parent->drawSubmenu(m_parent->which_sub);
 
-	XMapSubwindows(display, menu.window);
+	XMapSubwindows(m_display, menu.window);
 }
 
 
 void Basemenu::show(void) {
-	XMapSubwindows(display, menu.window);
-	XMapWindow(display, menu.window);
+	XMapSubwindows(m_display, menu.window);
+	XMapWindow(m_display, menu.window);
 	visible = True;
 
-	if (! parent) {
+	if (! m_parent) {
 		if (shown && (! shown->torn))
 			 shown->hide();
 
@@ -456,10 +436,10 @@ void Basemenu::show(void) {
 
 
 void Basemenu::hide(void) {
-	if ((! torn) && hide_tree && parent && parent->isVisible()) {
-		Basemenu *p = parent;
+	if ((! torn) && hide_tree && m_parent && m_parent->isVisible()) {
+		Basemenu *p = m_parent;
 
-		while (p->isVisible() && (! p->torn) && p->parent) p = p->parent;
+		while (p->isVisible() && (! p->torn) && p->m_parent) p = p->m_parent;
 		p->internal_hide();
 	} else
 		internal_hide();
@@ -472,24 +452,24 @@ void Basemenu::internal_hide(void) {
 		tmp->submenu()->internal_hide();
 	}
 
-	if (parent && (! torn)) {
-		parent->drawItem(parent->which_sub, False, True);
+	if (m_parent && (! torn)) {
+		m_parent->drawItem(m_parent->which_sub, False, True);
 
-		parent->which_sub = -1;
+		m_parent->which_sub = -1;
 	} else if (shown && shown->menu.window == menu.window)
 		shown = (Basemenu *) 0;
 
 	torn = visible = False;
 	which_sub = which_press = which_sub = -1;
 
-	XUnmapWindow(display, menu.window);
+	XUnmapWindow(m_display, menu.window);
 }
 
 
 void Basemenu::move(int x, int y) {
 	menu.x = x;
 	menu.y = y;
-	XMoveWindow(display, menu.window, x, y);
+	XMoveWindow(m_display, menu.window, x, y);
 	if (which_sub != -1)
 		drawSubmenu(which_sub);
 }
@@ -497,27 +477,19 @@ void Basemenu::move(int x, int y) {
 
 void Basemenu::redrawTitle(void) {
 	I18n *i18n = I18n::instance();
-	char *text = (char *) ((menu.label) ? menu.label :
+	const char *text = menu.label.size() ? menu.label.c_str() :
 			i18n->getMessage(
-#ifdef		NLS
-					BasemenuSet, BasemenuBlackboxMenu,
-#else // !NLS
-					0, 0,
-#endif // NLS
-					"fluxbox Menu"));
+					FBNLS::BasemenuSet, FBNLS::BasemenuBlackboxMenu,
+					"fluxbox Menu");
 	int dx = menu.bevel_w, len = strlen(text);
 	unsigned int l;
 
-	if (i18n->multibyte()) {
-		XRectangle ink, logical;
-		XmbTextExtents(screen->getMenuStyle()->titlefont.set, text, len, &ink, &logical);
-		l = logical.width;
-	} else
-		l = XTextWidth(screen->getMenuStyle()->titlefont.fontstruct, text, len);
+	l = m_screen->getMenuStyle()->titlefont->getTextWidth(text, len);
 
-	l +=	(menu.bevel_w * 2);
+	l += (menu.bevel_w * 2);
 
-	switch (screen->getMenuStyle()->titlefont.justify) {
+	//titlefont.justify
+	switch (m_screen->getMenuStyle()->titlefont_justify) {
 	case DrawUtil::Font::RIGHT:
 		dx += menu.width - l;
 		break;
@@ -530,71 +502,110 @@ void Basemenu::redrawTitle(void) {
 	}
 
 	if (i18n->multibyte())
-		XmbDrawString(display, menu.title, screen->getMenuStyle()->titlefont.set,
-			screen->getMenuStyle()->t_text_gc, dx, menu.bevel_w -
-			screen->getMenuStyle()->titlefont.set_extents->max_ink_extent.y,
+		XmbDrawString(m_display, menu.title, m_screen->getMenuStyle()->titlefont->getFontSet(),
+			m_screen->getMenuStyle()->t_text_gc, dx, menu.bevel_w -
+			m_screen->getMenuStyle()->titlefont->getFontSetExtents()->max_ink_extent.y,
 			text, len);
 	else
-		XDrawString(display, menu.title, screen->getMenuStyle()->t_text_gc, dx,
-		screen->getMenuStyle()->titlefont.fontstruct->ascent + menu.bevel_w,
+		XDrawString(m_display, menu.title, m_screen->getMenuStyle()->t_text_gc, dx,
+		m_screen->getMenuStyle()->titlefont->getFontStruct()->ascent + menu.bevel_w,
 		text, len);
 }
 
 
-void Basemenu::drawSubmenu(int index) {
-	if (which_sub >= 0 && which_sub != index) {
+void Basemenu::drawSubmenu(unsigned int index) {
+	if (which_sub >= 0 && static_cast<unsigned int>(which_sub) != index && 
+		static_cast<unsigned int>(which_sub) < menuitems.size()) {
 		BasemenuItem *itmp = menuitems[which_sub];
 
 		if (! itmp->submenu()->isTorn())
 			itmp->submenu()->internal_hide();
 	}
 
-	if (index >= 0 && index < menuitems.size()) {
+	if (index < menuitems.size()) {
 		BasemenuItem *item = menuitems[index];
 		if (item->submenu() && visible && (! item->submenu()->isTorn()) &&
 				item->isEnabled()) {
 			
-			if (item->submenu()->parent != this)
-				item->submenu()->parent = this;
+			if (item->submenu()->m_parent != this)
+				item->submenu()->m_parent = this;
 			
 			int sbl = index / menu.persub, i = index - (sbl * menu.persub),
 			x = menu.x +
-				((menu.item_w * (sbl + 1)) + screen->getBorderWidth()), y;
+				((menu.item_w * (sbl + 1)) + m_screen->getBorderWidth()), y;
 		
-			if (alignment == ALIGNTOP) {
+			if (m_alignment == ALIGNTOP) {
 				y = (((shifted) ? menu.y_shift : menu.y) +
-				 ((title_vis) ? menu.title_h + screen->getBorderWidth() : 0) -
+				 ((title_vis) ? menu.title_h + m_screen->getBorderWidth() : 0) -
 				 ((item->submenu()->title_vis) ?
-					item->submenu()->menu.title_h + screen->getBorderWidth() : 0));
+					item->submenu()->menu.title_h + m_screen->getBorderWidth() : 0));
 			} else {
 				y = (((shifted) ? menu.y_shift : menu.y) +
 				 (menu.item_h * i) +
-				 ((title_vis) ? menu.title_h + screen->getBorderWidth() : 0) -
+				 ((title_vis) ? menu.title_h + m_screen->getBorderWidth() : 0) -
 				 ((item->submenu()->title_vis) ?
-					item->submenu()->menu.title_h + screen->getBorderWidth() : 0));
+					item->submenu()->menu.title_h + m_screen->getBorderWidth() : 0));
 			}
 			
-			if (alignment == ALIGNBOTTOM &&
+			if (m_alignment == ALIGNBOTTOM &&
 					(y + item->submenu()->menu.height) > ((shifted) ? menu.y_shift :
 					menu.y) + menu.height) {
 				y = (((shifted) ? menu.y_shift : menu.y) +
 					 menu.height - item->submenu()->menu.height);
 			}
-			
-			if ((x + item->submenu()->getWidth()) > screen->getWidth()) {
-				x = ((shifted) ? menu.x_shift : menu.x) -
-					item->submenu()->getWidth() - screen->getBorderWidth();
-			}
-			
-			if (x < 0) x = 0;
 
-			if ((y + item->submenu()->getHeight()) > screen->getHeight()) {
-				y = screen->getHeight() - item->submenu()->getHeight() -
-					screen->getBorderWidth2x();
+			#ifdef XINERAMA
+			int head_x = 0,
+					head_y = 0,
+					head_w,
+					head_h;
+
+			unsigned int head = 0;
+			if (m_screen->hasXinerama()) {
+				head = m_screen->getHead(menu.x, menu.y);
+				head_x = m_screen->getHeadX(head);
+				head_y = m_screen->getHeadY(head);
+				head_w = m_screen->getHeadWidth(head);
+				head_h = m_screen->getHeadHeight(head);
+			} else {
+				head_w = m_screen->getWidth();
+				head_h = m_screen->getHeight();
+			}
+
+			if (static_cast<int>(x + item->submenu()->width()) > static_cast<int>(head_x + head_w)) {
+				x = ((shifted) ? menu.x_shift : menu.x) -
+					item->submenu()->width() - m_screen->getBorderWidth();
 			}
 			
-			if (y < 0) y = 0;
+			if (x < head_x)
+				x = head_x;
+
+			if (static_cast<int>(y + item->submenu()->height()) > static_cast<int>(head_y + head_h)) {
+				y = head_y + head_h -
+					item->submenu()->height() - m_screen->getBorderWidth2x();
+			}
+
+			if (y < head_y)
+				y = head_y;
+			#else // !XINERAMA
+
+			if ((x + item->submenu()->width()) > m_screen->getWidth()) {
+				x = ((shifted) ? menu.x_shift : menu.x) -
+					item->submenu()->width() - m_screen->getBorderWidth();
+			}
 			
+			if (x < 0)
+				x = 0;
+
+			if ((y + item->submenu()->height()) > m_screen->getHeight()) {
+				y = m_screen->getHeight() - item->submenu()->height() -
+					m_screen->getBorderWidth2x();
+			}
+			
+			if (y < 0)
+				y = 0;
+			#endif // XINERAMA
+
 			item->submenu()->move(x, y);
 			if (! moving)
 				drawItem(index, True);
@@ -610,27 +621,27 @@ void Basemenu::drawSubmenu(int index) {
 }
 
 
-Bool Basemenu::hasSubmenu(int index) {
-	if ((index >= 0) && (index < menuitems.size()))
-		if (menuitems[index]->submenu())
-			return True;
-		else
-			return False;
-	else
-		return False;
+bool Basemenu::hasSubmenu(unsigned int index) const {
+	if (index >= menuitems.size()) //boundary check
+		return false;
+	
+	if (!menuitems[index]->submenu()) //has submenu?
+		return false;
+	
+	return true;	
 }
 
 
-void Basemenu::drawItem(int index, Bool highlight, Bool clear,
+void Basemenu::drawItem(unsigned int index, bool highlight, bool clear,
 			 int x, int y, unsigned int w, unsigned int h)
 {
-	if (index < 0 || index > menuitems.size()) return;
+	if (index >= menuitems.size()) return;
 
 	BasemenuItem *item = menuitems[index];
 	if (! item) return;
 	
-	Bool dotext = True, dohilite = True, dosel = True;
-	const char *text = item->label();
+	bool dotext = true, dohilite = true, dosel = true;
+	const char *text = item->label().c_str();
 	int sbl = index / menu.persub, i = index - (sbl * menu.persub);
 	int item_x = (sbl * menu.item_w), item_y = (i * menu.item_h);
 	int hilite_x = item_x, hilite_y = item_y, hoff_x = 0, hoff_y = 0;
@@ -641,20 +652,19 @@ void Basemenu::drawItem(int index, Bool highlight, Bool clear,
 	I18n *i18n = I18n::instance();
 	
 	if (text) {		
-		if (i18n->multibyte()) {
-			XRectangle ink, logical;
-			XmbTextExtents(screen->getMenuStyle()->framefont.set, text, len, &ink, &logical);
-			text_w = logical.width;
+		text_w = m_screen->getMenuStyle()->framefont->getTextWidth(text, len);
+		
+		if (m_screen->getMenuStyle()->framefont->multibyte()) {
 			text_y = item_y + (menu.bevel_w / 2) -
-				screen->getMenuStyle()->framefont.set_extents->max_ink_extent.y;
+				m_screen->getMenuStyle()->framefont->getFontSetExtents()->max_ink_extent.y;
 		} else {
-			text_w = XTextWidth(screen->getMenuStyle()->framefont.fontstruct, text, len);
-			text_y =	item_y +
-				screen->getMenuStyle()->framefont.fontstruct->ascent +
+			text_y = item_y +
+				m_screen->getMenuStyle()->framefont->getFontStruct()->ascent +
 				(menu.bevel_w / 2);
 		}
-		
-		switch(screen->getMenuStyle()->framefont.justify) {
+
+		// framfont.justify
+		switch(m_screen->getMenuStyle()->framefont_justify) {
 		case DrawUtil::Font::LEFT:
 			text_x = item_x + menu.bevel_w + menu.item_h + 1;
 			break;
@@ -666,97 +676,97 @@ void Basemenu::drawItem(int index, Bool highlight, Bool clear,
 			text_x = item_x + ((menu.item_w + 1 - text_w) / 2);
 			break;
 		}
-		
+
 		text_h = menu.item_h - menu.bevel_w;
 	}
 	
 	GC gc =
-		((highlight || item->isSelected()) ? screen->getMenuStyle()->h_text_gc :
-		screen->getMenuStyle()->f_text_gc),
+		((highlight || item->isSelected()) ? m_screen->getMenuStyle()->h_text_gc :
+		m_screen->getMenuStyle()->f_text_gc),
 		tgc =
-			((highlight) ? screen->getMenuStyle()->h_text_gc :
-			((item->isEnabled()) ? screen->getMenuStyle()->f_text_gc :
-			screen->getMenuStyle()->d_text_gc));
+			((highlight) ? m_screen->getMenuStyle()->h_text_gc :
+			((item->isEnabled()) ? m_screen->getMenuStyle()->f_text_gc :
+			m_screen->getMenuStyle()->d_text_gc));
 	
 	sel_x = item_x;
 	
-	if (screen->getMenuStyle()->bullet_pos == RIGHT)
+	if (m_screen->getMenuStyle()->bullet_pos == RIGHT)
 		sel_x += (menu.item_w - menu.item_h - menu.bevel_w);
 	
 	sel_x += quarter_w;
 	sel_y = item_y + quarter_w;
 	
 	if (clear) {
-		XClearArea(display, menu.frame, item_x, item_y, menu.item_w, menu.item_h,
+		XClearArea(m_display, menu.frame, item_x, item_y, menu.item_w, menu.item_h,
 			False);
 	} else if (! (x == y && y == -1 && w == h && h == 0)) {
 		// calculate the which part of the hilite to redraw
-		if (! (max(item_x, x) <= (signed) min(item_x + menu.item_w, x + w) &&
-				max(item_y, y) <= (signed) min(item_y + menu.item_h, y + h))) {
+		if (! (std::max(item_x, x) <= (signed) std::min(item_x + menu.item_w, x + w) &&
+				std::max(item_y, y) <= (signed) std::min(item_y + menu.item_h, y + h))) {
 			dohilite = False;
 		} else {
-			hilite_x = max(item_x, x);
-			hilite_y = max(item_y, y);
-			hilite_w = min(item_x + menu.item_w, x + w) - hilite_x;
-			hilite_h = min(item_y + menu.item_h, y + h) - hilite_y;
+			hilite_x = std::max(item_x, x);
+			hilite_y = std::max(item_y, y);
+			hilite_w = std::min(item_x + menu.item_w, x + w) - hilite_x;
+			hilite_h = std::min(item_y + menu.item_h, y + h) - hilite_y;
 			hoff_x = hilite_x % menu.item_w;
 			hoff_y = hilite_y % menu.item_h;
 		}
 		
 		// check if we need to redraw the text		
 		int text_ry = item_y + (menu.bevel_w / 2);
-		if (! (max(text_x, x) <= (signed) min(text_x + text_w, x + w) &&
-				max(text_ry, y) <= (signed) min(text_ry + text_h, y + h)))
+		if (! (std::max(text_x, x) <= (signed) std::min(text_x + text_w, x + w) &&
+				std::max(text_ry, y) <= (signed) std::min(text_ry + text_h, y + h)))
 			dotext = False;
 		
 		// check if we need to redraw the select pixmap/menu bullet
-		if (! (max(sel_x, x) <= (signed) min(sel_x + half_w, x + w) &&
-				max(sel_y, y) <= (signed) min(sel_y + half_w, y + h)))
+		if (! (std::max(sel_x, x) <= (signed) std::min(sel_x + half_w, x + w) &&
+				std::max(sel_y, y) <= (signed) std::min(sel_y + half_w, y + h)))
 			dosel = False;
 	
 	}
 	
 	if (dohilite && highlight && (menu.hilite_pixmap != ParentRelative)) {
 		if (menu.hilite_pixmap) {
-			XCopyArea(display, menu.hilite_pixmap, menu.frame,		
-				screen->getMenuStyle()->hilite_gc, hoff_x, hoff_y,
+			XCopyArea(m_display, menu.hilite_pixmap, menu.frame,		
+				m_screen->getMenuStyle()->hilite_gc, hoff_x, hoff_y,
 				hilite_w, hilite_h, hilite_x, hilite_y);
 		} else {
-			XFillRectangle(display, menu.frame,
-				screen->getMenuStyle()->hilite_gc,
+			XFillRectangle(m_display, menu.frame,
+				m_screen->getMenuStyle()->hilite_gc,
 				hilite_x, hilite_y, hilite_w, hilite_h);
 		}
 	} else if (dosel && item->isSelected() &&
 					(menu.sel_pixmap != ParentRelative)) {
 		if (menu.sel_pixmap) {
-			XCopyArea(display, menu.sel_pixmap, menu.frame,		
-				screen->getMenuStyle()->hilite_gc, 0, 0,
+			XCopyArea(m_display, menu.sel_pixmap, menu.frame,		
+				m_screen->getMenuStyle()->hilite_gc, 0, 0,
 				half_w, half_w, sel_x, sel_y);
 		} else {
-			XFillRectangle(display, menu.frame,
-				screen->getMenuStyle()->hilite_gc,
+			XFillRectangle(m_display, menu.frame,
+				m_screen->getMenuStyle()->hilite_gc,
 				sel_x, sel_y, half_w, half_w);
 		}
 	}
 	
 	if (dotext && text) {
 		if (i18n->multibyte()) {
-			XmbDrawString(display, menu.frame, screen->getMenuStyle()->framefont.set,
+			XmbDrawString(m_display, menu.frame, m_screen->getMenuStyle()->framefont->getFontSet(),
 				tgc, text_x, text_y, text, len);
 		} else
-			XDrawString(display, menu.frame, tgc, text_x, text_y, text, len);
+			XDrawString(m_display, menu.frame, tgc, text_x, text_y, text, len);
 	}
 
 	if (dosel && item->submenu()) {
-		switch (screen->getMenuStyle()->bullet) {
+		switch (m_screen->getMenuStyle()->bullet) {
 		case SQUARE:
-			XDrawRectangle(display, menu.frame, gc, sel_x, sel_y, half_w, half_w);
+			XDrawRectangle(m_display, menu.frame, gc, sel_x, sel_y, half_w, half_w);
 			break;
 
 		case TRIANGLE:
 			XPoint tri[3];
 
-			if (screen->getMenuStyle()->bullet_pos == RIGHT) {
+			if (m_screen->getMenuStyle()->bullet_pos == RIGHT) {
 				tri[0].x = sel_x + quarter_w - 2;
 				tri[0].y = sel_y + quarter_w - 2;
 				tri[1].x = 4;
@@ -772,7 +782,7 @@ void Basemenu::drawItem(int index, Bool highlight, Bool clear,
 				tri[2].y = -4;
 			}
 			
-			XFillPolygon(display, menu.frame, gc, tri, 3, Convex,
+			XFillPolygon(m_display, menu.frame, gc, tri, 3, Convex,
 						CoordModePrevious);
 			break;
 			
@@ -788,7 +798,7 @@ void Basemenu::drawItem(int index, Bool highlight, Bool clear,
 			dia[3].x = -3;
 			dia[3].y = 3;
 
-			XFillPolygon(display, menu.frame, gc, dia, 4, Convex,
+			XFillPolygon(m_display, menu.frame, gc, dia, 4, Convex,
 					CoordModePrevious);
 			break;
 		}
@@ -797,51 +807,48 @@ void Basemenu::drawItem(int index, Bool highlight, Bool clear,
 
 
 void Basemenu::setLabel(const char *l) {
-	if (menu.label)
-		delete [] menu.label;
-
-	if (l) menu.label = StringUtil::strdup(l);
-	else menu.label = 0;
+	//we don't want 0 to std::string
+	menu.label = l ? l : "";
 }
 
 
-void Basemenu::setItemSelected(int index, Bool sel) {
-	if (index < 0 || index >= menuitems.size()) return;
+void Basemenu::setItemSelected(unsigned int index, bool sel) {
+	if (index >= menuitems.size()) return;
 
 	BasemenuItem *item = find(index);
 	if (! item) return;
 
 	item->setSelected(sel);
-	if (visible) drawItem(index, (index == which_sub), True);
+	if (visible) drawItem(index, (index == (unsigned int)which_sub), true);
 }
 
 
-Bool Basemenu::isItemSelected(int index) {
-	if (index < 0 || index >= menuitems.size()) return False;
+bool Basemenu::isItemSelected(unsigned int index) const{
+	if (index >= menuitems.size()) return false;
 
 	BasemenuItem *item = find(index);
-	if (! item) return False;
+	if (! item) return false;
 
 	return item->isSelected();
 }
 
 
-void Basemenu::setItemEnabled(int index, Bool enable) {
-	if (index < 0 || index >= menuitems.size()) return;
+void Basemenu::setItemEnabled(unsigned int index, bool enable) {
+	if (index >= menuitems.size()) return;
 
 	BasemenuItem *item = find(index);
 	if (! item) return;
 
 	item->setEnabled(enable);
-	if (visible) drawItem(index, (index == which_sub), True);
+	if (visible) drawItem(index, (index == static_cast<unsigned int>(which_sub)), True);
 }
 
 
-Bool Basemenu::isItemEnabled(int index) {
-	if (index < 0 || index >= menuitems.size()) return False;
+bool Basemenu::isItemEnabled(unsigned int index) const {
+	if (index >= menuitems.size()) return false;
 
 	BasemenuItem *item = find(index);
-	if (! item) return False;
+	if (! item) return false;
 
 	return item->isEnabled();
 }
@@ -852,7 +859,7 @@ void Basemenu::buttonPressEvent(XButtonEvent *be) {
 		int sbl = (be->x / menu.item_w), i = (be->y / menu.item_h);
 		int w = (sbl * menu.persub) + i;
 
-		if (w < menuitems.size() && w >= 0) {
+		if (w < static_cast<int>(menuitems.size()) && w >= 0) {
 			which_press = i;
 			which_sbl = sbl;
 
@@ -895,7 +902,7 @@ void Basemenu::buttonReleaseEvent(XButtonEvent *re) {
 					w = (sbl * menu.persub) + i,
 					p = (which_sbl * menu.persub) + which_press;
 
-			if (w < menuitems.size() && w >= 0) {
+			if (w < static_cast<int>(menuitems.size()) && w >= 0) {
 				drawItem(p, (p == which_sub), True);
 
 				if	(p == w && isItemEnabled(w)) {
@@ -915,9 +922,9 @@ void Basemenu::motionNotifyEvent(XMotionEvent *me) {
 	if (me->window == menu.title && (me->state & Button1Mask)) {
 		if (movable) {
 			if (! moving) {
-				if (parent && (! torn)) {
-					parent->drawItem(parent->which_sub, False, True);
-					parent->which_sub = -1;
+				if (m_parent && (! torn)) {
+					m_parent->drawItem(m_parent->which_sub, False, True);
+					m_parent->which_sub = -1;
 				}
 
 				moving = torn = True;
@@ -928,7 +935,7 @@ void Basemenu::motionNotifyEvent(XMotionEvent *me) {
 				menu.x = me->x_root - menu.x_move,
 				menu.y = me->y_root - menu.y_move;
 	
-				XMoveWindow(display, menu.window, menu.x, menu.y);
+				XMoveWindow(m_display, menu.window, menu.x, menu.y);
 
 				if (which_sub >= 0)
 					drawSubmenu(which_sub);
@@ -941,7 +948,7 @@ void Basemenu::motionNotifyEvent(XMotionEvent *me) {
 		w = (sbl * menu.persub) + i;
 
 		if ((i != which_press || sbl != which_sbl) &&
-				(w < menuitems.size() && w >= 0)) {
+				(w < static_cast<int>(menuitems.size()) && w >= 0)) {
 			if (which_press != -1 && which_sbl != -1) {
 				int p = (which_sbl * menu.persub) + which_press;
 				BasemenuItem *item = menuitems[p];
@@ -991,7 +998,7 @@ void Basemenu::exposeEvent(XExposeEvent *ee) {
 		for (i = sbl; i <= sbl_d; i++) {
 			// set the iterator to the first item in the sublevel needing redrawing
 			int index = id + i * menu.persub;
-			if (index < menuitems.size() && index >= 0) {
+			if (index < static_cast<int>(menuitems.size()) && index >= 0) {
 				Menuitems::iterator it = menuitems.begin() + index;
 				Menuitems::iterator it_end = menuitems.end();
 				for (ii = id; ii <= id_d && it != it_end; ++it, ii++) {
@@ -1007,28 +1014,62 @@ void Basemenu::exposeEvent(XExposeEvent *ee) {
 
 
 void Basemenu::enterNotifyEvent(XCrossingEvent *ce) {
+#ifdef XINERAMA
+	int head = m_screen->hasXinerama() ? m_screen->getCurrHead() : 0;
+
 	if (ce->window == menu.frame) {
 		menu.x_shift = menu.x, menu.y_shift = menu.y;
-		if (menu.x + menu.width > screen->getWidth()) {
-			menu.x_shift = screen->getWidth() - menu.width -
-				screen->getBorderWidth();
+		if (menu.x + menu.width >
+				(m_screen->getHeadX(head) + m_screen->getHeadWidth(head))) {
+			menu.x_shift = m_screen->getHeadX(head) +  m_screen->getHeadWidth(head) -
+				menu.width - m_screen->getBorderWidth2x();
+			shifted = True;
+		} else if (menu.x < m_screen->getHeadX(head)) {
+			menu.x_shift = m_screen->getHeadX(head);
+			shifted = True;
+		}
+
+		if (menu.y + menu.height >
+				(m_screen->getHeadY(head) + m_screen->getHeadHeight(head))) {
+			menu.y_shift = m_screen->getHeadY(head) + m_screen->getHeadHeight(head) -
+				menu.height - m_screen->getBorderWidth2x();
+			shifted = True;
+		} else if (menu.y + (signed) menu.title_h < m_screen->getHeadY(head)) {
+			menu.y_shift = m_screen->getHeadY(head);
+			shifted = True;
+		}
+
+#else // !XINERAMA
+
+	if (ce->window == menu.frame) {
+		menu.x_shift = menu.x, menu.y_shift = menu.y;
+		if (menu.x + menu.width > m_screen->getWidth()) {
+			menu.x_shift = m_screen->getWidth() - menu.width -
+				m_screen->getBorderWidth();
 			shifted = True;
 		} else if (menu.x < 0) {
-			menu.x_shift = -screen->getBorderWidth();
+			menu.x_shift = -m_screen->getBorderWidth();
 			shifted = True;
 		}
 
-		if (menu.y + menu.height > screen->getHeight()) {
-			menu.y_shift = screen->getHeight() - menu.height -
-				screen->getBorderWidth();
+		if (menu.y + menu.height > m_screen->getHeight()) {
+			menu.y_shift = m_screen->getHeight() - menu.height -
+				m_screen->getBorderWidth();
 			shifted = True;
 		} else if (menu.y + (signed) menu.title_h < 0) {
-			menu.y_shift = -screen->getBorderWidth();
+			menu.y_shift = -m_screen->getBorderWidth();
 			shifted = True;
 		}
 
-		if (shifted)
-			XMoveWindow(display, menu.window, menu.x_shift, menu.y_shift);
+#endif // XINERAMA
+
+		if (shifted) {
+		#ifdef XINERAMA
+			menu.x = menu.x_shift; // need to do this to avoid jumping beetween heads
+			menu.y = menu.y_shift;
+		#endif // XINERAMA
+			XMoveWindow(m_display, menu.window, menu.x_shift, menu.y_shift);
+		}
 
 		if (which_sub >= 0) {
 			BasemenuItem *tmp = menuitems[which_sub];
@@ -1059,7 +1100,7 @@ void Basemenu::leaveNotifyEvent(XCrossingEvent *ce) {
 		}
 
 		if (shifted) {
-			XMoveWindow(display, menu.window, menu.x, menu.y);
+			XMoveWindow(m_display, menu.window, menu.x, menu.y);
 			shifted = False;
 
 			if (which_sub >= 0) drawSubmenu(which_sub);
@@ -1069,12 +1110,12 @@ void Basemenu::leaveNotifyEvent(XCrossingEvent *ce) {
 
 
 void Basemenu::reconfigure(void) {
-	XSetWindowBackground(display, menu.window,
-					 screen->getBorderColor()->getPixel());
-	XSetWindowBorder(display, menu.window,
-			 screen->getBorderColor()->getPixel());
-	XSetWindowBorderWidth(display, menu.window, screen->getBorderWidth());
+	XSetWindowBackground(m_display, menu.window,
+					 m_screen->getBorderColor()->getPixel());
+	XSetWindowBorder(m_display, menu.window,
+			 m_screen->getBorderColor()->getPixel());
+	XSetWindowBorderWidth(m_display, menu.window, m_screen->getBorderWidth());
 
-	menu.bevel_w = screen->getBevelWidth();
+	menu.bevel_w = m_screen->getBevelWidth();
 	update();
 }
