@@ -22,7 +22,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: Configmenu.cc,v 1.10 2002/01/11 12:30:22 fluxgen Exp $
+// $Id: Configmenu.cc,v 1.13 2002/04/12 15:14:09 fluxgen Exp $
 
 // stupid macros needed to access some functions in version 2 of the GNU C
 // library
@@ -39,7 +39,8 @@
 #include "Toolbar.hh"
 #include "Window.hh"
 
-enum {CMENU_USE_TABS=9, CMENU_USE_ICONS, CMENU_SLOPPY_WIN_GROUP, CMENU_TAB_ROTATE=21};
+enum {CMENU_USE_TABS=9, CMENU_USE_ICONS, CMENU_SLOPPY_WIN_GROUP,
+		CMENU_WORKSPACE_WARPING, CMENU_TAB_ROTATE=21};
 
 
 Configmenu::Configmenu(BScreen *scr) : Basemenu(scr) {
@@ -48,105 +49,54 @@ Configmenu::Configmenu(BScreen *scr) : Basemenu(scr) {
 	I18n *i18n = I18n::instance();
 	
 	setLabel(i18n->getMessage(
-#ifdef		NLS
-				ConfigmenuSet, ConfigmenuConfigOptions,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Config options"));
+		FBNLS::ConfigmenuSet, FBNLS::ConfigmenuConfigOptions,
+		"Config options"));
 	
 	setInternalMenu();
 
 	focusmenu = new Focusmenu(this);
 	placementmenu = new Placementmenu(this);
 	tabmenu = new Tabmenu(this);
-
+	using namespace FBNLS;
 	insert(i18n->getMessage(
-#ifdef		NLS
-				ConfigmenuSet, ConfigmenuFocusModel,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Focus Model"), focusmenu);
+		ConfigmenuSet, ConfigmenuFocusModel,
+		"Focus Model"), focusmenu);
 	insert(i18n->getMessage(
-#ifdef		NLS
-				ConfigmenuSet, ConfigmenuWindowPlacement,
-#else //! NLS
-				0, 0,
-#endif // NLS
-				"Window Placement"), placementmenu);
+		ConfigmenuSet, ConfigmenuWindowPlacement,
+		"Window Placement"), placementmenu);
 	insert(i18n->getMessage(
-#ifdef		NLS
-				ConfigmenuSet, ConfigmenuTabPlacement,
-#else // !NLS
-				0, 0,
-#endif	// NLS
-				"Tab Placement"), tabmenu);
+		ConfigmenuSet, ConfigmenuTabPlacement,
+		"Tab Placement"), tabmenu);
 	insert(i18n->getMessage(
-#ifdef		NLS
-				ConfigmenuSet, ConfigmenuImageDithering,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Image Dithering"), 1);
+		ConfigmenuSet, ConfigmenuImageDithering,
+		"Image Dithering"), 1);
 	insert(i18n->getMessage(
-#ifdef		NLS
-				ConfigmenuSet, ConfigmenuOpaqueMove,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Opaque Window Moving"), 2);
+		ConfigmenuSet, ConfigmenuOpaqueMove,
+		"Opaque Window Moving"), 2);
 	insert(i18n->getMessage(
-#ifdef		NLS
-				ConfigmenuSet, ConfigmenuFullMax,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Full Maximization"), 3);
+		ConfigmenuSet, ConfigmenuFullMax,
+		"Full Maximization"), 3);
 	insert(i18n->getMessage(
-#ifdef		NLS
-				ConfigmenuSet, ConfigmenuFocusNew,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Focus New Windows"), 4);
+		ConfigmenuSet, ConfigmenuFocusNew,
+		"Focus New Windows"), 4);
 	insert(i18n->getMessage(
-#ifdef		NLS
-				ConfigmenuSet, ConfigmenuFocusLast,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Focus Last Window on Workspace"), 5);
+		ConfigmenuSet, ConfigmenuFocusLast,
+		"Focus Last Window on Workspace"), 5);
 	insert(i18n->getMessage(
-#ifdef		NLS
-				ConfigmenuSet, ConfigmenuMaxOverSlit,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Maxmize Over Slit"), 6);
-
-
+		ConfigmenuSet, ConfigmenuMaxOverSlit,
+		"Maxmize Over Slit"), 6);
     insert(i18n->getMessage(
-#ifdef   NLS
-       ConfigmenuSet, ConfigmenuTabs,
-#else // !NLS
-       0, 0,
-#endif // NLS
-       "Use Tabs"), CMENU_USE_TABS);
- insert(i18n->getMessage(
-#ifdef   NLS
-       ConfigmenuSet, ConfigmenuIcons,
-#else // !NLS
-       0, 0,
-#endif // NLS
-       "Use Icons"), CMENU_USE_ICONS);
- insert(i18n->getMessage(
-#ifdef   NLS
-       ConfigmenuSet, ConfigmenuSloppyWindowGrouping,
-#else // !NLS
-       0, 0,
-#endif // NLS
-       "Sloppy Window Grouping"), CMENU_SLOPPY_WIN_GROUP);
+		ConfigmenuSet, ConfigmenuTabs,
+		"Use Tabs"), CMENU_USE_TABS);
+	insert(i18n->getMessage(
+		ConfigmenuSet, ConfigmenuIcons,
+		"Use Icons"), CMENU_USE_ICONS);
+	insert(i18n->getMessage(
+		ConfigmenuSet, ConfigmenuSloppyWindowGrouping,
+		"Sloppy Window Grouping"), CMENU_SLOPPY_WIN_GROUP);
+	insert(i18n->getMessage(
+		ConfigmenuSet, ConfigmenuWorkspaceWarping,
+		"Workspace Warping"), CMENU_WORKSPACE_WARPING);
 
 	update();
 	setItemSelected(8, screen->doMaxOverSlit());
@@ -159,6 +109,7 @@ Configmenu::Configmenu(BScreen *scr) : Basemenu(scr) {
 	setItemSelected(CMENU_USE_TABS, Fluxbox::instance()->useTabs());
 	setItemSelected(CMENU_USE_ICONS, Fluxbox::instance()->useIconBar());
 	setItemSelected(CMENU_SLOPPY_WIN_GROUP, screen->isSloppyWindowGrouping());
+	setItemSelected(CMENU_WORKSPACE_WARPING, screen->isWorkspaceWarping());
 }
 
 
@@ -169,7 +120,7 @@ Configmenu::~Configmenu(void) {
 }
 
 
-void Configmenu::itemSelected(int button, int index) {
+void Configmenu::itemSelected(int button, unsigned int index) {
 	if (button == 1) {
 		BasemenuItem *item = find(index);
 
@@ -235,6 +186,13 @@ void Configmenu::itemSelected(int button, int index) {
 					screen->reconfigure();
 				}
 				break;
+			case CMENU_WORKSPACE_WARPING:
+				{
+					screen->saveWorkspaceWarping(!screen->isWorkspaceWarping());
+					setItemSelected(index, screen->isWorkspaceWarping());
+					screen->reconfigure();
+				}
+				break;
 
 			}
 		}
@@ -253,57 +211,38 @@ void Configmenu::reconfigure(void) {
 Configmenu::Focusmenu::Focusmenu(Configmenu *cm) : Basemenu(cm->screen) {
 	configmenu = cm;
 	I18n *i18n = I18n::instance();
+	using namespace FBNLS;
 	setLabel(i18n->getMessage(
-#ifdef		NLS
-					ConfigmenuSet, ConfigmenuFocusModel,
-#else // !NLS
-					0, 0,
-#endif // NLS
-					"Focus Model"));
+			ConfigmenuSet, ConfigmenuFocusModel,
+			"Focus Model"));
 	setInternalMenu();
 
 	insert(i18n->getMessage(
-#ifdef		NLS
-				ConfigmenuSet, ConfigmenuClickToFocus,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Click To Focus"), 1);
+			ConfigmenuSet, ConfigmenuClickToFocus,
+			"Click To Focus"), 1);
 	insert(i18n->getMessage(
-#ifdef		NLS
-				ConfigmenuSet, ConfigmenuSloppyFocus,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Sloppy Focus"), 2);
+		ConfigmenuSet, ConfigmenuSloppyFocus,
+		"Sloppy Focus"), 2);
 	insert(i18n->getMessage(
-#ifdef		NLS
-				ConfigmenuSet, ConfigmenuSemiSloppyFocus,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Semi Sloppy Focus"), 3);
+		ConfigmenuSet, ConfigmenuSemiSloppyFocus,
+		"Semi Sloppy Focus"), 3);
 	insert(i18n->getMessage(
-#ifdef		NLS
-				ConfigmenuSet, ConfigmenuAutoRaise,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Auto Raise"), 4);
+		ConfigmenuSet, ConfigmenuAutoRaise,
+		"Auto Raise"), 4);
 
 	update();
 
 	setItemSelected(0, !(configmenu->screen->isSloppyFocus() || 
-							configmenu->screen->isSemiSloppyFocus()));
+		configmenu->screen->isSemiSloppyFocus()));
 	setItemSelected(1, configmenu->screen->isSloppyFocus());
 	setItemSelected(2, configmenu->screen->isSemiSloppyFocus());
 	setItemEnabled(3, (configmenu->screen->isSloppyFocus() ||
-						configmenu->screen->isSemiSloppyFocus()));
+		configmenu->screen->isSemiSloppyFocus()));
 	setItemSelected(3, configmenu->screen->doAutoRaise());
 }
 
 
-void Configmenu::Focusmenu::itemSelected(int button, int index) {
+void Configmenu::Focusmenu::itemSelected(int button, unsigned int index) {
 	if (button == 1) {
 		BasemenuItem *item = find(index);
 
@@ -366,85 +305,53 @@ void Configmenu::Focusmenu::itemSelected(int button, int index) {
 Configmenu::Placementmenu::Placementmenu(Configmenu *cm) : Basemenu(cm->screen) {
 	configmenu = cm;
 	I18n *i18n = I18n::instance();
-
+	using namespace FBNLS;
 	setLabel(i18n->getMessage(
-#ifdef		NLS
-					ConfigmenuSet, ConfigmenuWindowPlacement,
-#else // !NLS
-					0, 0,
-#endif // NLS
-					"Window Placement"));
+		ConfigmenuSet, ConfigmenuWindowPlacement,
+		"Window Placement"));
 	setInternalMenu();
 
 	insert(i18n->getMessage(
-#ifdef		NLS
-				ConfigmenuSet, ConfigmenuSmartRows,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Smart Placement (Rows)"), BScreen::ROWSMARTPLACEMENT);
+		ConfigmenuSet, ConfigmenuSmartRows,
+		"Smart Placement (Rows)"), BScreen::ROWSMARTPLACEMENT);
 	insert(i18n->getMessage(
-#ifdef		NLS
-				ConfigmenuSet, ConfigmenuSmartCols,
-#else // !NLS
-				0, 0,
-#endif // NLS
-			"Smart Placement (Columns)"), BScreen::COLSMARTPLACEMENT);
+		ConfigmenuSet, ConfigmenuSmartCols,
+		"Smart Placement (Columns)"), BScreen::COLSMARTPLACEMENT);
 	insert(i18n->getMessage(
-#ifdef		NLS
-				ConfigmenuSet, ConfigmenuCascade,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Cascade Placement"), BScreen::CASCADEPLACEMENT);
+		ConfigmenuSet, ConfigmenuCascade,
+		"Cascade Placement"), BScreen::CASCADEPLACEMENT);
 	insert(i18n->getMessage(
-#ifdef		NLS
-				ConfigmenuSet, ConfigmenuLeftRight,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Left to Right"), BScreen::LEFTRIGHT);
+		ConfigmenuSet, ConfigmenuLeftRight,
+		"Left to Right"), BScreen::LEFTRIGHT);
 	insert(i18n->getMessage(
-#ifdef		NLS
-				ConfigmenuSet, ConfigmenuRightLeft,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Right to Left"), BScreen::RIGHTLEFT);
+		ConfigmenuSet, ConfigmenuRightLeft,
+		"Right to Left"), BScreen::RIGHTLEFT);
 	insert(i18n->getMessage(
-#ifdef		NLS
-				ConfigmenuSet, ConfigmenuTopBottom,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Top to Bottom"), BScreen::TOPBOTTOM);
+		ConfigmenuSet, ConfigmenuTopBottom,
+		"Top to Bottom"), BScreen::TOPBOTTOM);
 	insert(i18n->getMessage(
-#ifdef		NLS
-				ConfigmenuSet, ConfigmenuBottomTop,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Bottom to Top"), BScreen::BOTTOMTOP);
+		ConfigmenuSet, ConfigmenuBottomTop,
+		"Bottom to Top"), BScreen::BOTTOMTOP);
 
 	update();
 
 	switch (configmenu->screen->getPlacementPolicy()) {
 	case BScreen::ROWSMARTPLACEMENT:
-		setItemSelected(0, True);
+		setItemSelected(0, true);
 		break;
 
 	case BScreen::COLSMARTPLACEMENT:
-		setItemSelected(1, True);
+		setItemSelected(1, true);
 		break;
 
 	case BScreen::CASCADEPLACEMENT:
-		setItemSelected(2, True);
+		setItemSelected(2, true);
 		break;
 	}
 
-	Bool rl = (configmenu->screen->getRowPlacementDirection() ==
+	bool rl = (configmenu->screen->getRowPlacementDirection() ==
 				BScreen::LEFTRIGHT),
-				tb = (configmenu->screen->getColPlacementDirection() ==
+		tb = (configmenu->screen->getColPlacementDirection() ==
 				BScreen::TOPBOTTOM);
 
 	setItemSelected(3, rl);
@@ -455,7 +362,7 @@ Configmenu::Placementmenu::Placementmenu(Configmenu *cm) : Basemenu(cm->screen)
 }
 
 
-void Configmenu::Placementmenu::itemSelected(int button, int index) {
+void Configmenu::Placementmenu::itemSelected(int button, unsigned int index) {
 	if (button == 1) {
 		BasemenuItem *item = find(index);
 
@@ -464,61 +371,61 @@ void Configmenu::Placementmenu::itemSelected(int button, int index) {
 			case BScreen::ROWSMARTPLACEMENT:
 				configmenu->screen->savePlacementPolicy(item->function());
 
-				setItemSelected(0, True);
-				setItemSelected(1, False);
-				setItemSelected(2, False);
+				setItemSelected(0, true);
+				setItemSelected(1, false);
+				setItemSelected(2, false);
 				
 				break;
 
 			case BScreen::COLSMARTPLACEMENT:
 				configmenu->screen->savePlacementPolicy(item->function());
 
-				setItemSelected(0, False);
-				setItemSelected(1, True);
-				setItemSelected(2, False);
+				setItemSelected(0, false);
+				setItemSelected(1, true);
+				setItemSelected(2, false);
 
 				break;
 
 			case BScreen::CASCADEPLACEMENT:
 				configmenu->screen->savePlacementPolicy(item->function());
 
-				setItemSelected(0, False);
-				setItemSelected(1, False);
-				setItemSelected(2, True);
+				setItemSelected(0, false);
+				setItemSelected(1, false);
+				setItemSelected(2, true);
 
 				break;
 
 			case BScreen::LEFTRIGHT:
 				configmenu->screen->saveRowPlacementDirection(BScreen::LEFTRIGHT);
 
-				setItemSelected(3, True);
-				setItemSelected(4, False);
+				setItemSelected(3, true);
+				setItemSelected(4, false);
 
 				break;
 
 			case BScreen::RIGHTLEFT:
 				configmenu->screen->saveRowPlacementDirection(BScreen::RIGHTLEFT);
 
-				setItemSelected(3, False);
-				setItemSelected(4, True);
+				setItemSelected(3, false);
+				setItemSelected(4, true);
 
-	break;
+			break;
 
 			case BScreen::TOPBOTTOM:
-	configmenu->screen->saveColPlacementDirection(BScreen::TOPBOTTOM);
+			configmenu->screen->saveColPlacementDirection(BScreen::TOPBOTTOM);
 
-	setItemSelected(5, True);
-	setItemSelected(6, False);
+			setItemSelected(5, true);
+			setItemSelected(6, false);
 
-	break;
+			break;
 
 			case BScreen::BOTTOMTOP:
-	configmenu->screen->saveColPlacementDirection(BScreen::BOTTOMTOP);
+			configmenu->screen->saveColPlacementDirection(BScreen::BOTTOMTOP);
 
-	setItemSelected(5, False);
-	setItemSelected(6, True);
+			setItemSelected(5, false);
+			setItemSelected(6, true);
 
-	break;
+			break;
 			}
 		}
 	}
@@ -527,143 +434,74 @@ void Configmenu::Placementmenu::itemSelected(int button, int index) {
 Configmenu::Tabmenu::Tabmenu(Configmenu *cm) : Basemenu(cm->screen) {
 	configmenu = cm;
 	I18n *i18n = I18n::instance();
+	using namespace FBNLS;
 
 	setLabel(i18n->getMessage(
-#ifdef		NLS
-					ConfigmenuSet, ConfigmenuTabPlacement,
-#else // !NLS
-					0, 0,
-#endif // NLS
-					"Tab Placement"));
+		ConfigmenuSet, ConfigmenuTabPlacement,
+		"Tab Placement"));
 	setInternalMenu();
 
 	insert(i18n->getMessage(
-#ifdef		NLS
-				CommonSet, CommonPlacementTopLeft,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Top Left"), Tab::PTOP + Tab::ALEFT);
+		CommonSet, CommonPlacementTopLeft,
+		"Top Left"), Tab::PTOP + Tab::ALEFT);
 	insert(i18n->getMessage(
-#ifdef		NLS
-				CommonSet, CommonPlacementTopCenter,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Top Center"), Tab::PTOP + Tab::ACENTER);
+		CommonSet, CommonPlacementTopCenter,
+		"Top Center"), Tab::PTOP + Tab::ACENTER);
 	insert(i18n->getMessage(
-#ifdef		NLS
-				CommonSet, CommonPlacementTopRight,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Top Right"), Tab::PTOP + Tab::ARIGHT);
+		CommonSet, CommonPlacementTopRight,
+		"Top Right"), Tab::PTOP + Tab::ARIGHT);
 	insert(i18n->getMessage(
-#ifdef		NLS
-				CommonSet, CommonPlacementTopRelative,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Top Relative"), Tab::PTOP + Tab::ARELATIVE);
+		CommonSet, CommonPlacementTopRelative,
+		"Top Relative"), Tab::PTOP + Tab::ARELATIVE);
 	insert(i18n->getMessage(
-#ifdef		NLS
-				CommonSet, CommonPlacementBottomLeft,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Bottom Left"), Tab::PBOTTOM + Tab::ALEFT);
+		CommonSet, CommonPlacementBottomLeft,
+		"Bottom Left"), Tab::PBOTTOM + Tab::ALEFT);
 	insert(i18n->getMessage(
-#ifdef		NLS
-				CommonSet, CommonPlacementBottomCenter,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Bottom Center"), Tab::PBOTTOM + Tab::ACENTER);
+		CommonSet, CommonPlacementBottomCenter,
+		"Bottom Center"), Tab::PBOTTOM + Tab::ACENTER);
 	insert(i18n->getMessage(
-#ifdef		NLS
-				CommonSet, CommonPlacementBottomRight,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Bottom Right"), Tab::PBOTTOM + Tab::ARIGHT);
+		CommonSet, CommonPlacementBottomRight,
+		"Bottom Right"), Tab::PBOTTOM + Tab::ARIGHT);
 	insert(i18n->getMessage(
-#ifdef		NLS
-				CommonSet, CommonPlacementBottomRelative,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Bottom Relative"), Tab::PBOTTOM + Tab::ARELATIVE);
+		CommonSet, CommonPlacementBottomRelative,
+		"Bottom Relative"), Tab::PBOTTOM + Tab::ARELATIVE);
 	insert(i18n->getMessage(
-#ifdef		NLS
-				CommonSet, CommonPlacementLeftTop,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Left Top"), Tab::PLEFT + Tab::ARIGHT);
+		CommonSet, CommonPlacementLeftTop,
+		"Left Top"), Tab::PLEFT + Tab::ARIGHT);
 	insert(i18n->getMessage(
-#ifdef	NLS
-				CommonSet, CommonPlacementLeftCenter,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Left Center"), Tab::PLEFT + Tab::ACENTER);
+		CommonSet, CommonPlacementLeftCenter,
+		"Left Center"), Tab::PLEFT + Tab::ACENTER);
 	insert(i18n->getMessage(
-#ifdef		NLS
-				CommonSet, CommonPlacementLeftBottom,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Left Bottom"), Tab::PLEFT + Tab::ALEFT);
+		CommonSet, CommonPlacementLeftBottom,
+		"Left Bottom"), Tab::PLEFT + Tab::ALEFT);
 	insert(i18n->getMessage(
-#ifdef		NLS
-				CommonSet, CommonPlacementLeftRelative,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Left Relative"), Tab::PLEFT + Tab::ARELATIVE);
+		CommonSet, CommonPlacementLeftRelative,
+		"Left Relative"), Tab::PLEFT + Tab::ARELATIVE);
 	insert(i18n->getMessage(
-#ifdef		NLS
-				CommonSet, CommonPlacementRightTop,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Right Top"), Tab::PRIGHT + Tab::ARIGHT);
+		CommonSet, CommonPlacementRightTop,
+		"Right Top"), Tab::PRIGHT + Tab::ARIGHT);
 	insert(i18n->getMessage(
-#ifdef		NLS
-				CommonSet, CommonPlacementRightCenter,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Right Center"), Tab::PRIGHT + Tab::ACENTER);
+		CommonSet, CommonPlacementRightCenter,
+		"Right Center"), Tab::PRIGHT + Tab::ACENTER);
 	insert(i18n->getMessage(
-#ifdef		NLS
-				CommonSet, CommonPlacementRightBottom,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Right Bottom"), Tab::PRIGHT + Tab::ALEFT);
+		CommonSet, CommonPlacementRightBottom,
+		"Right Bottom"), Tab::PRIGHT + Tab::ALEFT);
 	insert(i18n->getMessage(
-#ifdef		NLS
-				CommonSet, CommonPlacementRightRelative,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Right Relative"), Tab::PRIGHT + Tab::ARELATIVE);
+		CommonSet, CommonPlacementRightRelative,
+		"Right Relative"), Tab::PRIGHT + Tab::ARELATIVE);
 	insert(i18n->getMessage(
-#ifdef		NLS
-				ConfigmenuSet, ConfigmenuTabRotateVertical,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Rotate Vertical Tabs"), CMENU_TAB_ROTATE);
+		ConfigmenuSet, ConfigmenuTabRotateVertical,
+		"Rotate Vertical Tabs"), CMENU_TAB_ROTATE);
 	update();
 	setSelected();
 }
 
 
-void Configmenu::Tabmenu::itemSelected(int button, int index) {
+void Configmenu::Tabmenu::itemSelected(int button, unsigned int index) {
 	if (button == 1) {
+		#ifdef DEBUG
 		fprintf(stderr, "Item(%d)\n", index);
+		#endif
 		BasemenuItem *item = find(index);
 		
 		switch (item->function()) {
@@ -808,40 +646,40 @@ void Configmenu::Tabmenu::itemSelected(int button, int index) {
 
 void Configmenu::Tabmenu::setSelected(void) {
 	setItemSelected(0, (configmenu->screen->getTabPlacement() == Tab::PTOP &&
-						configmenu->screen->getTabAlignment() == Tab::ALEFT));
+		configmenu->screen->getTabAlignment() == Tab::ALEFT));
 	setItemSelected(1, (configmenu->screen->getTabPlacement() == Tab::PTOP &&
-						configmenu->screen->getTabAlignment() == Tab::ACENTER));
+		configmenu->screen->getTabAlignment() == Tab::ACENTER));
 	setItemSelected(2, (configmenu->screen->getTabPlacement() == Tab::PTOP &&
-						configmenu->screen->getTabAlignment() == Tab::ARIGHT));
+		configmenu->screen->getTabAlignment() == Tab::ARIGHT));
 	setItemSelected(3, (configmenu->screen->getTabPlacement() == Tab::PTOP &&
-						configmenu->screen->getTabAlignment() == Tab::ARELATIVE));
+		configmenu->screen->getTabAlignment() == Tab::ARELATIVE));
 	setItemSelected(4, (configmenu->screen->getTabPlacement() == Tab::PBOTTOM &&
-						configmenu->screen->getTabAlignment() == Tab::ALEFT));
+		configmenu->screen->getTabAlignment() == Tab::ALEFT));
 	setItemSelected(5, (configmenu->screen->getTabPlacement() == Tab::PBOTTOM &&
-						configmenu->screen->getTabAlignment() == Tab::ACENTER));
+		configmenu->screen->getTabAlignment() == Tab::ACENTER));
 	setItemSelected(6, (configmenu->screen->getTabPlacement() == Tab::PBOTTOM &&
-						configmenu->screen->getTabAlignment() == Tab::ARIGHT));
+		configmenu->screen->getTabAlignment() == Tab::ARIGHT));
 	setItemSelected(7, (configmenu->screen->getTabPlacement() == Tab::PBOTTOM &&
-						configmenu->screen->getTabAlignment() == Tab::ARELATIVE));
+		configmenu->screen->getTabAlignment() == Tab::ARELATIVE));
 	setItemSelected(8, (configmenu->screen->getTabPlacement() == Tab::PLEFT &&
-						configmenu->screen->getTabAlignment() == Tab::ARIGHT));
+		configmenu->screen->getTabAlignment() == Tab::ARIGHT));
 	setItemSelected(9, (configmenu->screen->getTabPlacement() == Tab::PLEFT &&
-						configmenu->screen->getTabAlignment() == Tab::ACENTER));
+		configmenu->screen->getTabAlignment() == Tab::ACENTER));
 	setItemSelected(10, (configmenu->screen->getTabPlacement() == Tab::PLEFT &&
-						configmenu->screen->getTabAlignment() == Tab::ALEFT));
+		configmenu->screen->getTabAlignment() == Tab::ALEFT));
 	setItemSelected(11, (configmenu->screen->getTabPlacement() == Tab::PLEFT &&
-						configmenu->screen->getTabAlignment() == Tab::ARELATIVE));
+		configmenu->screen->getTabAlignment() == Tab::ARELATIVE));
 	setItemSelected(12, (configmenu->screen->getTabPlacement() == Tab::PRIGHT &&
-						configmenu->screen->getTabAlignment() == Tab::ARIGHT));
+		configmenu->screen->getTabAlignment() == Tab::ARIGHT));
 	setItemSelected(13, (configmenu->screen->getTabPlacement() == Tab::PRIGHT &&
-						configmenu->screen->getTabAlignment() == Tab::ACENTER));
+		configmenu->screen->getTabAlignment() == Tab::ACENTER));
 	setItemSelected(14, (configmenu->screen->getTabPlacement() == Tab::PRIGHT &&
-						configmenu->screen->getTabAlignment() == Tab::ALEFT));
+		configmenu->screen->getTabAlignment() == Tab::ALEFT));
 	setItemSelected(15, (configmenu->screen->getTabPlacement() == Tab::PRIGHT &&
-						configmenu->screen->getTabAlignment() == Tab::ARELATIVE));
+		configmenu->screen->getTabAlignment() == Tab::ARELATIVE));
 	setItemEnabled(16, (configmenu->screen->getTabPlacement() == Tab::PLEFT ||
-						configmenu->screen->getTabPlacement() == Tab::PRIGHT));
+		configmenu->screen->getTabPlacement() == Tab::PRIGHT));
 	setItemSelected(16, (configmenu->screen->isTabRotateVertical() &&
-						(configmenu->screen->getTabPlacement() == Tab::PLEFT ||
-						configmenu->screen->getTabPlacement() == Tab::PRIGHT))); 
+		(configmenu->screen->getTabPlacement() == Tab::PLEFT ||
+		configmenu->screen->getTabPlacement() == Tab::PRIGHT))); 
 }
