@@ -19,7 +19,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-//$Id: Keys.cc,v 1.12 2002/02/26 22:19:43 fluxgen Exp $
+//$Id: Keys.cc,v 1.15 2002/04/19 09:33:42 fluxgen Exp $
 
 #ifdef		HAVE_CONFIG_H
 #	 include "config.h"
@@ -123,6 +123,8 @@ Keys::t_actionstr Keys::m_actionlist[] = {
 	{"HorizontalDecrement", HORIZDEC},
 	{"VerticalDecrement", VERTDEC},
 	{"ToggleDecor", TOGGLEDECOR},	
+	{"ToggleTab", TOGGLETAB}, 
+	{"RootMenu", ROOTMENU},
 	{0, LASTKEYGRAB}
 	};	
 
@@ -286,6 +288,13 @@ bool Keys::load(char *filename) {
 							else
 								last_key->param = 0;
 						break;
+						case NEXTWINDOW:
+                        case PREVWINDOW:
+							if (argc + 1 < val.size())
+								last_key->param = atoi( val[argc+1].c_str());
+							else
+								last_key->param = 0;
+						break;
 						case LEFTWORKSPACE:
 						case RIGHTWORKSPACE:
 						case NEXTWORKSPACE:
