@@ -22,10 +22,9 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: Screen.cc,v 1.33 2002/02/28 15:46:01 fluxgen Exp $
+// $Id: Screen.cc,v 1.46 2002/04/12 14:56:15 fluxgen Exp $
 
-// stupid macros needed to access some functions in version 2 of the GNU C
-// library
+//use GNU extensions
 #ifndef	 _GNU_SOURCE
 #define	 _GNU_SOURCE
 #endif // _GNU_SOURCE
@@ -99,22 +98,18 @@
 
 using namespace std;
 
-static Bool running = True;
+static bool running = true;
 
 static int anotherWMRunning(Display *display, XErrorEvent *) {
 	fprintf(stderr,
 		I18n::instance()->
 		getMessage(
-#ifdef		NLS
-			ScreenSet, ScreenAnotherWMRunning,
-#else // !NLS
-			0, 0,
-#endif // NLS
+			FBNLS::ScreenSet, FBNLS::ScreenAnotherWMRunning,
 			"BScreen::BScreen: an error occured while querying the X server.\n"
 			"	another window manager already running on display %s.\n"),
 			DisplayString(display));
 
-	running = False;
+	running = false;
 
 	return(-1);
 }
@@ -206,6 +201,7 @@ full_max(rm, true, scrname+".fullMaximization", altscrname+".FullMaximization"),
 max_over_slit(rm, true, scrname+".maxOverSlit",altscrname+".MaxOverSlit"),
 tab_rotate_vertical(rm, true, scrname+".tab.rotatevertical", altscrname+".Tab.RotateVertical"),
 sloppy_window_grouping(rm, true, scrname+".sloppywindowgrouping", altscrname+".SloppyWindowGrouping"),
+workspace_warping(rm, true, scrname+".workspacewarping", altscrname+".WorkspaceWarping"),
 focus_last(rm, true, scrname+".focusLastWindow", altscrname+".FocusLastWindow"),
 focus_new(rm, true, scrname+".focusNewWindows", altscrname+".FocusNewWindows"),
 rootcommand(rm, "", scrname+".rootCommand", altscrname+".RootCommand"),
@@ -216,6 +212,9 @@ tab_width(rm, 64, scrname+".tab.width", altscrname+".Tab.Width"),
 tab_height(rm, 16, scrname+".tab.height", altscrname+".Tab.Height"),
 tab_placement(rm, Tab::PTOP, scrname+".tab.placement", altscrname+".Tab.Placement"),
 tab_alignment(rm, Tab::ALEFT, scrname+".tab.alignment", altscrname+".Tab.Alignment"),
+#ifdef XINERAMA
+toolbar_on_head(rm, 0, scrname+".toolbar.onhead", altscrname+".Toolbar.onHead"),
+#endif // XINERAMA
 toolbar_placement(rm, Toolbar::BOTTOMCENTER, scrname+".toolbar.placement", altscrname+".Toolbar.Placement")
 {
 
@@ -250,15 +249,11 @@ resource(rm, screenname, altscreenname)
 	fprintf(stderr,
 		i18n->
 		getMessage(
-		#ifdef NLS
-				ScreenSet, ScreenManagingScreen,
-		#else // !NLS
-				0, 0,
-		#endif // NLS
-				"BScreen::BScreen: managing screen %d "
-				"using visual 0x%lx, depth %d\n"),
-		getScreenNumber(), XVisualIDFromVisual(getVisual()),
-		getDepth());
+			FBNLS::ScreenSet, FBNLS::ScreenManagingScreen,
+			"BScreen::BScreen: managing screen %d "
+			"using visual 0x%lx, depth %d\n"),
+			getScreenNumber(), XVisualIDFromVisual(getVisual()),
+			getDepth());
 
 	rootmenu = 0;
 		
@@ -280,10 +275,10 @@ resource(rm, screenname, altscreenname)
 		fluxbox->getSessionCursor());
 
 	image_control =
-		new BImageControl(fluxbox, this, True, fluxbox->getColorsPerChannel(),
+		new BImageControl(fluxbox, this, true, fluxbox->getColorsPerChannel(),
 			fluxbox->getCacheLife(), fluxbox->getCacheMax());
 	image_control->installRootColormap();
-	root_colormap_installed = True;
+	root_colormap_installed = true;
 
 	fluxbox->load_rc(this);
 
@@ -309,11 +304,7 @@ resource(rm, screenname, altscreenname)
 	#endif //!NEWWMSPEC
 
 	const char *s =	i18n->getMessage(
-	#ifdef		NLS
-		ScreenSet, ScreenPositionLength,
-	#else // !NLS
-		0, 0,
-	#endif // NLS
+		FBNLS::ScreenSet, FBNLS::ScreenPositionLength,
 		"0: 0000 x 0: 0000");
 	
 	int l = strlen(s);
@@ -338,13 +329,13 @@ resource(rm, screenname, altscreenname)
 	unsigned long mask = CWBorderPixel | CWColormap | CWSaveUnder;
 	attrib.border_pixel = getBorderColor()->getPixel();
 	attrib.colormap = getColormap();
-	attrib.save_under = True;
+	attrib.save_under = true;
 
 	geom_window =
 		XCreateWindow(getBaseDisplay()->getXDisplay(), getRootWindow(),
 			0, 0, geom_w, geom_h, theme->getBorderWidth(), getDepth(),
 			InputOutput, getVisual(), mask, &attrib);
-	geom_visible = False;
+	geom_visible = false;
 
 	if (theme->getWindowStyle().l_focus.getTexture() & BImage::PARENTRELATIVE) {
 		if (theme->getWindowStyle().t_focus.getTexture() ==
@@ -381,27 +372,23 @@ resource(rm, screenname, altscreenname)
 		for (int i = 0; i < *resource.workspaces; ++i) {
 			wkspc = new Workspace(this, workspacesList.size());
 			workspacesList.push_back(wkspc);
-			workspacemenu->insert(wkspc->getName(), wkspc->getMenu());
+			workspacemenu->insert(wkspc->name().c_str(), wkspc->menu());
 		}
 	} else {
 		wkspc = new Workspace(this, workspacesList.size());
 		workspacesList.push_back(wkspc);
-		workspacemenu->insert(wkspc->getName(), wkspc->getMenu());
+		workspacemenu->insert(wkspc->name().c_str(), wkspc->menu());
 	}
 
 	workspacemenu->insert(i18n->
 		getMessage(
-		#ifdef NLS
-			 IconSet, IconIcons,
-		#else // !NLS
-			 0, 0,
-		#endif // NLS
+			FBNLS::IconSet, FBNLS::IconIcons,
 			 "Icons"),
 			iconmenu);
 	workspacemenu->update();
 
 	current_workspace = workspacesList.front();
-	workspacemenu->setItemSelected(2, True);
+	workspacemenu->setItemSelected(2, true);
 
 	toolbar = new Toolbar(this);
 
@@ -409,7 +396,7 @@ resource(rm, screenname, altscreenname)
 	slit = new Slit(this);
 	#endif // SLIT
 
-	InitMenu();
+	initMenu();
 
 	raiseWindows(0, 0);
 	rootmenu->update();
@@ -483,10 +470,10 @@ resource(rm, screenname, altscreenname)
 }
 
 namespace {
-    template<typename T>
-    void delete_obj(T * obj) {
-        delete obj;
-    }
+	template<typename T>
+	void delete_obj(T * obj) {
+		delete obj;
+	}
 }
 
 BScreen::~BScreen(void) {
@@ -503,22 +490,22 @@ BScreen::~BScreen(void) {
 
 	removeWorkspaceNames();
 
-    std::for_each(
-        workspacesList.begin(),
-        workspacesList.end(),
-        delete_obj<Workspace>);
+	std::for_each(
+		workspacesList.begin(),
+		workspacesList.end(),
+		delete_obj<Workspace>);
 
-    // don't delete items in the rootmenuList?
+	// don't delete items in the rootmenuList?
 
-    std::for_each(
-        iconList.begin(),
-        iconList.end(),
-        delete_obj<FluxboxWindow>);
+	std::for_each(
+		iconList.begin(),
+		iconList.end(),
+		delete_obj<FluxboxWindow>);
 
-    std::for_each(
-        netizenList.begin(),
-        netizenList.end(),
-        delete_obj<Netizen>);
+	std::for_each(
+		netizenList.begin(),
+		netizenList.end(),
+		delete_obj<Netizen>);
 
 #ifdef		HAVE_STRFTIME
 	if (resource.strftime_format)
@@ -553,12 +540,8 @@ void BScreen::reconfigure(void) {
 	I18n *i18n = I18n::instance();
 
 	const char *s = i18n->getMessage(
-#ifdef		NLS
-			ScreenSet, ScreenPositionLength,
-#else // !NLS
-			0, 0,
-#endif // NLS
-			"0: 0000 x 0: 0000");
+		FBNLS::ScreenSet, FBNLS::ScreenPositionLength,
+		"0: 0000 x 0: 0000");
 	int l = strlen(s);
 
 	if (i18n->multibyte()) {
@@ -614,8 +597,8 @@ void BScreen::reconfigure(void) {
 	iconmenu->reconfigure();
 
 	{
-		int remember_sub = rootmenu->getCurrentSubmenu();
-		InitMenu();
+		int remember_sub = rootmenu->currentSubmenu();
+		initMenu();
 		raiseWindows(0, 0);
 		rootmenu->reconfigure();
 		rootmenu->drawSubmenu(remember_sub);
@@ -647,7 +630,7 @@ void BScreen::reconfigure(void) {
 
 
 void BScreen::rereadMenu(void) {
-	InitMenu();
+	initMenu();
 	raiseWindows(0, 0);
 
 	rootmenu->reconfigure();
@@ -694,7 +677,7 @@ void BScreen::addIcon(FluxboxWindow *w) {
 
 	iconList.push_back(w);
 
-	iconmenu->insert((const char **) w->getIconTitle());	
+	iconmenu->insert(w->getIconTitle().c_str());
 	iconmenu->update();
 	toolbar->addIcon(w);
 }
@@ -704,7 +687,7 @@ void BScreen::removeIcon(FluxboxWindow *w) {
 	if (! w) return;
 	
 	{
-    Icons::iterator it = iconList.begin();
+	Icons::iterator it = iconList.begin();
 	Icons::iterator it_end = iconList.end();
 	for (; it != it_end; ++it) {
 		if (*it == w) {
@@ -718,7 +701,7 @@ void BScreen::removeIcon(FluxboxWindow *w) {
 	iconmenu->update();
 	toolbar->delIcon(w);
 	
-    Icons::iterator it = iconList.begin();
+	Icons::iterator it = iconList.begin();
 	Icons::iterator it_end = iconList.end();
 	for (int i = 0; it != it_end; ++it, ++i) {
 		(*it)->setWindowNumber(i);
@@ -726,11 +709,11 @@ void BScreen::removeIcon(FluxboxWindow *w) {
 }
 
 
-FluxboxWindow *BScreen::getIcon(int index) {
-	if (index >= 0 && index < iconList.size())
+FluxboxWindow *BScreen::getIcon(unsigned int index) {
+	if (index < iconList.size())
 		return iconList[index];
 
-	return (FluxboxWindow *) 0;
+	return 0;
 }
 
 
@@ -738,8 +721,8 @@ int BScreen::addWorkspace(void) {
 	Workspace *wkspc = new Workspace(this, workspacesList.size());
 	workspacesList.push_back(wkspc);
 	//add workspace to workspacemenu
-	workspacemenu->insert(wkspc->getName(), wkspc->getMenu(),
-		wkspc->getWorkspaceID() + 2); //+2 so we add it after "remove last"
+	workspacemenu->insert(wkspc->name().c_str(), wkspc->menu(),
+		wkspc->workspaceID() + 2); //+2 so we add it after "remove last"
 		
 	workspacemenu->update();
 	saveWorkspaces(workspacesList.size());
@@ -757,12 +740,12 @@ int BScreen::removeLastWorkspace(void) {
 	if (workspacesList.size() > 1) {
 		Workspace *wkspc = workspacesList.back();
 
-		if (current_workspace->getWorkspaceID() == wkspc->getWorkspaceID())
-			changeWorkspaceID(current_workspace->getWorkspaceID() - 1);
+		if (current_workspace->workspaceID() == wkspc->workspaceID())
+			changeWorkspaceID(current_workspace->workspaceID() - 1);
 
 		wkspc->removeAll();
 
-		workspacemenu->remove(wkspc->getWorkspaceID()+2); // + 2 is where workspaces starts
+		workspacemenu->remove(wkspc->workspaceID()+2); // + 2 is where workspaces starts
 		workspacemenu->update();
 		
 		//remove last workspace
@@ -780,17 +763,16 @@ int BScreen::removeLastWorkspace(void) {
 }
 
 
-void BScreen::changeWorkspaceID(int id) {
-	if (! current_workspace || id >= workspacesList.size() || id < 0)
+void BScreen::changeWorkspaceID(unsigned int id) {
+	if (! current_workspace || id >= workspacesList.size())
 		return;
 	
-	if (id != current_workspace->getWorkspaceID()) {
-		XSync(fluxbox->getXDisplay(), True);
+	if (id != current_workspace->workspaceID()) {
+		XSync(fluxbox->getXDisplay(), true);
 		
 		current_workspace->hideAll();
 
-		workspacemenu->setItemSelected(current_workspace->getWorkspaceID() + 2,
-					 False);
+		workspacemenu->setItemSelected(current_workspace->workspaceID() + 2, false);
 
 		if (fluxbox->getFocusedWindow() &&
 				fluxbox->getFocusedWindow()->getScreen() == this &&
@@ -803,9 +785,8 @@ void BScreen::changeWorkspaceID(int id) {
 
 		current_workspace = getWorkspace(id);
 
-		workspacemenu->setItemSelected(current_workspace->getWorkspaceID() + 2,
-					 True);
-		toolbar->redrawWorkspaceLabel(True);
+		workspacemenu->setItemSelected(current_workspace->workspaceID() + 2, true);
+		toolbar->redrawWorkspaceLabel(true);
 
 		current_workspace->showAll();
 
@@ -817,16 +798,13 @@ void BScreen::changeWorkspaceID(int id) {
 	updateNetizenCurrentWorkspace();
 }
 
-void BScreen::sendToWorkspace(int id) {
-	 BScreen::sendToWorkspace(id, true);
-}
 
-void BScreen::sendToWorkspace(int id, bool changeWS) {
+void BScreen::sendToWorkspace(unsigned int id, bool changeWS) {
 	FluxboxWindow *win;
-	if (! current_workspace || id >= workspacesList.size() || id < 0)
+	if (! current_workspace || id >= workspacesList.size())
 		return;
 
-	if (id != current_workspace->getWorkspaceID()) {
+	if (id != current_workspace->workspaceID()) {
 		XSync(fluxbox->getXDisplay(), True);
 
 		win = fluxbox->getFocusedWindow();
@@ -841,7 +819,7 @@ void BScreen::sendToWorkspace(int id, bool changeWS) {
 					}
 
 					win->withdraw();
-					BScreen::reassociateWindow(win, id, True);
+					BScreen::reassociateWindow(win, id, true);
 					if (changeWS) {
 						BScreen::changeWorkspaceID(id);
 						win->setInputFocus();
@@ -863,12 +841,12 @@ void BScreen::addNetizen(Netizen *n) {
 	for (; it != it_end; ++it) {
 		for (int i = 0; i < (*it)->getCount(); ++i) {
 			n->sendWindowAdd((*it)->getWindow(i)->getClientWindow(),
-											 (*it)->getWorkspaceID());
+				(*it)->workspaceID());
 		}
 	}
 
 	Window f = ((fluxbox->getFocusedWindow()) ?
-							fluxbox->getFocusedWindow()->getClientWindow() : None);
+		fluxbox->getFocusedWindow()->getClientWindow() : None);
 	n->sendWindowFocus(f);
 }
 
@@ -877,8 +855,9 @@ void BScreen::removeNetizen(Window w) {
 	Netizens::iterator it_end = netizenList.end();
 	for (; it != it_end; ++it) {
 		if ((*it)->getWindowID() == w) {
-			Netizen *n = *netizenList.erase(it);
+			Netizen *n = *it;
 			delete n;
+			netizenList.erase(it);			
 			break;
 		}
 	}
@@ -942,7 +921,7 @@ void BScreen::updateNetizenWindowFocus(void) {
 
 	Netizens::iterator it = netizenList.begin();
 	Netizens::iterator it_end = netizenList.end();
-  	Window f = ((fluxbox->getFocusedWindow()) ?
+	Window f = ((fluxbox->getFocusedWindow()) ?
 			fluxbox->getFocusedWindow()->getClientWindow() : None);
 	for (; it != it_end; ++it) {
 		(*it)->sendWindowFocus(f);
@@ -1006,38 +985,48 @@ void BScreen::raiseWindows(Window *workspace_stack, int num) {
 
 	Window session_stack[(num + workspacesList.size() + rootmenuList.size() + 30)];
 	int i = 0;	
-	XRaiseWindow(getBaseDisplay()->getXDisplay(), iconmenu->getWindowID());
-	session_stack[i++] = iconmenu->getWindowID();
+	XRaiseWindow(getBaseDisplay()->getXDisplay(), iconmenu->windowID());
+	session_stack[i++] = iconmenu->windowID();
 
 	Workspaces::iterator wit = workspacesList.begin();
 	Workspaces::iterator wit_end = workspacesList.end();
 	for (; wit != wit_end; ++wit) {
-		session_stack[i++] = (*wit)->getMenu()->getWindowID();
+		session_stack[i++] = (*wit)->menu()->windowID();
 	}
 
-	session_stack[i++] = workspacemenu->getWindowID();
+	session_stack[i++] = workspacemenu->windowID();
 
-	session_stack[i++] = configmenu->getFocusmenu()->getWindowID();
-	session_stack[i++] = configmenu->getPlacementmenu()->getWindowID();
-	session_stack[i++] = configmenu->getTabmenu()->getWindowID();
-	session_stack[i++] = configmenu->getWindowID();
+	session_stack[i++] = configmenu->getFocusmenu()->windowID();
+	session_stack[i++] = configmenu->getPlacementmenu()->windowID();
+	session_stack[i++] = configmenu->getTabmenu()->windowID();
+	session_stack[i++] = configmenu->windowID();
 
 	#ifdef		SLIT
-	session_stack[i++] = slit->getMenu()->getDirectionmenu()->getWindowID();
-	session_stack[i++] = slit->getMenu()->getPlacementmenu()->getWindowID();
-	session_stack[i++] = slit->getMenu()->getWindowID();
+	session_stack[i++] = slit->getMenu().getDirectionmenu()->windowID();
+	session_stack[i++] = slit->getMenu().getPlacementmenu()->windowID();
+	#ifdef XINERAMA
+	if (hasXinerama()) {
+		session_stack[i++] = slit->getMenu().getHeadmenu()->windowID();
+	}
+	#endif // XINERAMA
+	session_stack[i++] = slit->getMenu().windowID();
 	#endif // SLIT
 
 	session_stack[i++] =
-		toolbar->getMenu()->getPlacementmenu()->getWindowID();
-	session_stack[i++] = toolbar->getMenu()->getWindowID();
+		toolbar->getMenu()->getPlacementmenu()->windowID();
+	#ifdef XINERAMA
+	if (hasXinerama()) {
+		session_stack[i++] = toolbar->getMenu()->getHeadmenu()->windowID();
+	}
+	#endif // XINERAMA
+	session_stack[i++] = toolbar->getMenu()->windowID();
 
 	Rootmenus::iterator rit = rootmenuList.begin();
 	Rootmenus::iterator rit_end = rootmenuList.end();
 	for (; rit != rit_end; ++rit) {
-		session_stack[i++] = (*rit)->getWindowID();
+		session_stack[i++] = (*rit)->windowID();
 	}
-	session_stack[i++] = rootmenu->getWindowID();
+	session_stack[i++] = rootmenu->windowID();
 
 	if (toolbar->isOnTop())
 		session_stack[i++] = toolbar->getWindowID();
@@ -1071,9 +1060,9 @@ void BScreen::addWorkspaceName(char *name) {
 }
 
 
-void BScreen::getNameOfWorkspace(int id, char **name) {
-	if (id >= 0 && id < workspaceNames.size()) {
-		const char *wkspc_name = workspaceNames[id].c_str();
+void BScreen::getNameOfWorkspace(unsigned int workspace, char **name) {
+	if (workspace < workspaceNames.size()) {
+		const char *wkspc_name = workspaceNames[workspace].c_str();
 
 		if (wkspc_name)
 			*name = StringUtil::strdup(wkspc_name);
@@ -1082,11 +1071,11 @@ void BScreen::getNameOfWorkspace(int id, char **name) {
 }
 
 
-void BScreen::reassociateWindow(FluxboxWindow *w, int wkspc_id, Bool ignore_sticky) {
+void BScreen::reassociateWindow(FluxboxWindow *w, unsigned int wkspc_id, bool ignore_sticky) {
 	if (! w) return;
 
-	if (wkspc_id == -1)
-		wkspc_id = current_workspace->getWorkspaceID();
+	if (wkspc_id >= workspaceNames.size())
+		wkspc_id = current_workspace->workspaceID();
 
 	if (w->getWorkspaceNumber() == wkspc_id)
 		return;
@@ -1101,89 +1090,95 @@ void BScreen::reassociateWindow(FluxboxWindow *w, int wkspc_id, Bool ignore_stic
 }
 
 
-void BScreen::nextFocus(void) {
-	Bool have_focused = False;
+void BScreen::nextFocus(int opts) {
+	bool have_focused = false;
 	int focused_window_number = -1;
-	FluxboxWindow *next;
+	FluxboxWindow *focused;
 	const int num_windows = getCurrentWorkspace()->getCount();
 	
-	if (fluxbox->getFocusedWindow()) {
-		if (fluxbox->getFocusedWindow()->getScreen()->getScreenNumber() ==
+	if ((focused = fluxbox->getFocusedWindow())) {
+		if (focused->getScreen()->getScreenNumber() ==
 			getScreenNumber()) {
-			have_focused = True;
-			focused_window_number = fluxbox->getFocusedWindow()->getWindowNumber();
+			have_focused = true;
+			focused_window_number = focused->getWindowNumber();
 		}
 	}
 
 	if (num_windows > 1 && have_focused) {
-		int next_window_number = focused_window_number;
-	
-		//try to set next window to focus		
+		Workspace *wksp = getCurrentWorkspace();
+		Workspace::Windows wins = wksp->getWindowList();
+		Workspace::Windows::iterator it = wins.begin();
+		for (; *it != focused; ++it);
 		do {
-			if ((++next_window_number) >= num_windows)
-				next_window_number = 0;	
-			next = getCurrentWorkspace()->getWindow(next_window_number);
-		} while ((!next->setInputFocus()) && next_window_number !=
-				 focused_window_number);
-
-		if (next_window_number != focused_window_number) {
-			next->setInputFocus();
-			getCurrentWorkspace()->raiseWindow(next);
-		}
+			++it;
+			if (it == wins.end())
+				it = wins.begin();
+			// see if the window should be skipped
+			if (! (doSkipWindow(*it, opts) || !(*it)->setInputFocus()) )
+				break;
+		} while (*it != focused);
 
+		if (*it != focused)
+			wksp->raiseWindow(*it);
 	} else if (num_windows >= 1) {
-		next = current_workspace->getWindow(0);
-	 	
+		FluxboxWindow *next = current_workspace->getWindow(0);
 		//don't raise next window if input focus fails
 		if (next->setInputFocus())
 			current_workspace->raiseWindow(next);
 	}
+
 }
 
 
-void BScreen::prevFocus(void) {
-	Bool have_focused = False;
+void BScreen::prevFocus(int opts) {
+	bool have_focused = false;
 	int focused_window_number = -1;
-	FluxboxWindow *prev;
-
-	if (fluxbox->getFocusedWindow())
-		if (fluxbox->getFocusedWindow()->getScreen()->getScreenNumber() ==
-	getScreenNumber()) {
-			have_focused = True;
-			focused_window_number = fluxbox->getFocusedWindow()->getWindowNumber();
+	FluxboxWindow *focused;
+	int num_windows = getCurrentWorkspace()->getCount();
+	
+	if ((focused = fluxbox->getFocusedWindow())) {
+		if (focused->getScreen()->getScreenNumber() ==
+				getScreenNumber()) {
+			have_focused = true;
+			focused_window_number = focused->getWindowNumber();
 		}
-
-	if ((getCurrentWorkspace()->getCount() > 1) && have_focused) {
-		int prev_window_number = focused_window_number;
-		do {
-			if ((--prev_window_number) < 0)
-				prev_window_number = getCurrentWorkspace()->getCount() - 1;
-
-			prev = getCurrentWorkspace()->getWindow(prev_window_number);
-		} while ((! prev->setInputFocus()) && (prev_window_number !=
-						 focused_window_number));
-
-		if (prev_window_number != focused_window_number)
-			getCurrentWorkspace()->raiseWindow(prev);
-	} else if (getCurrentWorkspace()->getCount() >= 1) {
-		prev = current_workspace->getWindow(0);
-
-		current_workspace->raiseWindow(prev);
-		prev->setInputFocus();
 	}
+
+	if (num_windows > 1 && have_focused) {
+		Workspace *wksp = getCurrentWorkspace();
+		Workspace::Windows wins = wksp->getWindowList();
+		Workspace::Windows::iterator it = wins.begin();
+		for (; *it != focused; ++it);
+		do {
+			if (it == wins.begin())
+				it = wins.end();
+			--it;
+			// see if the window should be skipped
+			if (! (doSkipWindow(*it, opts) ||	!(*it)->setInputFocus()) )
+				break;
+		} while (*it != focused);
+		if (*it != focused)
+			wksp->raiseWindow(*it);
+	} else if (num_windows >= 1) {
+		FluxboxWindow *next = current_workspace->getWindow(0);
+		//don't raise next window if input focus fails
+		if (next->setInputFocus())
+			current_workspace->raiseWindow(next);
+	}
+
 }
 
 //--------- raiseFocus -----------
 // Raise the current focused window
 //--------------------------------
 void BScreen::raiseFocus(void) {
-	Bool have_focused = False;
+	bool have_focused = false;
 	int focused_window_number = -1;
 
 	if (fluxbox->getFocusedWindow())
 		if (fluxbox->getFocusedWindow()->getScreen()->getScreenNumber() ==
-	getScreenNumber()) {
-			have_focused = True;
+				getScreenNumber()) {
+			have_focused = true;
 			focused_window_number = fluxbox->getFocusedWindow()->getWindowNumber();
 		}
 
@@ -1192,18 +1187,18 @@ void BScreen::raiseFocus(void) {
 			raiseWindow(fluxbox->getFocusedWindow());
 }
 
-void BScreen::InitMenu(void) {
+void BScreen::initMenu(void) {
 	I18n *i18n = I18n::instance();
 	
 	if (rootmenu) {
 		rootmenuList.erase(rootmenuList.begin(), rootmenuList.end());
 
-		while (rootmenu->getCount())
+		while (rootmenu->numberOfItems())
 			rootmenu->remove(0);
 	} else
 		rootmenu = new Rootmenu(this);
 
-	Bool defaultMenu = True;
+	bool defaultMenu = true;
 
 	if (fluxbox->getMenuFilename()) {
 		ifstream menu_file(fluxbox->getMenuFilename());
@@ -1235,15 +1230,11 @@ void BScreen::InitMenu(void) {
 			} else {
 				fprintf(stderr,
 					i18n->getMessage(
-				#ifdef NLS
-						ScreenSet, ScreenEmptyMenuFile,
-				#else // !NLS
-						0, 0,
-				#endif // NLS
+						FBNLS::ScreenSet, FBNLS::ScreenEmptyMenuFile,
 						"%s: Empty menu file"),
 					fluxbox->getMenuFilename());
 			}
-			
+			menu_file.close();
 		} else
 			perror(fluxbox->getMenuFilename());
 	}
@@ -1251,42 +1242,26 @@ void BScreen::InitMenu(void) {
 	if (defaultMenu) {
 		rootmenu->setInternalMenu();
 		rootmenu->insert(i18n->getMessage(
-		#ifdef		NLS
-			ScreenSet, Screenxterm,
-		#else // !NLS
-				0, 0,
-		#endif // NLS
-				"xterm"),
-				BScreen::EXECUTE,
-				i18n->getMessage(
-		#ifdef NLS
-				ScreenSet, Screenxterm,
-		#else // !NLS
-				0, 0,
-		#endif // NLS
-				"xterm"));
+			FBNLS::ScreenSet, FBNLS::Screenxterm,
+			"xterm"),
+			BScreen::EXECUTE,
+			i18n->getMessage(
+			FBNLS::ScreenSet, FBNLS::Screenxterm,
+			"xterm"));
 		rootmenu->insert(i18n->getMessage(
-		#ifdef NLS
-				ScreenSet, ScreenRestart,
-		#else // !NLS
-				0, 0,
-		#endif // NLS
-						"Restart"),
-					BScreen::RESTART);
+			FBNLS::ScreenSet, FBNLS::ScreenRestart,
+			"Restart"),
+			BScreen::RESTART);
 		rootmenu->insert(i18n->getMessage(
-		#ifdef		NLS
-				ScreenSet, ScreenExit,
-		#else // !NLS
-				0, 0,
-		#endif // NLS
-				"Exit"),
-				BScreen::EXIT);
+			FBNLS::ScreenSet, FBNLS::ScreenExit,
+			"Exit"),
+			BScreen::EXIT);
 	} else
 		fluxbox->saveMenuFilename(fluxbox->getMenuFilename());
 }
 
 // looks through a menufile and adds correct items to the root-menu.
-Bool BScreen::parseMenuFile(ifstream &file, Rootmenu *menu, int &row) {
+bool BScreen::parseMenuFile(ifstream &file, Rootmenu *menu, int &row) {
 	
 	string line;
 
@@ -1294,7 +1269,7 @@ Bool BScreen::parseMenuFile(ifstream &file, Rootmenu *menu, int &row) {
 
 		if (getline(file, line)) {
 			row++;
-			if (line[0] != '#') {
+			if (line[0] != '#') { //the line is commented
 				int parse_pos = 0, err = 0;
 
 
@@ -1316,20 +1291,16 @@ Bool BScreen::parseMenuFile(ifstream &file, Rootmenu *menu, int &row) {
 
 				I18n *i18n = I18n::instance();
 				if (str_key == "end") {
-					return ((menu->getCount() == 0) ? True : False);
+					return ((menu->numberOfItems() == 0) ? true : false);
 				} else if (str_key == "nop") { 
 					menu->insert(str_label.c_str());
 				} else if (str_key == "exec") { // exec
 					if (!(str_label.size() && str_cmd.size())) {
 						fprintf(stderr,
 						i18n->getMessage(
-					#ifdef NLS
-						ScreenSet, ScreenEXECError,
-					#else // !NLS
-						0, 0,
-					#endif // NLS
-						"BScreen::parseMenuFile: [exec] error, "
-						"no menu label and/or command defined\n"));
+							FBNLS::ScreenSet, FBNLS::ScreenEXECError,
+							"BScreen::parseMenuFile: [exec] error, "
+							"no menu label and/or command defined\n"));
 						cerr<<"Row: "<<row<<endl;
 					} else
 						menu->insert(str_label.c_str(), BScreen::EXECUTE, str_cmd.c_str());
@@ -1338,13 +1309,9 @@ Bool BScreen::parseMenuFile(ifstream &file, Rootmenu *menu, int &row) {
 					if (!str_label.size()) {
 						fprintf(stderr,
 						i18n->getMessage(
-					#ifdef NLS
-						 ScreenSet, ScreenEXITError,
-					#else // !NLS
-						 0, 0,
-					#endif // NLS
-						 "BScreen::parseMenuFile: [exit] error, "
-						 "no menu label defined\n"));
+							FBNLS::ScreenSet, FBNLS::ScreenEXITError,
+							 "BScreen::parseMenuFile: [exit] error, "
+							 "no menu label defined\n"));
 						 cerr<<"Row: "<<row<<endl;
 					} else
 						menu->insert(str_label.c_str(), BScreen::EXIT);
@@ -1354,13 +1321,9 @@ Bool BScreen::parseMenuFile(ifstream &file, Rootmenu *menu, int &row) {
 						fprintf(stderr,
 						i18n->
 						getMessage(
-				#ifdef NLS
-					 	ScreenSet, ScreenSTYLEError,
-				#else // !NLS
-						 0, 0,
-				#endif // NLS
-					 	"BScreen::parseMenuFile: [style] error, "
-					 	"no menu label and/or filename defined\n"));
+						 	FBNLS::ScreenSet, FBNLS::ScreenSTYLEError,
+						 	"BScreen::parseMenuFile: [style] error, "
+						 	"no menu label and/or filename defined\n"));
 						cerr<<"Row: "<<row<<endl;
 					} else {
 						char *style;
@@ -1377,13 +1340,9 @@ Bool BScreen::parseMenuFile(ifstream &file, Rootmenu *menu, int &row) {
 						fprintf(stderr,
 						i18n->
 						getMessage(
-					#ifdef NLS
-						 ScreenSet, ScreenCONFIGError,
-					#else // !NLS
-						 0, 0,
-					#endif // NLS
-						 "BScreen::parseMenufile: [config] error, "
-						 "no label defined"));
+							FBNLS::ScreenSet, FBNLS::ScreenCONFIGError,
+							 "BScreen::parseMenufile: [config] error, "
+							 "no label defined"));
 						cerr<<"Row: "<<row<<endl;
 					} else
 						menu->insert(str_label.c_str(), configmenu);
@@ -1393,13 +1352,9 @@ Bool BScreen::parseMenuFile(ifstream &file, Rootmenu *menu, int &row) {
 						fprintf(stderr,
 						i18n->
 						getMessage(
-					#ifdef NLS
-						 	ScreenSet, ScreenINCLUDEError,
-					#else // !NLS
-						 	0, 0,
-					#endif // NLS
-				 		"BScreen::parseMenuFile: [include] error, "
-					 	"no filename defined\n"));
+						 	FBNLS::ScreenSet, FBNLS::ScreenINCLUDEError,
+					 		"BScreen::parseMenuFile: [include] error, "
+						 	"no filename defined\n"));
 						cerr<<"Row: "<<row<<endl;
 					} else {	// start of else 'x'
 						char *newfile;
@@ -1417,13 +1372,9 @@ Bool BScreen::parseMenuFile(ifstream &file, Rootmenu *menu, int &row) {
 									fprintf(stderr,
 									i18n->
 									getMessage(
-								#ifdef NLS
-						 			ScreenSet, ScreenINCLUDEErrorReg,
-								#else // !NLS
-						 			0, 0,
-								#endif // NLS
-						 			"BScreen::parseMenuFile: [include] error: "
-						 			"'%s' is not a regular file\n"), newfile);
+							 			FBNLS::ScreenSet, FBNLS::ScreenINCLUDEErrorReg,
+							 			"BScreen::parseMenuFile: [include] error: "
+							 			"'%s' is not a regular file\n"), newfile);
 									cerr<<"Row: "<<row<<endl;
 								}
 								
@@ -1444,13 +1395,9 @@ Bool BScreen::parseMenuFile(ifstream &file, Rootmenu *menu, int &row) {
 						fprintf(stderr,
 						i18n->
 						getMessage(
-					#ifdef		NLS
-				 		ScreenSet, ScreenSUBMENUError,
-					#else // !NLS
-				 		0, 0,
-					#endif // NLS
-				 		"BScreen::parseMenuFile: [submenu] error, "
-				 		"no menu label defined\n"));
+					 		FBNLS::ScreenSet, FBNLS::ScreenSUBMENUError,
+					 		"BScreen::parseMenuFile: [submenu] error, "
+					 		"no menu label defined\n"));
 						cerr<<"Row: "<<row<<endl;
 					} else {
 						Rootmenu *submenu = new Rootmenu(this);
@@ -1460,10 +1407,11 @@ Bool BScreen::parseMenuFile(ifstream &file, Rootmenu *menu, int &row) {
 						else
 							submenu->setLabel(str_label.c_str());
 
-						parseMenuFile(file, submenu, row);
+						parseMenuFile(file, submenu, row);						
 						submenu->update();
 						menu->insert(str_label.c_str(), submenu);
 						rootmenuList.push_back(submenu);
+						
 					}
 				} // end of sub
 				else if (str_key == "restart") {
@@ -1471,13 +1419,9 @@ Bool BScreen::parseMenuFile(ifstream &file, Rootmenu *menu, int &row) {
 						fprintf(stderr,
 						i18n->
 						getMessage(
-						#ifdef		NLS
-				 		ScreenSet, ScreenRESTARTError,
-						#else // !NLS
-				 		0, 0,
-						#endif // NLS
-				 		"BScreen::parseMenuFile: [restart] error, "
-				 		"no menu label defined\n"));
+					 		FBNLS::ScreenSet, FBNLS::ScreenRESTARTError,
+					 		"BScreen::parseMenuFile: [restart] error, "
+					 		"no menu label defined\n"));
 						cerr<<"Row: "<<row<<endl;
 					} else {
 					if (str_cmd.size())
@@ -1491,13 +1435,9 @@ Bool BScreen::parseMenuFile(ifstream &file, Rootmenu *menu, int &row) {
 						fprintf(stderr,
 						i18n->
 						getMessage(
-					#ifdef NLS
-				 		ScreenSet, ScreenRECONFIGError,
-					#else // !NLS
-				 		0, 0,
-					#endif // NLS
-				 		"BScreen::parseMenuFile: [reconfig] error, "
-				 		"no menu label defined\n"));
+					 		FBNLS::ScreenSet, FBNLS::ScreenRECONFIGError,
+					 		"BScreen::parseMenuFile: [reconfig] error, "
+					 		"no menu label defined\n"));
 						cerr<<"Row: "<<row<<endl;
 					} else 
 						menu->insert(str_label.c_str(), BScreen::RECONFIGURE);
@@ -1508,13 +1448,9 @@ Bool BScreen::parseMenuFile(ifstream &file, Rootmenu *menu, int &row) {
 						fprintf(stderr,
 						i18n->
 						getMessage(
-					#ifdef NLS
-						ScreenSet, ScreenSTYLESDIRError,
-					#else // !NLS
-						0, 0,
-					#endif // NLS
-				 		"BScreen::parseMenuFile: [stylesdir/stylesmenu]"
-				 		" error, no directory defined\n"));
+							FBNLS::ScreenSet, FBNLS::ScreenSTYLESDIRError,
+					 		"BScreen::parseMenuFile: [stylesdir/stylesmenu]"
+					 		" error, no directory defined\n"));
 						cerr<<"Row: "<<row<<endl;
 					} else { // else 'y'
 						createStyleMenu(menu, newmenu, str_label.c_str(), (newmenu) ? str_cmd.c_str() : str_label.c_str());						
@@ -1524,11 +1460,7 @@ Bool BScreen::parseMenuFile(ifstream &file, Rootmenu *menu, int &row) {
 					if (!str_label.size()) {
 						fprintf(stderr,
 						i18n->getMessage(
-					#ifdef NLS
-						 ScreenSet, ScreenWORKSPACESError,
-					#else // !NLS
-						 0, 0,
-					#endif // NLS
+						 FBNLS::ScreenSet, FBNLS::ScreenWORKSPACESError,
 						 "BScreen:parseMenuFile: [workspaces] error, "
 						 "no menu label defined\n"));
 						 cerr<<"Row: "<<row<<endl;
@@ -1539,7 +1471,7 @@ Bool BScreen::parseMenuFile(ifstream &file, Rootmenu *menu, int &row) {
 		}
 	}
 
-	return ((menu->getCount() == 0) ? true : false);
+	return ((menu->numberOfItems() == 0) ? true : false);
 }
 
 void BScreen::createStyleMenu(Rootmenu *menu, bool newmenu, const char *label, const char *directory) {
@@ -1600,11 +1532,7 @@ void BScreen::createStyleMenu(Rootmenu *menu, bool newmenu, const char *label, c
 			fprintf(stderr,
 			i18n->
 			getMessage(
-				#ifdef NLS
-				ScreenSet, ScreenSTYLESDIRErrorNotDir,
-				#else // !NLS
-				0, 0,
-				#endif // NLS	
+				FBNLS::ScreenSet, FBNLS::ScreenSTYLESDIRErrorNotDir,
 				"BScreen::parseMenuFile:"
 				" [stylesdir/stylesmenu] error, %s is not a"
 				" directory\n"), stylesdir.get());
@@ -1613,13 +1541,9 @@ void BScreen::createStyleMenu(Rootmenu *menu, bool newmenu, const char *label, c
 		fprintf(stderr,
 		i18n->
 		getMessage(
-		#ifdef NLS
-			ScreenSet, ScreenSTYLESDIRErrorNoExist,
-		#else // !NLS
-			0, 0,
-		#endif // NLS
-		"BScreen::parseMenuFile: [stylesdir/stylesmenu]"
-		" error, %s does not exist\n"), stylesdir.get());
+			FBNLS::ScreenSet, FBNLS::ScreenSTYLESDIRErrorNoExist,		
+			"BScreen::parseMenuFile: [stylesdir/stylesmenu]"
+			" error, %s does not exist\n"), stylesdir.get());
 	} // end of 'stat'
 
 }
@@ -1655,25 +1579,30 @@ void BScreen::shutdown(void) {
 
 void BScreen::showPosition(int x, int y) {
 	if (! geom_visible) {
+#ifdef XINERAMA
+		unsigned int head = hasXinerama() ? getCurrHead() : 0;
+
+		XMoveResizeWindow(getBaseDisplay()->getXDisplay(), geom_window,
+			getHeadX(head) + (getHeadWidth(head) - geom_w) / 2,
+			getHeadY(head) + (getHeadHeight(head) - geom_h) / 2, geom_w, geom_h);
+#else // !XINERMA
 		XMoveResizeWindow(getBaseDisplay()->getXDisplay(), geom_window,
 			(getWidth() - geom_w) / 2,
 			(getHeight() - geom_h) / 2, geom_w, geom_h);
+#endif // XINERAMA
+
 		XMapWindow(getBaseDisplay()->getXDisplay(), geom_window);
 		XRaiseWindow(getBaseDisplay()->getXDisplay(), geom_window);
 
-		geom_visible = True;
+		geom_visible = true;
 	}
 	const int label_size = 1024;
 	char label[label_size];
 	
 	snprintf(label, label_size,
 		I18n::instance()->getMessage(
-#ifdef		NLS
-				 ScreenSet, ScreenPositionFormat,
-#else // !NLS
-				 0, 0,
-#endif // NLS
-				 "X: %4d x Y: %4d"), x, y);
+			 FBNLS::ScreenSet, FBNLS::ScreenPositionFormat,
+			 "X: %4d x Y: %4d"), x, y);
 
 	XClearWindow(getBaseDisplay()->getXDisplay(), geom_window);
 
@@ -1695,25 +1624,29 @@ void BScreen::showPosition(int x, int y) {
 
 void BScreen::showGeometry(unsigned int gx, unsigned int gy) {
 	if (! geom_visible) {
+#ifdef XINERAMA
+		unsigned int head = hasXinerama() ? getCurrHead() : 0;
+
 		XMoveResizeWindow(getBaseDisplay()->getXDisplay(), geom_window,
-											(getWidth() - geom_w) / 2,
-											(getHeight() - geom_h) / 2, geom_w, geom_h);
+			getHeadX(head) + (getHeadWidth(head) - geom_w) / 2,
+			getHeadY(head) + (getHeadHeight(head) - geom_h) / 2, geom_w, geom_h);
+#else // !XINERMA
+		XMoveResizeWindow(getBaseDisplay()->getXDisplay(), geom_window,
+			(getWidth() - geom_w) / 2,
+			(getHeight() - geom_h) / 2, geom_w, geom_h);
+#endif // XINERAMA
 		XMapWindow(getBaseDisplay()->getXDisplay(), geom_window);
 		XRaiseWindow(getBaseDisplay()->getXDisplay(), geom_window);
 
-		geom_visible = True;
+		geom_visible = true;
 	}
 	
 	char label[1024];
 
 	sprintf(label,
 		I18n::instance()->getMessage(
-#ifdef		NLS
-				 ScreenSet, ScreenGeometryFormat,
-#else // !NLS
-				 0, 0,
-#endif // NLS
-				 "W: %4d x H: %4d"), gx, gy);
+			FBNLS::ScreenSet, FBNLS::ScreenGeometryFormat,
+			 "W: %4d x H: %4d"), gx, gy);
 
 	XClearWindow(getBaseDisplay()->getXDisplay(), geom_window);
 
@@ -1735,7 +1668,7 @@ void BScreen::showGeometry(unsigned int gx, unsigned int gy) {
 void BScreen::hideGeometry(void) {
 	if (geom_visible) {
 		XUnmapWindow(getBaseDisplay()->getXDisplay(), geom_window);
-		geom_visible = False;
+		geom_visible = false;
 	}
 }
 
@@ -1743,14 +1676,14 @@ void BScreen::hideGeometry(void) {
 // Goes to the workspace "right" of the current
 //--------------------------------------------
 void BScreen::nextWorkspace(const int delta) {
-    changeWorkspaceID( (getCurrentWorkspaceID()+delta) % getCount());
+	changeWorkspaceID( (getCurrentWorkspaceID()+delta) % getCount());
 }
 
 //------------- prevWorkspace ----------------
 // Goes to the workspace "left" of the current
 //--------------------------------------------
 void BScreen::prevWorkspace(const int delta) {
-    changeWorkspaceID( (getCurrentWorkspaceID()-delta+getCount()) % getCount());
+	changeWorkspaceID( (getCurrentWorkspaceID()-delta+getCount()) % getCount());
 }
 
 //-------------- rightWorkspace ---------------
@@ -1765,10 +1698,20 @@ void BScreen::rightWorkspace(const int delta) {
 // Goes to the workspace "left" of the current
 //--------------------------------------------
 void BScreen::leftWorkspace(const int delta) {
-	if (getCurrentWorkspaceID() >= delta)
+	if (getCurrentWorkspaceID() >= static_cast<unsigned int>(delta))
 		changeWorkspaceID(getCurrentWorkspaceID()-delta);
 }
 
+//-------- doSkipWindow
+// Returns true if the windows should be skiped
+// else false
+//----------
+bool BScreen::doSkipWindow(const FluxboxWindow *w, int opts) {
+	return ((opts & CYCLESKIPSTUCK) != 0 && w->isStuck() || // skip if stuck
+			(opts & CYCLESKIPLOWERTABS) != 0 && w->isLowerTab() || // skip if lower tab
+			(opts & CYCLESKIPSHADED) != 0 && w->isShaded()); // skip if shaded
+}
+
 #ifdef GNOME
 void BScreen::initGnomeAtoms(void) {
 
