@@ -19,8 +19,9 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// stupid macros needed to access some functions in version 2 of the GNU C
-// library
+// $Id: Slit.cc,v 1.12 2002/04/08 22:26:58 fluxgen Exp $
+
+//use GNU extensions
 #ifndef	 _GNU_SOURCE
 #define	 _GNU_SOURCE
 #endif // _GNU_SOURCE
@@ -43,8 +44,7 @@
 #include <algorithm>
 
 
-Slit::Slit(BScreen *scr) {
-	screen = scr;
+Slit::Slit(BScreen *scr):screen(scr), timer(this), slitmenu(this) {
 	fluxbox = Fluxbox::instance();
 
 	on_top = screen->isSlitOnTop();
@@ -53,11 +53,9 @@ Slit::Slit(BScreen *scr) {
 	display = screen->getBaseDisplay()->getXDisplay();
 	frame.window = frame.pixmap = None;
 
-	timer = new BTimer(fluxbox, this);
-	timer->setTimeout(fluxbox->getAutoRaiseDelay());
-	timer->fireOnce(True);
-
-	slitmenu = new Slitmenu(this);
+	
+	timer.setTimeout(fluxbox->getAutoRaiseDelay());
+	timer.fireOnce(True);
 
 	XSetWindowAttributes attrib;
 	unsigned long create_mask = CWBackPixmap | CWBackPixel | CWBorderPixel |
@@ -76,8 +74,8 @@ Slit::Slit(BScreen *scr) {
 	frame.window =
 		XCreateWindow(display, screen->getRootWindow(), frame.x, frame.y,
 			frame.width, frame.height, screen->getBorderWidth(),
-									screen->getDepth(), InputOutput, screen->getVisual(),
-									create_mask, &attrib);
+			screen->getDepth(), InputOutput, screen->getVisual(),
+			create_mask, &attrib);
 	fluxbox->saveSlitSearch(frame.window, this);
 
 	reconfigure();
@@ -87,10 +85,6 @@ Slit::Slit(BScreen *scr) {
 Slit::~Slit() {
 	fluxbox->grab();
 
-	delete timer;
-
-	delete slitmenu;
-
 	screen->getImageControl()->removeImage(frame.pixmap);
 
 	fluxbox->removeSlitSearch(frame.window);
@@ -103,9 +97,6 @@ Slit::~Slit() {
 
 void Slit::addClient(Window w) {
 	fluxbox->grab();
-	#ifdef DEBUG
-	fprintf(stderr, "%s(%d): adding Client\n", __FILE__, __LINE__);
-	#endif 
 	if (fluxbox->validateWindow(w)) {
 		SlitClient *client = new SlitClient;
 		client->client_window = w;
@@ -130,16 +121,8 @@ void Slit::addClient(Window w) {
 			client->icon_window = None;
 			client->window = client->client_window;
 		}
-	#ifndef KDE 
 		XWindowAttributes attrib;
-		if (XGetWindowAttributes(display, client->window, &attrib)) {
-			client->width = attrib.width;
-			client->height = attrib.height;
-		} else {
-			client->width = client->height = 64;
-		}
-	#else //KDE stuff starts here
-	  XWindowAttributes attrib;
+	#ifdef KDE
 		//Check and see if new client is a KDE dock applet
 		//If so force reasonable size
 		bool iskdedockapp=false;
@@ -171,14 +154,16 @@ void Slit::addClient(Window w) {
 
 		if (iskdedockapp)
 			client->width = client->height = 24;
-		else {
+		else
+	#endif // KDE
+		{
 			if (XGetWindowAttributes(display, client->window, &attrib)) {
 				client->width = attrib.width;
 				client->height = attrib.height;
-			}	else
+			} else {
 				client->width = client->height = 64;
+			}
 		}
-	#endif // KDE
 
 		XSetWindowBorderWidth(display, client->window, 0);
 
@@ -416,98 +401,119 @@ void Slit::reconfigure(void) {
 		break;
 	}
 
-	slitmenu->reconfigure();
+	slitmenu.reconfigure();
 }
 
 
 void Slit::reposition(void) {
+	int head_x = 0,
+			head_y = 0,
+			head_w,
+			head_h;
+#ifdef XINERMA
+	if (screen->hasXinerama()) {
+		unsigned int head = screen->getSlitOnHead();
+
+		head_x = screen->getHeadX(head);
+		head_y = screen->getHeadY(head);
+		head_w = screen->getHeadWidth(head);
+		head_h = screen->getHeadHeight(head);
+	} else {
+		head_w = screen->getWidth();
+		head_h = screen->getHeight();
+	}
+#else // !XINERAMA
+		head_w = screen->getWidth();
+		head_h = screen->getHeight();
+#endif // XINERAMA
+
 	// place the slit in the appropriate place
 	switch (screen->getSlitPlacement()) {
 	case TOPLEFT:
-		frame.x = 0;
-		frame.y = 0;
+		frame.x = head_x;
+		frame.y = head_y;
 		if (screen->getSlitDirection() == VERTICAL) {
 			frame.x_hidden = screen->getBevelWidth() -
 				screen->getBorderWidth() - frame.width;
-			frame.y_hidden = 0;
+			frame.y_hidden = head_y;
 		} else {
-			frame.x_hidden = 0;
+			frame.x_hidden = head_x;
 			frame.y_hidden = screen->getBevelWidth() -
 				screen->getBorderWidth() - frame.height;
 		}
 		break;
 
 	case CENTERLEFT:
-		frame.x = 0;
-		frame.y = (screen->getHeight() - frame.height) / 2;
-		frame.x_hidden = screen->getBevelWidth() -
+		frame.x = head_x;
+		frame.y = head_y + (head_h - frame.height) / 2;
+		frame.x_hidden = head_x + screen->getBevelWidth() -
 			screen->getBorderWidth() - frame.width;
 		frame.y_hidden = frame.y;
 		break;
 
 	case BOTTOMLEFT:
-		frame.x = 0;
-		frame.y = screen->getHeight() - frame.height - screen->getBorderWidth2x();
+		frame.x = head_x;
+		frame.y = head_h - frame.height - screen->getBorderWidth2x();
 		if (screen->getSlitDirection() == VERTICAL) {
-			frame.x_hidden = screen->getBevelWidth() - screen->getBorderWidth()
-								 - frame.width;
+			frame.x_hidden = head_x + screen->getBevelWidth() -
+				screen->getBorderWidth() - frame.width;
 			frame.y_hidden = frame.y;
 		} else {
-			frame.x_hidden = 0;
-			frame.y_hidden = screen->getHeight() -
+			frame.x_hidden = head_x;
+			frame.y_hidden = head_y + head_h -
 				screen->getBevelWidth() - screen->getBorderWidth();
 		}
 		break;
 
 	case TOPCENTER:
-		frame.x = (screen->getWidth() - frame.width) / 2;
-		frame.y = 0;
+		frame.x = head_x + ((head_w - frame.width) / 2);
+		frame.y = head_y;
 		frame.x_hidden = frame.x;
-		frame.y_hidden = screen->getBevelWidth() -
+		frame.y_hidden = head_y + screen->getBevelWidth() -
 			screen->getBorderWidth() - frame.height;
 		break;
 
 	case BOTTOMCENTER:
-		frame.x = (screen->getWidth() - frame.width) / 2;
-		frame.y = screen->getHeight() - frame.height - screen->getBorderWidth2x();
+		frame.x = head_x + ((head_w - frame.width) / 2);
+		frame.y = head_y + head_h - frame.height - screen->getBorderWidth2x();
 		frame.x_hidden = frame.x;
-		frame.y_hidden = screen->getHeight() -
+		frame.y_hidden = head_y + head_h -
 			screen->getBevelWidth() - screen->getBorderWidth();
 		break;
 
 	case TOPRIGHT:
-		frame.x = screen->getWidth() - frame.width - screen->getBorderWidth2x();
-		frame.y = 0;
+		frame.x = head_x + head_w - frame.width - screen->getBorderWidth2x();
+		frame.y = head_y;
 		if (screen->getSlitDirection() == VERTICAL) {
-			frame.x_hidden = screen->getWidth() -
+			frame.x_hidden = head_x + head_w -
 				screen->getBevelWidth() - screen->getBorderWidth();
-			frame.y_hidden = 0;
+			frame.y_hidden = head_y;
 		} else {
 			frame.x_hidden = frame.x;
-			frame.y_hidden = screen->getBevelWidth() -
+			frame.y_hidden = head_y + screen->getBevelWidth() -
 				screen->getBorderWidth() - frame.height;
 		}
 		break;
 
 	case CENTERRIGHT:
 	default:
-		frame.x = screen->getWidth() - frame.width - screen->getBorderWidth2x();
-		frame.y = (screen->getHeight() - frame.height) / 2;
-		frame.x_hidden = screen->getWidth() -
+		frame.x = head_x + head_w - frame.width - screen->getBorderWidth2x();
+		frame.y = head_y + ((head_h - frame.height) / 2);
+		frame.x_hidden = head_x + head_w -
 			screen->getBevelWidth() - screen->getBorderWidth();
 		frame.y_hidden = frame.y;
 		break;
 
 	case BOTTOMRIGHT:
-		frame.x = screen->getWidth() - frame.width - screen->getBorderWidth2x();
-		frame.y = screen->getHeight() - frame.height - screen->getBorderWidth2x();
+		frame.x = head_x + head_w - frame.width - screen->getBorderWidth2x();
+		frame.y = head_y + head_h - frame.height - screen->getBorderWidth2x();
 		if (screen->getSlitDirection() == VERTICAL) {
-			frame.x_hidden = screen->getWidth() - 
+			frame.x_hidden = head_x + head_w - 
 				screen->getBevelWidth() - screen->getBorderWidth();
 			frame.y_hidden = frame.y;
 		} else {
 			frame.x_hidden = frame.x;
-			frame.y_hidden = screen->getHeight() - 
+			frame.y_hidden = head_y + head_h - 
 				screen->getBevelWidth() - screen->getBorderWidth();
 		}
 		break;
@@ -560,26 +566,24 @@ void Slit::buttonPressEvent(XButtonEvent *e) {
 	} else if (e->button == Button2 && (! on_top)) {
 		XLowerWindow(display, frame.window);
 	} else if (e->button == Button3) {
-		if (! slitmenu->isVisible()) {
-			int x, y;
-
-			x = e->x_root - (slitmenu->getWidth() / 2);
-			y = e->y_root - (slitmenu->getHeight() / 2);
+		if (! slitmenu.isVisible()) {
+			int x = e->x_root - (slitmenu.width() / 2),
+				y = e->y_root - (slitmenu.height() / 2); 
 
 			if (x < 0)
 				x = 0;
-			else if (x + slitmenu->getWidth() > screen->getWidth())
-				x = screen->getWidth() - slitmenu->getWidth();
+			else if (x + slitmenu.width() > screen->getWidth())
+				x = screen->getWidth() - slitmenu.width();
 
 			if (y < 0)
 				y = 0;
-			else if (y + slitmenu->getHeight() > screen->getHeight())
-				y = screen->getHeight() - slitmenu->getHeight();
+			else if (y + slitmenu.height() > screen->getHeight())
+				y = screen->getHeight() - slitmenu.height();
 
-			slitmenu->move(x, y);
-			slitmenu->show();
+			slitmenu.move(x, y);
+			slitmenu.show();
 		} else
-			slitmenu->hide();
+			slitmenu.hide();
 	}
 }
 
@@ -589,9 +593,9 @@ void Slit::enterNotifyEvent(XCrossingEvent *) {
 		return;
 
 	if (hidden) {
-		if (! timer->isTiming()) timer->start();
+		if (! timer.isTiming()) timer.start();
 	} else {
-		if (timer->isTiming()) timer->stop();
+		if (timer.isTiming()) timer.stop();
 	}
 }
 
@@ -601,9 +605,9 @@ void Slit::leaveNotifyEvent(XCrossingEvent *) {
 		return;
 
 	if (hidden) {
-		if (timer->isTiming()) timer->stop();
-	} else if (! slitmenu->isVisible()) {
-		if (! timer->isTiming()) timer->start();
+		if (timer.isTiming()) timer.stop();
+	} else if (! slitmenu.isVisible()) {
+		if (! timer.isTiming()) timer.start();
 	}
 }
 
@@ -659,87 +663,87 @@ void Slit::timeout(void) {
 Slitmenu::Slitmenu(Slit *sl) : Basemenu(sl->screen) {
 	slit = sl;
 	I18n *i18n = I18n::instance();
-	
+	using namespace FBNLS;
 	setLabel(i18n->getMessage(
-#ifdef		NLS
-					SlitSet, SlitSlitTitle,
-#else // !NLS
-					0, 0,
-#endif // NLS
-					"Slit"));
+		SlitSet, SlitSlitTitle,
+		"Slit"));
 	setInternalMenu();
 
 	directionmenu = new Directionmenu(this);
 	placementmenu = new Placementmenu(this);
+#ifdef XINERAMA
+	if (slit->screen->hasXinerama()) { // only create if we need
+		headmenu = new Headmenu(this);
+	}
+#endif // XINERAMA
 
 	insert(i18n->getMessage(
-#ifdef		NLS
-				CommonSet, CommonDirectionTitle,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Direction"),
+		CommonSet, CommonDirectionTitle,
+		"Direction"),
 	 directionmenu);
 	insert(i18n->getMessage(
-#ifdef		NLS
-				CommonSet, CommonPlacementTitle,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Placement"),
+		CommonSet, CommonPlacementTitle,
+		"Placement"),
 	 placementmenu);
+
+#ifdef XINERAMA
+	//TODO: NLS
+	if (slit->screen->hasXinerama()) {
+		insert(i18n->getMessage(0, 0, "Place on Head"), headmenu);
+	}
+#endif // XINERAMA
+
 	insert(i18n->getMessage(
-#ifdef		NLS
-				CommonSet, CommonAlwaysOnTop,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Always on top"), 1);
+		CommonSet, CommonAlwaysOnTop,
+		"Always on top"), 1);
 	insert(i18n->getMessage(
-#ifdef		NLS
-				CommonSet, CommonAutoHide,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Auto hide"), 2);
+		CommonSet, CommonAutoHide,
+		"Auto hide"), 2);
 
 	update();
 
-	if (slit->isOnTop()) setItemSelected(2, True);
-	if (slit->doAutoHide()) setItemSelected(3, True);
+	if (slit->isOnTop()) setItemSelected(2, true);
+	if (slit->doAutoHide()) setItemSelected(3, true);
 }
 
 
 Slitmenu::~Slitmenu(void) {
 	delete directionmenu;
 	delete placementmenu;
+#ifdef XINERAMA
+	if (slit->screen->hasXinerama()) {
+		delete headmenu;
+	}
+#endif // XINERAMA
 }
 
 
-void Slitmenu::itemSelected(int button, int index) {
+void Slitmenu::itemSelected(int button, unsigned int index) {
 	if (button == 1) {
 		BasemenuItem *item = find(index);
 		if (! item) return;
 
 		switch (item->function()) {
 		case 1: // always on top
-			{
-	Bool change = ((slit->isOnTop()) ?	False : True);
-	slit->on_top = change;
-	setItemSelected(2, change);
+		{
+			bool change = ((slit->isOnTop()) ?	false : true);
+			slit->on_top = change;
+			setItemSelected(2, change);
 
-	if (slit->isOnTop()) slit->screen->raiseWindows((Window *) 0, 0);
-	break;
-			}
+			if (slit->isOnTop())
+				slit->screen->raiseWindows((Window *) 0, 0);
+			
+		break;
+		}
 
 		case 2: // auto hide
-			{
-	Bool change = ((slit->doAutoHide()) ?	False : True);
-	slit->do_auto_hide = change;
-	setItemSelected(3, change);
+		{
+			bool change = ((slit->doAutoHide()) ?	false : true);
+			slit->do_auto_hide = change;
+			setItemSelected(3, change);
 
-	break;
-			}
+		break;
+		}
 		}
 	}
 }
@@ -755,6 +759,11 @@ void Slitmenu::internal_hide(void) {
 void Slitmenu::reconfigure(void) {
 	directionmenu->reconfigure();
 	placementmenu->reconfigure();
+#ifdef XINERAMA
+	if (slit->screen->hasXinerama()) {
+		headmenu->reconfigure();
+	}
+#endif // XINERAMA
 
 	Basemenu::reconfigure();
 }
@@ -763,43 +772,31 @@ void Slitmenu::reconfigure(void) {
 Slitmenu::Directionmenu::Directionmenu(Slitmenu *sm) : Basemenu(sm->slit->screen) {
 	slitmenu = sm;
 	I18n *i18n = I18n::instance();
-	
+	using namespace FBNLS;	
 	setLabel(i18n->getMessage(
-#ifdef		NLS
-					SlitSet, SlitSlitDirection,
-#else // !NLS
-					0, 0,
-#endif // NLS
-					"Slit Direction"));
+		SlitSet, SlitSlitDirection,
+		"Slit Direction"));
 	setInternalMenu();
 
 	insert(i18n->getMessage(
-#ifdef		NLS
-				CommonSet, CommonDirectionHoriz,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Horizontal"),
+		CommonSet, CommonDirectionHoriz,
+		"Horizontal"),
 	 Slit::HORIZONTAL);
 	insert(i18n->getMessage(
-#ifdef		NLS
-				CommonSet, CommonDirectionVert,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Vertical"),
+		CommonSet, CommonDirectionVert,
+		"Vertical"),
 	 Slit::VERTICAL);
 
 	update();
 
 	if (sm->slit->screen->getSlitDirection() == Slit::HORIZONTAL)
-		setItemSelected(0, True);
+		setItemSelected(0, true);
 	else
-		setItemSelected(1, True);
+		setItemSelected(1, true);
 }
 
 
-void Slitmenu::Directionmenu::itemSelected(int button, int index) {
+void Slitmenu::Directionmenu::itemSelected(int button, unsigned int index) {
 	if (button == 1) {
 		BasemenuItem *item = find(index);
 		if (! item) return;
@@ -807,11 +804,11 @@ void Slitmenu::Directionmenu::itemSelected(int button, int index) {
 		slitmenu->slit->screen->saveSlitDirection(item->function());
 
 		if (item->function() == Slit::HORIZONTAL) {
-			setItemSelected(0, True);
-			setItemSelected(1, False);
+			setItemSelected(0, true);
+			setItemSelected(1, false);
 		} else {
-			setItemSelected(0, False);
-			setItemSelected(1, True);
+			setItemSelected(0, false);
+			setItemSelected(1, true);
 		}
 
 		hide();
@@ -823,88 +820,52 @@ void Slitmenu::Directionmenu::itemSelected(int button, int index) {
 Slitmenu::Placementmenu::Placementmenu(Slitmenu *sm) : Basemenu(sm->slit->screen) {
 	slitmenu = sm;
 	I18n *i18n = I18n::instance();
-	
+	using namespace FBNLS;	
 	setLabel(i18n->getMessage(
-#ifdef		NLS
-					SlitSet, SlitSlitPlacement,
-#else // !NLS
-					0, 0,
-#endif // NLS
-					"Slit Placement"));
+		SlitSet, SlitSlitPlacement,
+		"Slit Placement"));
 	setMinimumSublevels(3);
 	setInternalMenu();
 
 	insert(i18n->getMessage(
-#ifdef		NLS
-				CommonSet, CommonPlacementTopLeft,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Top Left"),
+		CommonSet, CommonPlacementTopLeft,
+		"Top Left"),
 	 Slit::TOPLEFT);
 	insert(i18n->getMessage(
-#ifdef		NLS
-				CommonSet, CommonPlacementCenterLeft,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Center Left"),
+		CommonSet, CommonPlacementCenterLeft,
+		"Center Left"),
 	 Slit::CENTERLEFT);
 	insert(i18n->getMessage(
-#ifdef		NLS
-				CommonSet, CommonPlacementBottomLeft,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Bottom Left"),
+		CommonSet, CommonPlacementBottomLeft,
+		"Bottom Left"),
 	 Slit::BOTTOMLEFT);
 	insert(i18n->getMessage(
-#ifdef		NLS
-				CommonSet, CommonPlacementTopCenter,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Top Center"),
+		CommonSet, CommonPlacementTopCenter,
+		"Top Center"),
 	 Slit::TOPCENTER);
 	insert("");
 	insert(i18n->getMessage(
-#ifdef		NLS
-				CommonSet, CommonPlacementBottomCenter,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Bottom Center"),
+		CommonSet, CommonPlacementBottomCenter,
+		"Bottom Center"),
 	 Slit::BOTTOMCENTER);
 	insert(i18n->getMessage(
-#ifdef		NLS
-				CommonSet, CommonPlacementTopRight,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Top Right"),
+		CommonSet, CommonPlacementTopRight,
+		"Top Right"),
 	 Slit::TOPRIGHT);
 	insert(i18n->getMessage(
-#ifdef		NLS
-				CommonSet, CommonPlacementCenterRight,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Center Right"),
+		CommonSet, CommonPlacementCenterRight,
+		"Center Right"),
 	 Slit::CENTERRIGHT);
 	insert(i18n->getMessage(
-#ifdef		NLS
-				CommonSet, CommonPlacementBottomRight,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Bottom Right"),
+		CommonSet, CommonPlacementBottomRight,
+		"Bottom Right"),
 	 Slit::BOTTOMRIGHT);
 
 	update();
 }
 
 
-void Slitmenu::Placementmenu::itemSelected(int button, int index) {
+void Slitmenu::Placementmenu::itemSelected(int button, unsigned int index) {
 	if (button == 1) {
 		BasemenuItem *item = find(index);
 		if (! item) return;
@@ -917,5 +878,39 @@ void Slitmenu::Placementmenu::itemSelected(int button, int index) {
 	}
 }
 
+#ifdef XINERAMA
+
+Slitmenu::Headmenu::Headmenu(Slitmenu *sm)
+	: Basemenu(sm->slit->screen) {
+	slitmenu = sm;
+	I18n *i18n = I18n::instance();
+
+	setLabel(i18n->getMessage(0, 0, "Place on Head")); //TODO: NLS
+	setInternalMenu();
+
+	int numHeads = slitmenu->slit->screen->getNumHeads();
+	// fill menu with head entries
+	for (int i = 0; i < numHeads; i++) {
+		char headName[32];
+		sprintf(headName, "Head %i", i+1); //TODO: NLS
+		insert(i18n->getMessage(0, 0, headName), i);
+	}
+
+	update();
+}
+
+void Slitmenu::Headmenu::itemSelected(int button, unsigned int index) {
+	if (button == 1) {
+		BasemenuItem *item = find(index);
+		if (! item)
+			return;
+
+		slitmenu->slit->screen->saveSlitOnHead(item->function());
+		hide();
+		slitmenu->slit->reconfigure();
+	}
+}
+
+#endif // XINERAMA
 
 #endif // SLIT
