@@ -1,3 +1,6 @@
+// Toolbar.cc for Fluxbox
+// Copyright (c) 2002 Henrik Kinnunen (fluxgen@linuxmail.org)
+//
 // Toolbar.cc for Blackbox - an X11 Window manager
 // Copyright (c) 1997 - 2000 Brad Hughes (bhughes@tcac.net)
 //
@@ -13,13 +16,13 @@
 //
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.	IN NO EVENT SHALL
+// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 // THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: Toolbar.cc,v 1.12 2002/02/20 23:13:32 fluxgen Exp $
+// $Id: Toolbar.cc,v 1.21 2002/04/09 23:16:28 fluxgen Exp $
 
 // stupid macros needed to access some functions in version 2 of the GNU C
 // library
@@ -71,20 +74,20 @@ using namespace std;
 Toolbar::Toolbar(BScreen *scrn):
 screen(scrn),
 image_ctrl(screen->getImageControl()),
+clock_timer(this), 	// get the clock updating every minute
 iconbar(0)
 {
 
-	fluxbox = Fluxbox::instance();
-	
-	// get the clock updating every minute
-	clock_timer = new BTimer(fluxbox, this);
+	fluxbox = Fluxbox::instance();	
+
+
 	timeval now;
 	gettimeofday(&now, 0);
-	clock_timer->setTimeout((60 - (now.tv_sec % 60)) * 1000);
-	clock_timer->start();
+	clock_timer.setTimeout((60 - (now.tv_sec % 60)) * 1000);
+	clock_timer.start();
 
 	hide_handler.toolbar = this;
-	hide_timer = new BTimer(fluxbox, &hide_handler);
+	hide_timer = new BTimer(&hide_handler);
 	hide_timer->setTimeout(fluxbox->getAutoRaiseDelay());
 	hide_timer->fireOnce(True);
 
@@ -199,7 +202,6 @@ Toolbar::~Toolbar(void) {
 	XDestroyWindow(display, frame.window);
 
 	delete hide_timer;
-	delete clock_timer;
 	delete toolbarmenu;
 	if (iconbar)
 		delete iconbar; 
@@ -216,9 +218,38 @@ void Toolbar::delIcon(FluxboxWindow *w) {
 }
 		
 void Toolbar::reconfigure(void) {
+	int head_x = 0,
+			head_y = 0,
+			head_w,
+			head_h;
+
 	frame.bevel_w = screen->getBevelWidth();
+#ifdef XINERAMA
+	int head = (screen->hasXinerama())
+		? screen->getToolbarOnHead()
+		: -1;
+
+	if (head >= 0) { // toolbar on head nr, if -1 then over ALL heads
+		head_x = screen->getHeadX(head);
+		head_y = screen->getHeadY(head);
+		head_w = screen->getHeadWidth(head);
+		head_h = screen->getHeadHeight(head);
+
+		frame.width =
+			(screen->getHeadWidth(head) * screen->getToolbarWidthPercent() / 100);
+	}	else {
+		head_w = screen->getWidth();
+		head_h = screen->getHeight();
+
+		frame.width = screen->getWidth() * screen->getToolbarWidthPercent() / 100;
+	}
+#else // !XINERAMA
+	head_w = screen->getWidth();
+	head_h = screen->getHeight();
+
 	frame.width = screen->getWidth() * screen->getToolbarWidthPercent() / 100;
-	
+#endif // XINERAMA
+
 	I18n *i18n = I18n::instance();
 	
 	if (i18n->multibyte())
@@ -234,50 +265,51 @@ void Toolbar::reconfigure(void) {
 	
 	switch (screen->getToolbarPlacement()) {
 	case TOPLEFT:
-		frame.x = 0;
-		frame.y = 0;
-		frame.x_hidden = 0;
-		frame.y_hidden = screen->getBevelWidth() - screen->getBorderWidth() - frame.height;
+		frame.x = head_x;
+		frame.y = head_y;
+		frame.x_hidden = head_x;
+		frame.y_hidden = head_y +
+			screen->getBevelWidth() - screen->getBorderWidth() - frame.height;
 		break;
 
 	case BOTTOMLEFT:
-		frame.x = 0;
-		frame.y = screen->getHeight() - frame.height - screen->getBorderWidth2x();
-		frame.x_hidden = 0;
-		frame.y_hidden = screen->getHeight() - screen->getBevelWidth() - 
+		frame.x = head_x;
+		frame.y = head_y + head_h - frame.height - screen->getBorderWidth2x();
+		frame.x_hidden = head_x;
+		frame.y_hidden = head_y + head_h - screen->getBevelWidth() - 
 			screen->getBorderWidth();
 		break;
 
 	case TOPCENTER:
-		frame.x = (screen->getWidth() - frame.width) / 2;
-		frame.y = 0;
+		frame.x = head_x + ((head_w - frame.width) / 2);
+		frame.y = head_y;
 		frame.x_hidden = frame.x;
-		frame.y_hidden = screen->getBevelWidth() - screen->getBorderWidth() -
-			frame.height;
+		frame.y_hidden = head_y +
+			screen->getBevelWidth() - screen->getBorderWidth() - frame.height;
 		break;
 
 	case BOTTOMCENTER:
 	default:
-		frame.x = (screen->getWidth() - frame.width) / 2;
-		frame.y = screen->getHeight() - frame.height - screen->getBorderWidth2x();
+		frame.x = head_x + ((head_w - frame.width) / 2);
+		frame.y = head_y + head_h - frame.height - screen->getBorderWidth2x();
 		frame.x_hidden = frame.x;
-		frame.y_hidden = screen->getHeight() - screen->getBevelWidth() - 
+		frame.y_hidden = head_y + head_h - screen->getBevelWidth() - 
 			screen->getBorderWidth();
 		break;
 
 	case TOPRIGHT:
-		frame.x = screen->getWidth() - frame.width - screen->getBorderWidth2x();
-		frame.y = 0;
+		frame.x = head_x + head_w - frame.width - screen->getBorderWidth2x();
+		frame.y = head_y;
 		frame.x_hidden = frame.x;
-		frame.y_hidden = screen->getBevelWidth() - screen->getBorderWidth() - 
-			frame.height;
+		frame.y_hidden = head_y +
+			screen->getBevelWidth() - screen->getBorderWidth() - frame.height;
 		break;
 
 	case BOTTOMRIGHT:
-		frame.x = screen->getWidth() - frame.width - screen->getBorderWidth2x();
-		frame.y = screen->getHeight() - frame.height - screen->getBorderWidth2x();
+		frame.x = head_x + head_w - frame.width - screen->getBorderWidth2x();
+		frame.y = head_y + head_h - frame.height - screen->getBorderWidth2x();
 		frame.x_hidden = frame.x;
-		frame.y_hidden = screen->getHeight() - screen->getBevelWidth() - 
+		frame.y_hidden = head_y + head_h - screen->getBevelWidth() - 
 			screen->getBorderWidth();
 		break;
 	}
@@ -334,7 +366,7 @@ void Toolbar::reconfigure(void) {
 	
 	#endif // HAVE_STRFTIME
 
-	int i;
+	unsigned int i;
 	unsigned int w = 0;
 	frame.workspace_label_w = 0;
 
@@ -342,14 +374,14 @@ void Toolbar::reconfigure(void) {
 		if (i18n->multibyte()) {
 			XRectangle ink, logical;
 			XmbTextExtents(screen->getToolbarStyle()->font.set,
-				 screen->getWorkspace(i)->getName(),
-				 strlen(screen->getWorkspace(i)->getName()),
+				 screen->getWorkspace(i)->name().c_str(),
+				 screen->getWorkspace(i)->name().size(),
 				 &ink, &logical);
 			w = logical.width;
 		} else
 			w = XTextWidth(screen->getToolbarStyle()->font.fontstruct,
-				screen->getWorkspace(i)->getName(),
-				strlen(screen->getWorkspace(i)->getName()));
+				screen->getWorkspace(i)->name().c_str(),
+				screen->getWorkspace(i)->name().size());
 
 		w += (frame.bevel_w * 4);
 
@@ -695,17 +727,17 @@ void Toolbar::redrawWindowLabel(Bool redraw) {
 		if (foc->getScreen() != screen)
 			return;
 
-		int dx = (frame.bevel_w * 2), dlen = strlen(*foc->getTitle());
+		int dx = (frame.bevel_w * 2), dlen = foc->getTitle().size();
 		unsigned int l;
 		I18n *i18n = I18n::instance();
 		
 		if (i18n->multibyte()) {
 			XRectangle ink, logical;
-			XmbTextExtents(screen->getToolbarStyle()->font.set, *foc->getTitle(), dlen,
+			XmbTextExtents(screen->getToolbarStyle()->font.set, foc->getTitle().c_str(), dlen,
 				 &ink, &logical);
 			l = logical.width;
 		} else
-			l = XTextWidth(screen->getToolbarStyle()->font.fontstruct, *foc->getTitle(), dlen);
+			l = XTextWidth(screen->getToolbarStyle()->font.fontstruct, foc->getTitle().c_str(), dlen);
 		
 		l += (frame.bevel_w * 4);
 
@@ -714,10 +746,10 @@ void Toolbar::redrawWindowLabel(Bool redraw) {
 				if (i18n->multibyte()) {
 					XRectangle ink, logical;
 					XmbTextExtents(screen->getToolbarStyle()->font.set,
-						 *foc->getTitle(), dlen, &ink, &logical);
+						 foc->getTitle().c_str(), dlen, &ink, &logical);
 					l = logical.width;
 				} else
-					l = XTextWidth(screen->getToolbarStyle()->font.fontstruct, *foc->getTitle(), dlen);
+					l = XTextWidth(screen->getToolbarStyle()->font.fontstruct, foc->getTitle().c_str(), dlen);
 	
 				l += (frame.bevel_w * 4);
 	
@@ -742,19 +774,19 @@ void Toolbar::redrawWindowLabel(Bool redraw) {
 				screen->getToolbarStyle()->font.set,
 				screen->getToolbarStyle()->w_text_gc, dx, 1 -
 				screen->getToolbarStyle()->font.set_extents->max_ink_extent.y,
-				*foc->getTitle(), dlen);
+				foc->getTitle().c_str(), dlen);
 		else
 			XDrawString(display, frame.window_label,
-			screen->getToolbarStyle()->w_text_gc, dx,
-			screen->getToolbarStyle()->font.fontstruct->ascent	+ 1,
-			*foc->getTitle(), dlen);
+				screen->getToolbarStyle()->w_text_gc, dx,
+				screen->getToolbarStyle()->font.fontstruct->ascent	+ 1,
+				foc->getTitle().c_str(), dlen);
 	} else
 		XClearWindow(display, frame.window_label);
 }
  
  
 void Toolbar::redrawWorkspaceLabel(Bool redraw) {
-	if (screen->getCurrentWorkspace()->getName()) {
+	if (screen->getCurrentWorkspace()->name().size()>0) {
 		
 		if (redraw)
 			XClearWindow(display, frame.workspace_label);
@@ -763,10 +795,10 @@ void Toolbar::redrawWorkspaceLabel(Bool redraw) {
 					&screen->getToolbarStyle()->font,
 					frame.workspace_label_w, frame.width,
 				frame.bevel_w, 
-				const_cast<char *>(screen->getCurrentWorkspace()->getName()));*/
+				const_cast<char *>(screen->getCurrentWorkspace()->name().c_str()));*/
 		
 		int dx = (frame.bevel_w * 2), dlen =
-						strlen(screen->getCurrentWorkspace()->getName());
+						screen->getCurrentWorkspace()->name().size();
 		unsigned int l;
 		I18n *i18n = I18n::instance();
 		
@@ -774,12 +806,12 @@ void Toolbar::redrawWorkspaceLabel(Bool redraw) {
 		if (i18n->multibyte()) {
 			XRectangle ink, logical;
 			XmbTextExtents(screen->getToolbarStyle()->font.set,
-						screen->getCurrentWorkspace()->getName(), dlen,
+						screen->getCurrentWorkspace()->name().c_str(), dlen,
 						&ink, &logical);
 			l = logical.width;
 		} else
 			l = XTextWidth(screen->getToolbarStyle()->font.fontstruct,
-					screen->getCurrentWorkspace()->getName(), dlen);
+					screen->getCurrentWorkspace()->name().c_str(), dlen);
 		
 		l += (frame.bevel_w * 4);
 		
@@ -788,12 +820,12 @@ void Toolbar::redrawWorkspaceLabel(Bool redraw) {
 				if (i18n->multibyte()) {
 					XRectangle ink, logical;
 					XmbTextExtents(screen->getToolbarStyle()->font.set,
-						screen->getCurrentWorkspace()->getName(), dlen,
+						screen->getCurrentWorkspace()->name().c_str(), dlen,
 						&ink, &logical);
 						l = logical.width;
 				} else {
 					l = XTextWidth(screen->getWindowStyle()->font.fontstruct,
-							screen->getCurrentWorkspace()->getName(), dlen);
+							screen->getCurrentWorkspace()->name().c_str(), dlen);
 				}
 				
 				l += (frame.bevel_w * 4);
@@ -820,12 +852,12 @@ void Toolbar::redrawWorkspaceLabel(Bool redraw) {
 				screen->getToolbarStyle()->font.set,
 				screen->getToolbarStyle()->l_text_gc, dx, 1 -
 				screen->getToolbarStyle()->font.set_extents->max_ink_extent.y,
-				(char *) screen->getCurrentWorkspace()->getName(), dlen);
+				(char *) screen->getCurrentWorkspace()->name().c_str(), dlen);
 		} else {
 			XDrawString(display, frame.workspace_label,
 				screen->getToolbarStyle()->l_text_gc, dx,
 				screen->getToolbarStyle()->font.fontstruct->ascent + 1,
-				(char *) screen->getCurrentWorkspace()->getName(), dlen);
+				(char *) screen->getCurrentWorkspace()->name().c_str(), dlen);
 		}
 	}
 }
@@ -1003,18 +1035,18 @@ void Toolbar::buttonPressEvent(XButtonEvent *be) {
 		if (! toolbarmenu->isVisible()) {
 			int x, y;
 
-			x = be->x_root - (toolbarmenu->getWidth() / 2);
-			y = be->y_root - (toolbarmenu->getHeight() / 2);
+			x = be->x_root - (toolbarmenu->width() / 2);
+			y = be->y_root - (toolbarmenu->height() / 2);
 
 			if (x < 0)
 				x = 0;
-			else if (x + toolbarmenu->getWidth() > screen->getWidth())
-				x = screen->getWidth() - toolbarmenu->getWidth();
+			else if (x + toolbarmenu->width() > screen->getWidth())
+				x = screen->getWidth() - toolbarmenu->width();
 
 			if (y < 0)
 				y = 0;
-			else if (y + toolbarmenu->getHeight() > screen->getHeight())
-				y = screen->getHeight() - toolbarmenu->getHeight();
+			else if (y + toolbarmenu->height() > screen->getHeight())
+				y = screen->getHeight() - toolbarmenu->height();
 
 			toolbarmenu->move(x, y);
 			toolbarmenu->show();
@@ -1106,9 +1138,7 @@ void Toolbar::exposeEvent(XExposeEvent *ee) {
 
 void Toolbar::keyPressEvent(XKeyEvent *ke) {
 	if (ke->window == frame.workspace_label && editing) {
-		BaseDisplay::GrabGuard gg(*fluxbox);
-		fluxbox->grab();
-
+		
 		KeySym ks;
 		char keychar[1];
 		XLookupString(ke, keychar, 1, &ks, 0);
@@ -1126,17 +1156,17 @@ void Toolbar::keyPressEvent(XKeyEvent *ke) {
 				XSetInputFocus(display, PointerRoot, None, CurrentTime);
 			
 			if (ks == XK_Return)	//change workspace name if keypress = Return
-				screen->getCurrentWorkspace()->setName(const_cast<char *>(new_workspace_name.c_str()));
+				screen->getCurrentWorkspace()->setName(new_workspace_name.c_str());
 
 			new_workspace_name.erase(); //erase temporary workspace name
 			
-			screen->getCurrentWorkspace()->getMenu()->hide();
+			screen->getCurrentWorkspace()->menu()->hide();
 			screen->getWorkspacemenu()->
-				remove(screen->getCurrentWorkspace()->getWorkspaceID() + 2);
+				remove(screen->getCurrentWorkspace()->workspaceID() + 2);
 			screen->getWorkspacemenu()->
-				insert(screen->getCurrentWorkspace()->getName(),
-			screen->getCurrentWorkspace()->getMenu(),
-			screen->getCurrentWorkspace()->getWorkspaceID() + 1);
+				insert(screen->getCurrentWorkspace()->name().c_str(),
+			screen->getCurrentWorkspace()->menu(),
+			screen->getCurrentWorkspace()->workspaceID() + 1);
 			screen->getWorkspacemenu()->update();
 
 			reconfigure();
@@ -1186,9 +1216,8 @@ void Toolbar::keyPressEvent(XKeyEvent *ke) {
 			XDrawRectangle(display, frame.workspace_label,
 				screen->getWindowStyle()->l_text_focus_gc, x + tw, 0, 1,
 				frame.label_h - 1);
-		}
+		}		
 		
-		fluxbox->ungrab();
 	}
 }
 
@@ -1198,7 +1227,7 @@ void Toolbar::timeout(void) {
 
 	timeval now;
 	gettimeofday(&now, 0);
-	clock_timer->setTimeout((60 - (now.tv_sec % 60)) * 1000);
+	clock_timer.setTimeout((60 - (now.tv_sec % 60)) * 1000);
 }
 
 
@@ -1216,66 +1245,64 @@ void Toolbar::HideHandler::timeout(void) {
 Toolbarmenu::Toolbarmenu(Toolbar *tb) : Basemenu(tb->screen) {
 	toolbar = tb;
 	I18n *i18n = I18n::instance();
-	
+	using namespace FBNLS;
 	setLabel(i18n->getMessage(
-#ifdef		NLS
-				ToolbarSet, ToolbarToolbarTitle,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Toolbar"));
+		ToolbarSet, ToolbarToolbarTitle,
+		"Toolbar"));
 	setInternalMenu();
 
 	placementmenu = new Placementmenu(this);
+#ifdef XINERAMA
+	if (toolbar->screen->hasXinerama()) { // only create if we need it
+		headmenu = new Headmenu(this);
+	}
+#endif // XINERAMA
 
 	insert(i18n->getMessage(
-#ifdef		NLS
-				CommonSet, CommonPlacementTitle,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Placement"),
-				placementmenu);
+		CommonSet, CommonPlacementTitle,
+		"Placement"),
+		placementmenu);
+
+#ifdef XINERAMA
+	if (toolbar->screen->hasXinerama()) { //TODO: NLS
+		insert(i18n->getMessage(0, 0, "Place on Head"), headmenu);
+	}
+#endif // XINERAMA
+
 	insert(i18n->getMessage(
-#ifdef		NLS
-				CommonSet, CommonAlwaysOnTop,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Always on top"),
-				1);
+		CommonSet, CommonAlwaysOnTop,
+		"Always on top"),
+		1);
 	insert(i18n->getMessage(
-#ifdef		NLS
-				CommonSet, CommonAutoHide,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Auto hide"),
+		CommonSet, CommonAutoHide,
+		"Auto hide"),
 				2);
 	insert(i18n->getMessage(
-#ifdef		NLS
-				ToolbarSet, ToolbarEditWkspcName,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Edit current workspace name"),
-				3);
+		ToolbarSet, ToolbarEditWkspcName,
+		"Edit current workspace name"),
+		3);
 
 	update();
 
 	if (toolbar->isOnTop())
-		setItemSelected(1, True);
+		setItemSelected(1, true);
 	if (toolbar->doAutoHide())
-		setItemSelected(2, True);
+		setItemSelected(2, true);
 }
 
 
 Toolbarmenu::~Toolbarmenu(void) {
 	delete placementmenu;
+#ifdef XINERAMA
+	if (toolbar->screen->hasXinerama()) {
+		delete headmenu;
+	}
+#endif // XINERAMA
+
 }
 
 
-void Toolbarmenu::itemSelected(int button, int index) {
+void Toolbarmenu::itemSelected(int button, unsigned int index) {
 	if (button == 1) {
 		BasemenuItem *item = find(index);
 		if (! item) return;
@@ -1283,7 +1310,7 @@ void Toolbarmenu::itemSelected(int button, int index) {
 		switch (item->function()) {
 		case 1: // always on top
 			{
-			Bool change = ((toolbar->isOnTop()) ? False : True);
+			bool change = ((toolbar->isOnTop()) ? False : True);
 			toolbar->on_top = change;
 				setItemSelected(1, change);
 
@@ -1293,7 +1320,7 @@ void Toolbarmenu::itemSelected(int button, int index) {
 
 		case 2: // auto hide
 			{
-	Bool change = ((toolbar->doAutoHide()) ?	False : True);
+	bool change = ((toolbar->doAutoHide()) ?	False : True);
 	toolbar->do_auto_hide = change;
 	setItemSelected(2, change);
 
@@ -1322,6 +1349,11 @@ void Toolbarmenu::internal_hide(void) {
 
 void Toolbarmenu::reconfigure(void) {
 	placementmenu->reconfigure();
+#ifdef XINERAMA
+	if (toolbar->screen->hasXinerama()) {
+		headmenu->reconfigure();
+	}
+#endif // XINERAMA
 
 	Basemenu::reconfigure();
 }
@@ -1331,71 +1363,43 @@ Toolbarmenu::Placementmenu::Placementmenu(Toolbarmenu *tm)
 	: Basemenu(tm->toolbar->screen) {
 	toolbarmenu = tm;
 	I18n *i18n = I18n::instance();
-	
+	using namespace FBNLS;
 	setLabel(i18n->getMessage(
-#ifdef		NLS
-					ToolbarSet, ToolbarToolbarPlacement,
-#else // !NLS
-					0, 0,
-#endif // NLS
-					"Toolbar Placement"));
+		ToolbarSet, ToolbarToolbarPlacement,
+		"Toolbar Placement"));
 	setInternalMenu();
 	setMinimumSublevels(3);
 
 	insert(i18n->getMessage(
-#ifdef		NLS
-				CommonSet, CommonPlacementTopLeft,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Top Left"),
-				Toolbar::TOPLEFT);
+		CommonSet, CommonPlacementTopLeft,
+		"Top Left"),
+		Toolbar::TOPLEFT);
 	insert(i18n->getMessage(
-#ifdef		NLS
-				CommonSet, CommonPlacementBottomLeft,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Bottom Left"),
-				Toolbar::BOTTOMLEFT);
+		CommonSet, CommonPlacementBottomLeft,
+		"Bottom Left"),
+		Toolbar::BOTTOMLEFT);
 	insert(i18n->getMessage(
-#ifdef		NLS
-				CommonSet, CommonPlacementTopCenter,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Top Center"),
-				Toolbar::TOPCENTER);
+		CommonSet, CommonPlacementTopCenter,
+		"Top Center"),
+		Toolbar::TOPCENTER);
 	insert(i18n->getMessage(
-#ifdef		NLS
-				CommonSet, CommonPlacementBottomCenter,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Bottom Center"),
-				Toolbar::BOTTOMCENTER);
+		CommonSet, CommonPlacementBottomCenter,
+		"Bottom Center"),
+		Toolbar::BOTTOMCENTER);
 	insert(i18n->getMessage(
-#ifdef		NLS
-				CommonSet, CommonPlacementTopRight,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Top Right"),
-				Toolbar::TOPRIGHT);
+		CommonSet, CommonPlacementTopRight,
+		"Top Right"),
+		Toolbar::TOPRIGHT);
 	insert(i18n->getMessage(
-#ifdef		NLS
-				CommonSet, CommonPlacementBottomRight,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Bottom Right"),
+		CommonSet, CommonPlacementBottomRight,
+		"Bottom Right"),
 	Toolbar::BOTTOMRIGHT);
 
 	update();
 }
 
 
-void Toolbarmenu::Placementmenu::itemSelected(int button, int index) {
+void Toolbarmenu::Placementmenu::itemSelected(int button, unsigned int index) {
 	if (button == 1) {
 		BasemenuItem *item = find(index);
 		if (! item)
@@ -1414,3 +1418,49 @@ void Toolbarmenu::Placementmenu::itemSelected(int button, int index) {
 
 	}
 }
+
+#ifdef XINERAMA
+
+Toolbarmenu::Headmenu::Headmenu(Toolbarmenu *tm)
+	: Basemenu(tm->toolbar->screen) {
+	toolbarmenu = tm;
+	I18n *i18n = I18n::instance();
+
+	setLabel(i18n->getMessage(0, 0, "Place on Head")); //TODO: NLS
+	setInternalMenu();
+
+	int numHeads = toolbarmenu->toolbar->screen->getNumHeads();
+	// fill menu with head entries
+	for (int i = 0; i < numHeads; i++) {
+		char headName[32];
+		sprintf(headName, "Head %i", i+1); //TODO: NLS
+		insert(i18n->getMessage(0, 0, headName), i);
+	}
+
+	insert(i18n->getMessage(0, 0, "All Heads"), -1); //TODO: NLS
+
+	update();
+}
+
+
+void Toolbarmenu::Headmenu::itemSelected(int button, unsigned int index) {
+	if (button == 1) {
+		BasemenuItem *item = find(index);
+		if (! item)
+			return;
+
+		toolbarmenu->toolbar->screen->saveToolbarOnHead(
+			static_cast<int>(item->function()));
+		hide();
+		toolbarmenu->toolbar->reconfigure();
+
+#ifdef SLIT
+		// reposition the slit as well to make sure it doesn't intersect the
+		// toolbar
+		toolbarmenu->toolbar->screen->getSlit()->reposition();
+#endif // SLIT
+
+	}
+}
+
+#endif // XINERAMA
