@@ -1,3 +1,5 @@
+// Windowmenu.cc for Fluxbox
+// Copyright (c) 2001-2002 Henrik Kinnunen (fluxgen@linuxmail.org)
 // Windowmenu.cc for Blackbox - an X11 Window manager
 // Copyright (c) 1997 - 2000 Brad Hughes (bhughes@tcac.net)
 //
@@ -19,6 +21,8 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
+// $Id: Windowmenu.cc,v 1.13 2002/04/14 22:26:43 fluxgen Exp $
+
 //use GNU extensions
 #ifndef	 _GNU_SOURCE
 #define	 _GNU_SOURCE
@@ -52,96 +56,52 @@ screen(window->getScreen()){
 	
 	sendToMenu = new SendtoWorkspacemenu(this);
 	sendGroupToMenu = new SendGroupToWorkspacemenu(this);
-	
+	using namespace FBNLS;	
 	insert(i18n->getMessage(
-#ifdef		NLS
-				WindowmenuSet, WindowmenuSendTo,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Send To ..."),
+		WindowmenuSet, WindowmenuSendTo,
+		"Send To ..."),
 	 sendToMenu);
 	 
 	insert(i18n->getMessage(
-#ifdef		NLS
-				WindowmenuSet, WindowmenuSendGroupTo,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Send Group To ..."),
+		WindowmenuSet, WindowmenuSendGroupTo,
+		"Send Group To ..."),
 	sendGroupToMenu);
 	 
 	insert(i18n->getMessage(
-#ifdef		NLS
-				WindowmenuSet, WindowmenuShade,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Shade"),
+		WindowmenuSet, WindowmenuShade,
+		"Shade"),
 	 BScreen::WINDOWSHADE);
 	insert(i18n->getMessage(
-#ifdef		NLS
-				WindowmenuSet, WindowmenuIconify,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Iconify"),
+		WindowmenuSet, WindowmenuIconify,
+		"Iconify"),
 	 BScreen::WINDOWICONIFY);
 	insert(i18n->getMessage(
-#ifdef		NLS
-				WindowmenuSet, WindowmenuMaximize,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Maximize"),
+		WindowmenuSet, WindowmenuMaximize,
+		"Maximize"),
 	 BScreen::WINDOWMAXIMIZE);
 	insert(i18n->getMessage(
-#ifdef		NLS
-				WindowmenuSet, WindowmenuRaise,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Raise"),
+		WindowmenuSet, WindowmenuRaise,
+		"Raise"),
 	 BScreen::WINDOWRAISE);
 	insert(i18n->getMessage(
-#ifdef		NLS
-				WindowmenuSet, WindowmenuLower,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Lower"),
+		WindowmenuSet, WindowmenuLower,
+		"Lower"),
 	 BScreen::WINDOWLOWER);
 	insert(i18n->getMessage(
-#ifdef		NLS
-				WindowmenuSet, WindowmenuStick,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Stick"),
+		WindowmenuSet, WindowmenuStick,
+		"Stick"),
 	 BScreen::WINDOWSTICK);
 	insert(i18n->getMessage(
-#ifdef		NLS
-				WindowmenuSet, WindowmenuKillClient,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Kill Client"),
+		WindowmenuSet, WindowmenuKillClient,
+		"Kill Client"),
 	 BScreen::WINDOWKILL);
 	insert(i18n->getMessage(
-#ifdef		NLS
-				WindowmenuSet, WindowmenuClose,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Close"),
+		WindowmenuSet, WindowmenuClose,
+		"Close"),
 	 BScreen::WINDOWCLOSE);
 	insert(i18n->getMessage(
-#ifdef		NLS
-				WindowmenuSet, WindowmenuTab,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Tab"),
+		WindowmenuSet, WindowmenuTab,
+		"Tab"),
 	 BScreen::WINDOWTAB);
 
 	update();
@@ -150,7 +110,7 @@ screen(window->getScreen()){
 	setItemEnabled(3, window->isIconifiable());
 	setItemEnabled(4, window->isMaximizable());
 	setItemEnabled(9, window->isClosable());
-	setItemEnabled(10, window->hasTab());
+	setItemEnabled(10, true); //we should always be able to enable the tab
 
 }
 
@@ -170,7 +130,7 @@ void Windowmenu::show(void) {
 }
 
 
-void Windowmenu::itemSelected(int button, int index) {
+void Windowmenu::itemSelected(int button, unsigned int index) {
 	BasemenuItem *item = find(index);
 
 	switch (item->function()) {
@@ -254,7 +214,7 @@ Windowmenu::SendtoWorkspacemenu::SendtoWorkspacemenu(Windowmenu *w)
 }
 
 
-void Windowmenu::SendtoWorkspacemenu::itemSelected(int button, int index) {
+void Windowmenu::SendtoWorkspacemenu::itemSelected(int button, unsigned int index) {
 	if (button > 2) return;
 
 	if (index <= windowmenu->screen->getCount()) {
@@ -272,14 +232,14 @@ void Windowmenu::SendtoWorkspacemenu::itemSelected(int button, int index) {
 
 
 void Windowmenu::SendtoWorkspacemenu::update(void) {
-	int i, r = getCount();
+	unsigned int i, r = numberOfItems();
 
-	if (getCount() != 0) {
+	if (numberOfItems() != 0) {
 		for (i = 0; i < r; ++i)
 			remove(0);
 	}
 	for (i = 0; i < windowmenu->screen->getCount(); ++i)
-		insert(windowmenu->screen->getWorkspace(i)->getName());
+		insert(windowmenu->screen->getWorkspace(i)->name().c_str());
 
 	Basemenu::update();
 }
@@ -291,7 +251,7 @@ void Windowmenu::SendtoWorkspacemenu::show(void) {
 	Basemenu::show();
 }
 
-void Windowmenu::SendGroupToWorkspacemenu::itemSelected(int button, int index) {
+void Windowmenu::SendGroupToWorkspacemenu::itemSelected(int button, unsigned int index) {
 	if (button > 2)
 		return;
 
