@@ -21,8 +21,9 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// stupid macros needed to access some functions in version 2 of the GNU C
-// library
+// $Id: Workspacemenu.cc,v 1.6 2002/04/09 23:19:17 fluxgen Exp $
+
+//use GNU extension
 #ifndef   _GNU_SOURCE
 #define   _GNU_SOURCE
 #endif // _GNU_SOURCE
@@ -45,39 +46,26 @@ Workspacemenu::Workspacemenu(BScreen *scrn) : Basemenu(scrn) {
 	
 	setInternalMenu();
 	I18n *i18n = I18n::instance();
-	
+	using namespace FBNLS;
 	setLabel(i18n->getMessage(
-#ifdef    NLS
-					WorkspacemenuSet, WorkspacemenuWorkspacesTitle,
-#else // !NLS
-					0, 0,
-#endif // NLS
-					"Workspaces"));
-
+		WorkspacemenuSet, WorkspacemenuWorkspacesTitle,
+		"Workspaces"));
 	insert(i18n->getMessage(
-#ifdef    NLS
-				WorkspacemenuSet, WorkspacemenuNewWorkspace,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"New Workspace"));
+		WorkspacemenuSet, WorkspacemenuNewWorkspace,
+		"New Workspace"));
 	insert(i18n->getMessage(
-#ifdef    NLS
-				WorkspacemenuSet, WorkspacemenuRemoveLast,
-#else // !NLS
-				0, 0,
-#endif // NLS
-				"Remove Last"));
+		WorkspacemenuSet, WorkspacemenuRemoveLast,
+		"Remove Last"));
 }
 
 
-void Workspacemenu::itemSelected(int button, int index) {
+void Workspacemenu::itemSelected(int button, unsigned int index) {
   if (button == 1) {
     if (index == 0)
       screen->addWorkspace();
     else if (index == 1)
       screen->removeLastWorkspace();
-    else if ((screen->getCurrentWorkspace()->getWorkspaceID() !=
+    else if ((screen->getCurrentWorkspace()->workspaceID() !=
 	      (index - 2)) && ((index - 2) < screen->getCount()))
       screen->changeWorkspaceID(index - 2);
 
