@@ -22,7 +22,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: fluxbox.cc,v 1.38 2002/03/01 15:28:56 fluxgen Exp $
+// $Id: fluxbox.cc,v 1.52 2002/04/19 09:33:42 fluxgen Exp $
 
 //Use some GNU extensions
 #ifndef	 _GNU_SOURCE
@@ -282,7 +282,7 @@ getString() {
 Fluxbox *Fluxbox::singleton=0;
 
 //------------ instance ---------------------
-//returns singleton object of blackbox class
+//returns singleton object of Fluxbox class
 //since we only need to create one instance of Fluxbox
 //-------------------------------------------
 Fluxbox *Fluxbox::instance(int m_argc, char **m_argv, char *dpy_name, char *rc) {
@@ -306,8 +306,8 @@ m_rc_titlebar_left(m_resourcemanager, TitlebarList(&m_titlebar_left[0], &m_title
 m_rc_titlebar_right(m_resourcemanager, TitlebarList(&m_titlebar_right[0], &m_titlebar_right[3]), "session.titlebar.right", "Session.Titlebar.Right"),
 m_rc_cache_life(m_resourcemanager, 5, "session.cacheLife", "Session.CacheLife"),
 m_rc_cache_max(m_resourcemanager, 200, "session.cacheMax", "Session.CacheMax"),
-focused_window(0),
-masked_window(0),
+focused_window(0), masked_window(0),
+timer(this),
 no_focus(false),
 rc_file(rc),
 argv(m_argv), argc(m_argc), 
@@ -333,11 +333,7 @@ key(0)
 	
 	masked = None;
 	
-	windowSearchList = new LinkedList<WindowSearch>;
-	menuSearchList = new LinkedList<MenuSearch>;
-
 	#ifdef SLIT
-	slitSearchList = new LinkedList<SlitSearch>;
 	#ifdef KDE
 	//For KDE dock applets
 	kwm1_dockwindow = XInternAtom(getXDisplay(), "KWM_DOCKWINDOW", False); //KDE v1.x
@@ -346,18 +342,10 @@ key(0)
 
 	#endif // SLIT
 
-	toolbarSearchList = new LinkedList<ToolbarSearch>;
-	tabSearchList = new LinkedList<TabSearch>;
-	groupSearchList = new LinkedList<WindowSearch>;
-
-	menuTimestamps = new LinkedList<MenuTimestamp>;
-
-
 	#ifdef HAVE_GETPID
 	fluxbox_pid = XInternAtom(getXDisplay(), "_BLACKBOX_PID", False);
 	#endif // HAVE_GETPID
 
-	screenList = new LinkedList<BScreen>;
 	int i;
 	load_rc();
 	//allocate screens
@@ -372,19 +360,15 @@ key(0)
 			continue;
 		}
 
-		screenList->insert(screen);
+		screenList.push_back(screen);
 	}
 	
 	I18n *i18n = I18n::instance();
-	if (! screenList->count()) {
+	if (screenList.size() == 0) {
 		fprintf(stderr,
 			i18n->
 				getMessage(
-#ifdef		NLS
-				blackboxSet, blackboxNoManagableScreens,
-#else // !NLS
-				 0, 0,
-#endif // NLS
+				FBNLS::blackboxSet, FBNLS::blackboxNoManagableScreens,
 				"Fluxbox::Fluxbox: no managable screens found, aborting.\n"));
 
 		throw static_cast<int>(3);
@@ -394,10 +378,9 @@ key(0)
 	XSync(getXDisplay(), False);
 
 	reconfigure_wait = reread_menu_wait = false;
-
-	timer = new BTimer(this, this);
-	timer->setTimeout(0);
-	timer->fireOnce(True);
+	
+	timer.setTimeout(0);
+	timer.fireOnce(True);
 
 	//create keybindings handler and load keys file
 	char *keyfilename = StringUtil::expandFilename((*m_rc_keyfile).c_str());
@@ -409,12 +392,10 @@ key(0)
 
 
 Fluxbox::~Fluxbox(void) {
-	
-	while (screenList->count())
-		delete screenList->remove(0);
-
-	while (menuTimestamps->count()) {
-		MenuTimestamp *ts = menuTimestamps->remove(0);
+	std::list<MenuTimestamp *>::iterator it = menuTimestamps.begin();
+	std::list<MenuTimestamp *>::iterator it_end = menuTimestamps.end();
+	for (; it != it_end; ++it) {
+		MenuTimestamp *ts = *it;
 
 		if (ts->filename)
 			delete [] ts->filename;
@@ -424,21 +405,6 @@ Fluxbox::~Fluxbox(void) {
 	
 	delete key;
 	key = 0;
-
-	delete timer;
-
-	delete screenList;
-	delete menuTimestamps;
-
-	delete windowSearchList;
-	delete menuSearchList;
-	delete toolbarSearchList;
-	delete tabSearchList;
-	delete groupSearchList;
-
-#ifdef		SLIT
-	delete slitSearchList;
-#endif // SLIT
 }
 
 //---------- setupConfigFiles -----------
@@ -605,13 +571,9 @@ void Fluxbox::process_event(XEvent *e) {
 		fprintf(stderr,
 			I18n::instance()->
 				getMessage(
-				#ifdef NLS
-				 blackboxSet, blackboxMapRequest,
-				#else // !NLS
-				 0, 0,
-				#endif // NLS
+				FBNLS::blackboxSet, FBNLS::blackboxMapRequest,
 				 "Fluxbox::process_event(): MapRequest for 0x%lx\n"),
-							e->xmaprequest.window);
+					e->xmaprequest.window);
 		#endif // DEBUG
 		
 		#ifdef SLIT
@@ -648,10 +610,9 @@ void Fluxbox::process_event(XEvent *e) {
 
 		if (iskdedockapp) {
 			XSelectInput(getXDisplay(), e->xmaprequest.window, StructureNotifyMask);
-			LinkedListIterator<BScreen> it(screenList);
-			for (; it.current() == screenList->last(); it++) {
-				BScreen *screen = it.current();
-				screen->getSlit()->addClient(e->xmaprequest.window);
+			std::list<BScreen *>::iterator it = screenList.begin();			
+			for (; (*it) == screenList.back(); ++it) {
+				(*it)->getSlit()->addClient(e->xmaprequest.window);
 			}
 			return;
 		}
@@ -937,11 +898,12 @@ void Fluxbox::handleButtonEvent(XButtonEvent &be) {
 		} else if ((tab = searchTab(be.window))) {
 			tab->buttonPressEvent(&be);
 		} else {
-			LinkedListIterator<BScreen> it(screenList);
+			std::list<BScreen *>::iterator it = screenList.begin();
+			std::list<BScreen *>::iterator it_end = screenList.end();
 
-			for (; it.current(); it++) {
+			for (; it != it_end; ++it) {
 
-				BScreen *screen = it.current();
+				BScreen *screen = *it;
 				if (be.window != screen->getRootWindow())
 					continue;
 				
@@ -957,24 +919,24 @@ void Fluxbox::handleButtonEvent(XButtonEvent &be) {
 						
 				} else if (be.button == 2) {
 					int mx = be.x_root -
-						(screen->getWorkspacemenu()->getWidth() / 2);
+						(screen->getWorkspacemenu()->width() / 2);
 					int my = be.y_root -
-						(screen->getWorkspacemenu()->getTitleHeight() / 2);
+						(screen->getWorkspacemenu()->titleHeight() / 2);
 	
 					if (mx < 0) mx = 0;
 					if (my < 0) my = 0;
 
-					if (mx + screen->getWorkspacemenu()->getWidth() >
+					if (mx + screen->getWorkspacemenu()->width() >
 						screen->getWidth()) {
 						mx = screen->getWidth() -
-							screen->getWorkspacemenu()->getWidth() -						
+							screen->getWorkspacemenu()->width() -						
 							screen->getBorderWidth();
 					}
 
-					if (my + screen->getWorkspacemenu()->getHeight() >
+					if (my + screen->getWorkspacemenu()->height() >
 							screen->getHeight()) {
 						my = screen->getHeight() -
-							screen->getWorkspacemenu()->getHeight() -
+							screen->getWorkspacemenu()->height() -
 							screen->getBorderWidth();
 					}
 					screen->getWorkspacemenu()->move(mx, my);
@@ -987,23 +949,23 @@ void Fluxbox::handleButtonEvent(XButtonEvent &be) {
 				//calculate placement of workspace menu
 				//and show/hide it				
 					int mx = be.x_root -
-						(screen->getRootmenu()->getWidth() / 2);
+						(screen->getRootmenu()->width() / 2);
 					int my = be.y_root -
-						(screen->getRootmenu()->getTitleHeight() / 2);
+						(screen->getRootmenu()->titleHeight() / 2);
 
 					if (mx < 0) mx = 0;
 					if (my < 0) my = 0;
 
-					if (mx + screen->getRootmenu()->getWidth() > screen->getWidth()) {
+					if (mx + screen->getRootmenu()->width() > screen->getWidth()) {
 						mx = screen->getWidth() -
-							screen->getRootmenu()->getWidth() -
+							screen->getRootmenu()->width() -
 							screen->getBorderWidth();
 					}
 
-					if (my + screen->getRootmenu()->getHeight() >
+					if (my + screen->getRootmenu()->height() >
 							screen->getHeight()) {
 						my = screen->getHeight() -
-							screen->getRootmenu()->getHeight() -
+							screen->getRootmenu()->height() -
 							screen->getBorderWidth();
 					}
 					screen->getRootmenu()->move(mx, my);
@@ -1012,7 +974,11 @@ void Fluxbox::handleButtonEvent(XButtonEvent &be) {
 						checkMenu();
 						screen->getRootmenu()->show();
 					}
-				} // end button == 3
+				} else if (be.button == 4) {
+					screen->nextWorkspace(1);
+				} else if (be.button == 5) {
+					screen->prevWorkspace(1);
+				}
 			}
 		}
 	}
@@ -1097,7 +1063,7 @@ void Fluxbox::handleClientMessage(XClientMessageEvent &ce) {
 		BScreen *screen = searchScreen(ce.window);
 
 		if (screen && ce.data.l[0] >= 0 &&
-				ce.data.l[0] < screen->getCount())
+				ce.data.l[0] < (signed)screen->getCount())
 			screen->changeWorkspaceID(ce.data.l[0]);
 				
 	} else if (ce.message_type == getFluxboxChangeWindowFocusAtom()) {
@@ -1123,7 +1089,7 @@ void Fluxbox::handleClientMessage(XClientMessageEvent &ce) {
 			net.attrib = ce.data.l[1];
 			net.workspace = ce.data.l[2];
 			net.stack = ce.data.l[3];
-			net.decoration = static_cast<BaseDisplay::Decor>(ce.data.l[4]);
+			net.decoration = static_cast<int>(ce.data.l[4]);
 			win->changeBlackboxHints(&net);
 		}
 	} else {
@@ -1227,10 +1193,10 @@ void Fluxbox::handleKeyEvent(XKeyEvent &ke) {
 					focused_window->getClientWindow());
 			break;
 			case Keys::NEXTWINDOW:	//activate next window
-				screen->nextFocus();
+				screen->nextFocus(key->getParam());
 			break;
 			case Keys::PREVWINDOW:	//activate prev window
-				screen->prevFocus();
+				screen->prevFocus(key->getParam());
 			break;
 			case Keys::NEXTTAB: 
 				if (focused_window && focused_window->getTab()) {
@@ -1278,6 +1244,48 @@ void Fluxbox::handleKeyEvent(XKeyEvent &ke) {
 				#endif // !__EMX__
 						
 			
+			}
+			break;
+			case Keys::ROOTMENU: //show root menu
+			{
+                std::list<BScreen *>::iterator it = screenList.begin();
+                std::list<BScreen *>::iterator it_end = screenList.end();
+
+				for (; it != it_end; ++it) {
+
+					BScreen *screen = (*it);
+					if (ke.window != screen->getRootWindow())
+						continue;
+						
+					//calculate placement of workspace menu
+					//and show/hide it				
+					int mx = ke.x_root -
+						(screen->getRootmenu()->width() / 2);
+					int my = ke.y_root -
+						(screen->getRootmenu()->titleHeight() / 2);
+
+					if (mx < 0) mx = 0;
+					if (my < 0) my = 0;
+
+					if (mx + screen->getRootmenu()->width() > screen->getWidth()) {
+						mx = screen->getWidth() -
+							screen->getRootmenu()->width() -
+							screen->getBorderWidth();
+					}
+
+					if (my + screen->getRootmenu()->height() >
+							screen->getHeight()) {
+						my = screen->getHeight() -
+							screen->getRootmenu()->height() -
+							screen->getBorderWidth();
+					}
+					screen->getRootmenu()->move(mx, my);
+
+					if (! screen->getRootmenu()->isVisible()) {
+						checkMenu();
+						screen->getRootmenu()->show();
+					}
+				}
 			}
 			break;
 			default: //try to see if its a window action
@@ -1433,8 +1441,14 @@ void Fluxbox::doWindowAction(Keys::KeyAction action, const int param) {
 					if (t_alignment != Tab::PRIGHT)
 						focused_window->getTab()->setPosition();
 				}
-		break;		
-		default:	
+		break;
+		case Keys::TOGGLEDECOR:
+			focused_window->toggleDecoration();
+		break;
+		case Keys::TOGGLETAB:
+			focused_window->setTab(!focused_window->hasTab());
+		break;
+		default: //do nothing
 		break;							
 	}
 
@@ -1456,12 +1470,12 @@ bool Fluxbox::checkGnomeAtoms(XClientMessageEvent &ce) {
 		#endif//!DEBUG
 		if ( win !=0 && // the message sent to client window?
 				win->getScreen() && ce.data.l[0] >= 0 &&
-				ce.data.l[0] < win->getScreen()->getCount()) {
+				ce.data.l[0] < (signed)win->getScreen()->getCount()) {
 			win->getScreen()->changeWorkspaceID(ce.data.l[0]);
 					
 		} else if (screen!=0 && //the message sent to root window?
 				ce.data.l[0] >= 0 &&
-				ce.data.l[0] < screen->getCount())
+				ce.data.l[0] < (signed)screen->getCount())
 			screen->changeWorkspaceID(ce.data.l[0]);
 		return true;
 	} else if (win) {
@@ -1479,7 +1493,7 @@ bool Fluxbox::checkGnomeAtoms(XClientMessageEvent &ce) {
 	
 			//get new states			
 			int flag = ce.data.l[0] & ce.data.l[1];
-			//set states	
+			//set states			
 			win->setGnomeState(flag);
 			
 		} else if (ce.message_type == getGnomeHintsAtom()) {
@@ -1543,12 +1557,13 @@ Bool Fluxbox::handleSignal(int sig) {
 
 BScreen *Fluxbox::searchScreen(Window window) {
 	BScreen *screen = (BScreen *) 0;
-	LinkedListIterator<BScreen> it(screenList);
+	std::list<BScreen *>::iterator it = screenList.begin();
+	std::list<BScreen *>::iterator it_end = screenList.end();
 
-	for (; it.current(); it++) {
-		if (it.current()) {
-			if (it.current()->getRootWindow() == window) {
-				screen = it.current();
+	for (; it != it_end; ++it) {
+		if (*it) {
+			if ((*it)->getRootWindow() == window) {
+				screen = (*it);
 				return screen;
 			}
 		}
@@ -1559,228 +1574,100 @@ BScreen *Fluxbox::searchScreen(Window window) {
 
 
 FluxboxWindow *Fluxbox::searchWindow(Window window) {
-	LinkedListIterator<WindowSearch> it(windowSearchList);
-
-	for (; it.current(); it++) {
-		WindowSearch *tmp = it.current();
-		if (tmp && tmp->getWindow() == window)
-			return tmp->getData(); 			
-	}
-
-	return (FluxboxWindow *) 0;
+	std::map<Window, FluxboxWindow *>::iterator it = windowSearch.find(window);
+	return it == windowSearch.end() ? 0 : it->second;
 }
 
 
 FluxboxWindow *Fluxbox::searchGroup(Window window, FluxboxWindow *win) {
-	FluxboxWindow *w = (FluxboxWindow *) 0;
-	LinkedListIterator<WindowSearch> it(groupSearchList);
-
-	for (; it.current(); it++) {
-		WindowSearch *tmp = it.current();
-		if (tmp) {
-			if (tmp->getWindow() == window) {
-				w = tmp->getData();
-				if (w->getClientWindow() != win->getClientWindow())
-					return win;
-			}
-		}
-	}
-
-	return (FluxboxWindow *) 0;
+	std::map<Window, FluxboxWindow *>::iterator it = groupSearch.find(window);
+	return it == groupSearch.end() ? 0 : it->second;
 }
 
 
 Basemenu *Fluxbox::searchMenu(Window window) {
-	Basemenu *menu = (Basemenu *) 0;
-	LinkedListIterator<MenuSearch> it(menuSearchList);
-
-	for (; it.current(); it++) {
-		MenuSearch *tmp = it.current();
-
-		if (tmp) {
-			if (tmp->getWindow() == window) {
-				menu = tmp->getData();
-				return menu;
-			}
-		}
-	}
-
-	return (Basemenu *) 0;
+	std::map<Window, Basemenu *>::iterator it = menuSearch.find(window);
+	return it == menuSearch.end() ? 0 : it->second;
 }
 
 
 Toolbar *Fluxbox::searchToolbar(Window window) {
-	Toolbar *tbar = (Toolbar *) 0;
-	LinkedListIterator<ToolbarSearch> it(toolbarSearchList);
-
-	for (; it.current(); it++) {
-		ToolbarSearch *tmp = it.current();
-
-		if (tmp) {
-			if (tmp->getWindow() == window) {
-				tbar = tmp->getData();
-				return tbar;
-			}
-		}
-	}
-
-	return (Toolbar *) 0;
+	std::map<Window, Toolbar *>::iterator it = toolbarSearch.find(window);
+	return it == toolbarSearch.end() ? 0 : it->second;
 }
 
 Tab *Fluxbox::searchTab(Window window) {
-	LinkedListIterator<TabSearch> it(tabSearchList);
-
-	for (; it.current(); it++) {
-		TabSearch *tmp = it.current();
-		if (tmp && tmp->getWindow() == window)
-			return tmp->getData();			
-	}
-
-	return 0;
+	std::map<Window, Tab *>::iterator it = tabSearch.find(window);
+	return it == tabSearch.end() ? 0 : it->second;
 }
 
 
 #ifdef		SLIT
 Slit *Fluxbox::searchSlit(Window window) {
-	Slit *s = (Slit *) 0;
-	LinkedListIterator<SlitSearch> it(slitSearchList);
-
-	for (; it.current(); it++) {
-		SlitSearch *tmp = it.current();
-
-		if (tmp) {
-			if (tmp->getWindow() == window) {
-				s = tmp->getData();
-				return s;
-			}
-		}
-	}
-
-	return (Slit *) 0;
+	std::map<Window, Slit *>::iterator it = slitSearch.find(window);
+	return it == slitSearch.end() ? 0 : it->second;
 }
 #endif // SLIT
 
 
 void Fluxbox::saveWindowSearch(Window window, FluxboxWindow *data) {
-	windowSearchList->insert(new WindowSearch(window, data));
+	windowSearch[window] = data;
 }
 
 
 void Fluxbox::saveGroupSearch(Window window, FluxboxWindow *data) {
-	groupSearchList->insert(new WindowSearch(window, data));
+	groupSearch[window] = data;
 }
 
 
 void Fluxbox::saveMenuSearch(Window window, Basemenu *data) {
-	menuSearchList->insert(new MenuSearch(window, data));
+	menuSearch[window] = data;
 }
 
 
 void Fluxbox::saveToolbarSearch(Window window, Toolbar *data) {
-	toolbarSearchList->insert(new ToolbarSearch(window, data));
+	toolbarSearch[window] = data;
 }
 
 
 void Fluxbox::saveTabSearch(Window window, Tab *data) {
-	tabSearchList->insert(new TabSearch(window, data));
+	tabSearch[window] = data;
 }
 
 #ifdef		SLIT
 void Fluxbox::saveSlitSearch(Window window, Slit *data) {
-	slitSearchList->insert(new SlitSearch(window, data));
+	slitSearch[window] = data;
 }
 #endif // SLIT
 
 
 void Fluxbox::removeWindowSearch(Window window) {
-	LinkedListIterator<WindowSearch> it(windowSearchList);
-	for (; it.current(); it++) {
-		WindowSearch *tmp = it.current();
-
-		if (tmp) {
-			if (tmp->getWindow() == window) {
-				windowSearchList->remove(tmp);
-				delete tmp;
-				break;
-			}
-		}
-	}
+	windowSearch.erase(window);
 }
 
 
 void Fluxbox::removeGroupSearch(Window window) {
-	LinkedListIterator<WindowSearch> it(groupSearchList);
-	for (; it.current(); it++) {
-		WindowSearch *tmp = it.current();
-
-		if (tmp) {
-			if (tmp->getWindow() == window) {
-				groupSearchList->remove(tmp);
-				delete tmp;
-				break;
-			}
-		}
-	}
+	groupSearch.erase(window);
 }
 
 
 void Fluxbox::removeMenuSearch(Window window) {
-	LinkedListIterator<MenuSearch> it(menuSearchList);
-	for (; it.current(); it++) {
-		MenuSearch *tmp = it.current();
-
-		if (tmp) {
-			if (tmp->getWindow() == window) {
-				menuSearchList->remove(tmp);
-				delete tmp;
-				break;
-			}
-		}
-	}
+	menuSearch.erase(window);
 }
 
 
 void Fluxbox::removeToolbarSearch(Window window) {
-	LinkedListIterator<ToolbarSearch> it(toolbarSearchList);
-	for (; it.current(); it++) {
-		ToolbarSearch *tmp = it.current();
-		if (tmp) {
-			if (tmp->getWindow() == window) {
-				toolbarSearchList->remove(tmp);	
-				delete tmp;
-				break;
-			}
-		}
-	}
+	toolbarSearch.erase(window);
 }
 
 
 void Fluxbox::removeTabSearch(Window window) {
-	LinkedListIterator<TabSearch> it(tabSearchList);
-	for (; it.current(); it++) {
-		TabSearch *tmp = it.current();
-		if (tmp && tmp->getWindow() == window) {
-			tabSearchList->remove(tmp);	
-			delete tmp;
-			break;
-		}
-	}
+	tabSearch.erase(window);
 }
 
 #ifdef		SLIT
 void Fluxbox::removeSlitSearch(Window window) {
-	LinkedListIterator<SlitSearch> it(slitSearchList);
-	for (; it.current(); it++) {
-		SlitSearch *tmp = it.current();
-
-		if (tmp) {
-			if (tmp->getWindow() == window) {
-				slitSearchList->remove(tmp);
-				delete tmp;
-				break;
-			}
-		}
-	}
+	slitSearch.erase(window);
 }
 #endif // SLIT
 
@@ -1804,10 +1691,13 @@ void Fluxbox::shutdown(void) {
 
 	XSetInputFocus(getXDisplay(), PointerRoot, None, CurrentTime);
 
-	LinkedListIterator<BScreen> it(screenList);
-	for (; it.current(); it++)
-		if(it.current())
-			it.current()->shutdown();
+	std::list<BScreen *>::iterator it = screenList.begin();
+	std::list<BScreen *>::iterator it_end = screenList.end();
+	for (; it != it_end; ++it) {
+		if(*it) {
+			(*it)->shutdown();
+		}
+	}
 
 	XSync(getXDisplay(), False);
 
@@ -1846,12 +1736,13 @@ void Fluxbox::save_rc(void) {
 					(resource.auto_raise_delay.tv_usec / 1000)));
 	XrmPutLineResource(&new_blackboxrc, rc_string);
 
-	LinkedListIterator<BScreen> it(screenList);
+	std::list<BScreen *>::iterator it = screenList.begin();
+	std::list<BScreen *>::iterator it_end = screenList.end();
 
 	//Save screen resources
 
-	for (; it.current(); it++) {
-		BScreen *screen = it.current();
+	for (; it != it_end; ++it) {
+		BScreen *screen = *it;
 		int screen_number = screen->getScreenNumber();
 
 		#ifdef SLIT
@@ -1884,7 +1775,13 @@ void Fluxbox::save_rc(void) {
 		sprintf(rc_string, "session.screen%d.slit.autoHide: %s", screen_number,
 			((screen->getSlit()->doAutoHide()) ? "True" : "False"));
 		XrmPutLineResource(&new_blackboxrc, rc_string);
-		
+
+		#ifdef XINERAMA
+		sprintf(rc_string, "session.screen%d.slit.onHead: %d", screen_number,
+			screen->getSlitOnHead());
+		XrmPutLineResource(&new_blackboxrc, rc_string);
+		#endif // XINERAMA
+
 		#endif // SLIT
 
 		sprintf(rc_string, "session.screen%d.rowPlacementDirection: %s", screen_number,
@@ -1959,9 +1856,9 @@ void Fluxbox::save_rc(void) {
 		sprintf(rc_string, "session.screen%d.workspaceNames: ", screen_number);
 		string workspaces_string(rc_string);
 
-		for (int workspace=0; workspace < screen->getCount(); workspace++) {
-			if (screen->getWorkspace(workspace)->getName()!=0)
-				workspaces_string.append(screen->getWorkspace(workspace)->getName());
+		for (unsigned int workspace=0; workspace < screen->getCount(); workspace++) {
+			if (screen->getWorkspace(workspace)->name().size()!=0)
+				workspaces_string.append(screen->getWorkspace(workspace)->name());
 			else
 				workspaces_string.append("Null");
 			workspaces_string.append(",");
@@ -2246,6 +2143,20 @@ void Fluxbox::load_rc(BScreen *screen) {
 			screen->saveSlitAutoHide(False);
 	else
 		screen->saveSlitAutoHide(False);
+
+	#ifdef XINERAMA
+	int tmp_head;
+	sprintf(name_lookup, "session.screen%d.slit.onHead", screen_number);
+	sprintf(class_lookup, "Session.Screen%d.Slit.OnHead", screen_number);
+	if (XrmGetResource(*database, name_lookup, class_lookup, &value_type,
+			&value)) {
+		if (sscanf(value.addr, "%d", &tmp_head) != 1)
+			tmp_head = 0;
+	} else
+		tmp_head = 0;
+	screen->saveSlitOnHead(tmp_head);
+	#endif // XINERAMA
+
 #endif // SLIT
 
 #ifdef HAVE_STRFTIME
@@ -2327,7 +2238,7 @@ void Fluxbox::reload_rc(void) {
 void Fluxbox::reconfigure(void) {
 	reconfigure_wait = true;
 
-	if (! timer->isTiming()) timer->start();
+	if (! timer.isTiming()) timer.start();
 }
 
 
@@ -2346,8 +2257,10 @@ void Fluxbox::real_reconfigure(void) {
 	if (old_blackboxrc)
 		XrmDestroyDatabase(old_blackboxrc);
 
-	for (int i = 0, n = menuTimestamps->count(); i < n; i++) {
-		MenuTimestamp *ts = menuTimestamps->remove(0);
+	std::list<MenuTimestamp *>::iterator it = menuTimestamps.begin();
+	std::list<MenuTimestamp *>::iterator it_end = menuTimestamps.end();
+	for (; it != it_end; ++it) {
+		MenuTimestamp *ts = *it;
 
 		if (ts) {
 			if (ts->filename)
@@ -2356,12 +2269,12 @@ void Fluxbox::real_reconfigure(void) {
 			delete ts;
 		}
 	}
+	menuTimestamps.erase(menuTimestamps.begin(), menuTimestamps.end());
 
-	LinkedListIterator<BScreen> it(screenList);
-	for (; it.current(); it++) {
-		BScreen *screen = it.current();
-
-		screen->reconfigure();
+	std::list<BScreen *>::iterator sit = screenList.begin();
+	std::list<BScreen *>::iterator sit_end = screenList.end();
+	for (; sit != sit_end; ++sit) {
+		(*sit)->reconfigure();
 	}
 	
 	//reconfigure keys
@@ -2380,18 +2293,16 @@ void Fluxbox::real_reconfigure(void) {
 // ---------------------------------------
 void Fluxbox::reconfigureTabs(void) {
 	//tab reconfiguring
-	LinkedListIterator<TabSearch> it(tabSearchList);
+	std::map<Window, Tab *>::iterator it = tabSearch.begin();
+	std::map<Window, Tab *>::iterator it_end = tabSearch.end();
 	//setting all to unconfigured
-	for (; it.current(); it++) {
-		TabSearch *tmp = it.current();
-		if (tmp)
-			tmp->getData()->setConfigured(false);
+	for (; it != it_end; ++it) {
+		it->second->setConfigured(false);
 	}
-	it.reset(); // resetting list and start configure tabs
+	it = tabSearch.begin(); // resetting list and start configure tabs
 	//reconfiguring
-	for (; it.current(); it++) {
-		TabSearch *tmp = it.current();
-		Tab *tab = tmp->getData();
+	for (; it != it_end; ++it) {
+		Tab *tab = it->second;
 		if (!tab->configured()) {
 			tab->setConfigured(true);
 			tab->resizeGroup(); 
@@ -2403,12 +2314,13 @@ void Fluxbox::reconfigureTabs(void) {
 
 void Fluxbox::checkMenu(void) {
 	Bool reread = False;
-	LinkedListIterator<MenuTimestamp> it(menuTimestamps);
-	for (; it.current() && (! reread); it++) {
+	std::list<MenuTimestamp *>::iterator it = menuTimestamps.begin();
+	std::list<MenuTimestamp *>::iterator it_end = menuTimestamps.end();
+	for (; it != it_end && (! reread); ++it) {
 		struct stat buf;
 
-		if (! stat(it.current()->filename, &buf)) {
-			if (it.current()->timestamp != buf.st_ctime)
+		if (! stat((*it)->filename, &buf)) {
+			if ((*it)->timestamp != buf.st_ctime)
 				reread = True;
 		} else
 			reread = True;
@@ -2421,33 +2333,40 @@ void Fluxbox::checkMenu(void) {
 void Fluxbox::rereadMenu(void) {
 	reread_menu_wait = True;
 
-	if (! timer->isTiming()) timer->start();
+	if (! timer.isTiming()) timer.start();
 }
 
 
 void Fluxbox::real_rereadMenu(void) {
-	for (int i = 0, n = menuTimestamps->count(); i < n; i++) {
-		MenuTimestamp *ts = menuTimestamps->remove(0);
+	std::list<MenuTimestamp *>::iterator it = menuTimestamps.begin();
+	std::list<MenuTimestamp *>::iterator it_end = menuTimestamps.end();
+	for (; it != it_end; ++it) {
+		MenuTimestamp *ts = *it;
 
 		if (ts) {
 			if (ts->filename)
-	delete [] ts->filename;
+				delete [] ts->filename;
 
 			delete ts;
 		}
 	}
+	menuTimestamps.erase(menuTimestamps.begin(), menuTimestamps.end());
 
-	LinkedListIterator<BScreen> it(screenList);
-	for (; it.current(); it++)
-		it.current()->rereadMenu();
+	ScreenList::iterator sit = screenList.begin();
+	ScreenList::iterator sit_end = screenList.end();
+	for (; sit != sit_end; ++sit) {
+		(*sit)->rereadMenu();
+	}
 }
 
 void Fluxbox::saveMenuFilename(const char *filename) {
 	Bool found = False;
 
-	LinkedListIterator<MenuTimestamp> it(menuTimestamps);
-	for (; it.current() && (! found); it++)
-		if (! strcmp(it.current()->filename, filename)) found = True;
+	std::list<MenuTimestamp *>::iterator it = menuTimestamps.begin();
+	std::list<MenuTimestamp *>::iterator it_end = menuTimestamps.end();
+	for (; it != it_end; ++it) {
+		if (! strcmp((*it)->filename, filename)) found = True;
+	}
 
 	if (! found) {
 		struct stat buf;
@@ -2458,7 +2377,7 @@ void Fluxbox::saveMenuFilename(const char *filename) {
 			ts->filename = StringUtil::strdup(filename);
 			ts->timestamp = buf.st_ctime;
 
-			menuTimestamps->insert(ts);
+			menuTimestamps.push_back(ts);
 		}
 	}
 }
@@ -2490,7 +2409,7 @@ void Fluxbox::setFocusedWindow(FluxboxWindow *win) {
 		old_wkspc = old_screen->getWorkspace(old_win->getWorkspaceNumber());
 
 		old_win->setFocusFlag(False);
-		old_wkspc->getMenu()->setItemSelected(old_win->getWindowNumber(), False);
+		old_wkspc->menu()->setItemSelected(old_win->getWindowNumber(), false);
 		
 	}
 
@@ -2501,7 +2420,7 @@ void Fluxbox::setFocusedWindow(FluxboxWindow *win) {
 		wkspc = screen->getWorkspace(win->getWorkspaceNumber());		
 		focused_window = win;
 		win->setFocusFlag(True);
-		wkspc->getMenu()->setItemSelected(win->getWindowNumber(), True);
+		wkspc->menu()->setItemSelected(win->getWindowNumber(), true);
 		
 	} else
 		focused_window = (FluxboxWindow *) 0;
